/***************************************************************
  TORNADO-P6x utilities (C6x DSP Environment). v.4B
  (C) MicroLAB Systems Ltd, 1998-2010

  http://www.mlabsys.com
  ftp://ftp.mlabsys.com
  email: support@mlabsys.com

  covers:
  -------
  	TP62/TP67 rev.1A..2B
	TP6202/TP6203 rev.1A..1C
	TP64xx h/w rev.1A with f/w rev.1A, rev.1A-1, rev.1A-2
	TP64xx h/w rev.1B with f/w rev.1B

  Description:
  ------------
	This file contains definitions for TORNADO-P6x DSP environments and
	must be included in user code.


  Revision history:
  -----------------
	rev.1A	- 1998, initial release for TP62/TP67

	rev.1B	- 2000, FLASH support added, structurized utilities

	rev.2A 	- 2000, added support for TP6202/T6203

	rev.3A 	- 2002, added support for TP64xx (TP6414/TP6415/TP6416),
					added new compiler keys,
					significant API structurization
	rev.3B 	- 2003, EMIF settings updated, fixed minor bugs

	rev.3C 	- Feb-2004
					1. Added support for TP64xx boards rev.1A with f/w rev.1A-1 and rev.1A-2
			           (IDs for SBSRAM/SDRAM/SDPRAM capacity, ID fr DSP clock frequency).
				  	2. __TP6X_DPRAM256K__ compiler def has been removed, since it worked with
					   TP64xx f/w rev.1A boards only, which have been fabbed with 128Kx32 DPRAM
					   only by the date of release of TP6x API rev.3C.
				  	2. TP6X_init_dsp_emif() function has been modified to properly
				  	   configure the DSP EMIF upon the DSP EMIF clock and SDRAM/DPRAM capacity.
				  	   __TP6X_SDRAM128M__ compiler option is not required with TP64xx boards with
					   f/w rev.1A-1 and rev.1A-2.
					3. EMIF register settings for TP64xx EMIF-B@CE1 FLASH area has been modified to
					   allocate FLASH memory as 8-bit ASYNC area. FLASH programming utilities have
					   been modified.

	rev.4A 	- June-2004
					1. Added support for TP64xx boards h/w rev.1B with f/w rev.1B
			           (XMEM2_LEN_ID register with ID for FLASH capacity, FLASH page selector).
					2. AMD FLASH programming utilities have been modified to support FLASH memory capacity up to 8Mx8
					   for TP64xx rev.1B boards.
					3. Several known bugs have been fixed and minor improvements have been done.

	rev.4B	- 2010, updated DSP EMIF registers settings for Micron SDRAM chips 133MHz '-75' speed grade.


  Notes:
  ------
	1. 	This C-header file is include file for TI C6x compile tools. C6X_DEF.H
		header file for general C6x DSP definitions must also be available for
		inclusion if required.
	2.	This header file is externally controlled from user C-code by run-time
		compiler keys definitions in order to select correct DSP controller type
		and to include/exclude extended TP6x API functions for DSP EMIF INIT and
		FLASH programming control.
			__TP6X_SELECT_TP64XX__ or __C6X_SELECT_C64XX_DSP__
				- if defined in user code prior inclusion of this header file,
				  then TP64xx DSP system definitions are enabled, otherwise
				  TP62/TP67/TP6202/TP6203 DSP system defs are enabled.
				  Correspondingly, __C6X_SELECT_C64XX_DSP__ definition is
				  defined for correct processing of C6X_DEF.H general C6xx DSP
				  header file. Default is TP62/TP67/TP6202/TP6203 DSP system defs.

			__TP6X_EXCLUDE_INIT_DSP_EMIF__
				- if defined in user code prior inclusion of this header file,
				  then the DSP EMIF INIT function are excluded from the user
				  source code. In case user application comprises of several .C linked
				  together modules, then this run-time compiler key must be included in
				  all modules except for the one module in order to exclude multiple
				  function body appearance. Default is INCLUDED.

			__TP6X_SDRAM128M__ (TP64xx f/w rev.1A only)
				- if defined in user code prior inclusion of this header file,
				  then 128Mbyte (16Mx64) RAM is installed. Otherwise, 32Mbyte (4Mx64) is
				  assumed to be installed. Default is 32Mbyte. This compiler key is not
				  required for TP64xx f/w rev.1A-1 and rev.1A-2 boards, which provide the SDRAM
				  length ID via the XMEM_LEN_ID register.


			__TP6X_DPRAM64K__ (TP62/TP620x/TP67 only)
				- if defined in user code prior inclusion of this
				  header file, then 64Kx32 DPRAM is installed. Otherwise, 32Kx32 DPRAM is
				  assumed to be installed. This compiler key is used to define address of
				  DPRAM locations, which are used to generate Host->DSP and DSP->Host
				  interrupts. Default for TP62/TP620x/TP67 is 32Kx32 DPRAM.

			__TP6X_INCLUDE_AMD_FLASH_CONTROL__
				- if defined in user code prior inclusion of this header file,
				  then the AMD FLASH PROGRAMMING CONTROL function are included
				  into the user source code. In case your application comprises of several
				  .C modules, which are linked together, then this run-time compiler
				  key must be included into only one module in order to exclude
				  multiple function body appearance. Default is EXCLUDED.

			__TP6X_API_V2_COMPATIBILITY__
				- if defined in user code prior inclusion of this header file,
				  then API v.2A and earlier confusing macros will be provided for
				  100% functional compatibility with API v.2A utilities, otherwise
				  these confusing macros will be provided in accordance with new API v.3A
				  synopsis. Default is NO COMPATIBILITY with API v.2A utilities.
				  Confusing macros include (refer MACROS section below for more details):
						TP6X_set_amwen(v)
						TP6X_set_amren(v)
						TP6X_amwen()
						TP6X_amren()
						TP6X_amen()


  Copyright:
  ----------
	This utility is supplied free of charge as it is without any obligation
	from MicroLAB Systems. No responsibility is assumed	for any use or misuse
	of these utilities.

***********************************************************************/

// check for this header file has been already included
#ifndef __TP6X_DEF__
#define __TP6X_DEF__ 	1


// general definitions
#define	ON				1
#define	OFF				0
#define	TP6X_ON			1
#define	TP6X_OFF		0


//===== DSP-type specific run-time compiler keys processing ==================
// (this is required in order to exclude key confusions)

//	Note:!!
//		If __TP6X_SELECT_TP64XX__ or __C6X_SELECT_C64XX_DSP__ compiler keys are 
//		not defined in user code, then TP62/TP620x/TP67 definitions are 
//		assumed to be used.

#if (defined(__TP6X_SELECT_TP64XX__) || defined(__C6X_SELECT_C64XX_DSP__))
	// this is TP64xx definition
	#undef	__TP6X_SELECT_TP62_TP67_TP620X__
	#undef	__TP6X_SELECT_TP64XX__
	#define	__TP6X_SELECT_TP64XX__				1
	#undef	__C6X_SELECT_C64XX_DSP__
	#define	__C6X_SELECT_C64XX_DSP__			1	// use C64xx defs in C6X_DEF.H file
	#undef	__TP6X_DPRAM64K__
#else
	// this is default TP62/TP620x/TP67 definition
	#undef	__TP6X_SELECT_TP62_TP67_TP620X__
	#define	__TP6X_SELECT_TP62_TP67_TP620X__	1
	#undef	__TP6X_SDRAM128M__
	#undef	__TP6X_DPRAM256K__
#endif
//============================================================================


//================ General TP6x/TP64xx DSP defs ==============================

	//----- DSP clock defs
#define	TP62_DSP_CLK_FREQ				2.0e+8	// TP62 DSP clock frequency (Hz)
#define	TP67_DSP_CLK_FREQ				1.67e+8	// TP67 DSP clock frequency (Hz)
#define	TP6202_DSP_CLK_FREQ				2.5e+8	// TP6202 DSP clock frequency (Hz)
#define	TP6203_DSP_CLK_FREQ				3.0e+8	// TP6203 DSP clock frequency (Hz)
#define	TP64XX_DSP_CLK_FREQ_600MHZ		6.0e+8	// 600MHz TP64xx DSP clock frequency (Hz)
#define	TP64XX_DSP_CLK_FREQ_720MHZ		7.2e+8	// 720MHz TP64xx DSP clock frequency (Hz)
#define	TP64XX_DSP_CLK_FREQ_1000MHZ		1.0e+9	// 1GHz TP64xx DSP clock frequency (Hz)
//============================================================================



#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
//================== TP62/TP67/TP620x Memory/IO Areas ========================
// DSP memory areas for TP62/TP67/TP620x DSP systems


	//----- SBSRAM length defs
	// Common for all TP6x boards. TP6202/TP6203 must enable SBSRAM prior using it.
	// SBSRAM area for TP6x can be either 256Kbytes (64Kx32), or 512Kbyte (128Kx32),
	// or 1Mbytes (256Kx32), or 2Mbytes (512Kx32).
		// length in x32 words
#define	TP6X_SBSRAM_LEN_0K				0x00000000
#define	TP6X_SBSRAM_LEN_64K				0x00040000
#define	TP6X_SBSRAM_LEN_128K			0x00080000
#define	TP6X_SBSRAM_LEN_256K			0x00100000
#define	TP6X_SBSRAM_LEN_512K			0x00200000
#define	TP6X_SBSRAM_LEN_1M				0x00400000
#define	TP6X_SBSRAM_LEN_MAX				TP6X_SBSRAM_LEN_512K
		// aliases in kbytes/Mbytes
#define	TP6X_SBSRAM_LEN_0KB				TP6X_SBSRAM_LEN_0K
#define	TP6X_SBSRAM_LEN_256KB			TP6X_SBSRAM_LEN_64K
#define	TP6X_SBSRAM_LEN_512KB			TP6X_SBSRAM_LEN_128K
#define	TP6X_SBSRAM_LEN_1MB				TP6X_SBSRAM_LEN_256K
#define	TP6X_SBSRAM_LEN_2MB				TP6X_SBSRAM_LEN_512K
#define	TP6X_SBSRAM_LEN_4MB				TP6X_SBSRAM_LEN_1M

	//----- SDRAM length defs
	// common for all TP6x boards. TP6202/TP6203 must enable SDRAM prior using it.
	// SDRAM area for TP6x can be 16Mbytes (4Mx16).
		// length in x32 words
#define	TP6X_SDRAM_LEN_0M				0x00000000
#define	TP6X_SDRAM_LEN_4M				0x01000000
#define	TP6X_SDRAM_LEN_8M				0x02000000
#define	TP6X_SDRAM_LEN_MAX				TP6X_SDRAM_LEN_4M
		// aliases in kbytes/Mbytes
#define	TP6X_SDRAM_LEN_0MB				TP6X_SDRAM_LEN_0M
#define	TP6X_SDRAM_LEN_16MB				TP6X_SDRAM_LEN_4M
#define	TP6X_SDRAM_LEN_32MB				TP6X_SDRAM_LEN_8M

	//----- DPRAM length defs
		// length in x32 words
#define	TP6X_DPRAM_LEN_0K				0x00000000
#define	TP6X_DPRAM_LEN_32K				0x00020000
#define	TP6X_DPRAM_LEN_64K				0x00040000
#define	TP6X_DPRAM_LEN_MAX				TP6X_DPRAM_LEN_64K
		// aliases in kbytes/Mbytes
#define	TP6X_DPRAM_LEN_0KB				TP6X_DPRAM_LEN_0K
#define	TP6X_DPRAM_LEN_128KB			TP6X_DPRAM_LEN_32K
#define	TP6X_DPRAM_LEN_256KB			TP6X_DPRAM_LEN_64K


	//----------- SBSRAM area allocation @CE-2
#define	TP6X_SBSRAM_SADDR				0x02000000
#define	TP6X_SBSRAM_EADDR				(TP6X_SBSRAM_SADDR + TP6X_SBSRAM_LEN_MAX -1)
#define	TP6X_SBSRAM_LEN					TP6X_SBSRAM_LEN_MAX

	//----------- SDRAM area allocation @CE-3
#define	TP6X_SDRAM_SADDR				0x03000000
#define	TP6X_SDRAM_EADDR				(TP6X_SDRAM_SADDR + TP6X_SDRAM_LEN_MAX -1)
#define	TP6X_SDRAM_LEN					TP6X_SDRAM_LEN_MAX

	//----------- SDRAM area allocation @CE-2
	// specific to TP6202/TP6203 only at the SBSRAM area when 8Mx32 SDRAM is installed.
	// TP6202/TP6203 shall enable SDRAM prior using it.
	// SDRAM area for TP6x can be 16Mbytes (4Mx16).
#define	TP6X_SDRAM2_SADDR				0x02000000
#define	TP6X_SDRAM2_EADDR				(TP6X_SDRAM2_SADDR + TP6X_SDRAM_LEN_MAX -1)
#define	TP6X_SDRAM2_LEN					TP6X_SDRAM_LEN_MAX

	//----------- FLASH/EPROM area @CE-1
	// FLASH/EPROM area can be 128Kx8, or 256Kx8 (EPROM only), or 512Kx8,
	// or 1Mx8 (EPROM only). Note, that FLASH/EPROM length is treated as 32-bit words for TP6x boards.
	   // FLASH area params (EMIF-CE1) (these defs are introduces since s/w rev.4A for support of TP64xx rev.1B boards with large/paged FLASH memories)
#define	TP6X_FLASH_AREA_SADDR			0x01000000
#define	TP6X_FLASH_AREA_EADDR			0x013fffff
#define	TP6X_FLASH_AREA_LEN				0x00400000
#define	TP6X_FLASH_AREA_ADDR_STEP		4

			// FLASH PAGE defs fro compatibility with TP64xx rev.1B API v.4.A
#define	TP6X_FLASH_PAGE_SADDR			TP6X_FLASH_AREA_SADDR
#define	TP6X_FLASH_PAGE_EADDR			TP6X_FLASH_AREA_EADDR
#define	TP6X_FLASH_PAGE_LEN				TP6X_FLASH_AREA_LEN
#define	TP6X_FLASH_PAGE_ADDR_STEP		TP6X_FLASH_AREA_ADDR_STEP

			// aliases with the APIs prior rev.4A
#define	TP6X_FLASH_SADDR				TP6X_FLASH_AREA_SADDR
#define	TP6X_FLASH_EADDR				TP6X_FLASH_AREA_EADDR
#define	TP6X_FLASH_LEN					TP6X_FLASH_AREA_LEN
#define	TP6X_FLASH_ADDR_STEP			TP6X_FLASH_AREA_ADDR_STEP

		// FLASH/EPROM area length defs for TP6x
#define	TP6X_FLASH_LEN_128K				0x00080000
#define	TP6X_FLASH_LEN_256K				0x00100000
#define	TP6X_FLASH_LEN_512K				0x00200000
#define	TP6X_FLASH_LEN_1M				0x00400000
			// aliases with API rev.3B and earlier
#define	TP6X_FLASH128K_LEN				TP6X_FLASH_LEN_128K
#define	TP6X_FLASH256K_LEN				TP6X_FLASH_LEN_256K
#define	TP6X_FLASH512K_LEN				TP6X_FLASH_LEN_512K
#define	TP6X_FLASH1M_LEN				TP6X_FLASH_LEN_1M


	//----------- DPRAM/PIOX/AUX I/O @CE-0
		// DPRAM len for TP6x can be either 32Kx32 (default) or 64Kx32.
#define TP6X_DPRAM_SADDR				0x00000000	// DPRAM r/w, D0..D31 (8/16/32-bit cycles)
#ifdef	__TP6X_DPRAM64K__
#define TP6X_DPRAM_EADDR				0x0003ffff	// DPRAM 64Kx32 defs
#define TP6X_DPRAM_LEN					0x00040000
#else
#define TP6X_DPRAM_EADDR				0x0001ffff	// DPRAM 32Kx32 defs
#define TP6X_DPRAM_LEN					0x00020000
#endif
#define	TP6X_DPRAM_HM_RQ_ADDR			(TP6X_DPRAM_EADDR-3)	// DPRAM address for Host-to-DSP irq
#define	TP6X_DPRAM_MH_RQ_ADDR			(TP6X_DPRAM_EADDR-7)	// DPRAM address for DSP-to-HOST irq

		// DPSEM area. D0 bit is valid only of 32-bit datawords.
#define TP6X_DPSEM_SADDR				0x00080000	// DPSEM r/w, D0 only
#define TP6X_DPSEM_EADDR				0x00080007
#define TP6X_DPSEM_LEN					0x00000008

		// Note, that 32-bit PIOX I/F for TP64xx has 19 address lines and can address
		// 512K 32-bit I/F datawords only.
#define TP6X_PIOX_SADDR					0x00200000	// 32-bit PIOX r/w, D0..D31 (8/16/32-bit cycles)
#define TP6X_PIOX_EADDR					0x003fffff
#define TP6X_PIOX_LEN					0x00200000	// PIOX length in allocated 32-bit DSP area datawords
#define TP6X_PIOX16_SADDR				0x00200000	// 16-bit PIOX-16 r/w, D0..D15 (LSW of 32-bit words, byte control is ignored)
#define TP6X_PIOX16_EADDR				0x0023ffff
#define TP6X_PIOX16_LEN					0x00040000	// PIOX-16 length in allocated 32-bit DSP area datawords

#define TP6X_PIOX_LENW					0x00080000	// PIOX length in 32-bit I/F data words (512K)
#define TP6X_PIOX16_LENW				0x00010000	// PIOX-16 length in 16-bit I/F data words (64K)

			// aliases
#define TP6X_PX_SADDR					TP6X_PIOX_SADDR
#define TP6X_PX_EADDR					TP6X_PIOX_EADDR
#define TP6X_PX_LEN						TP6X_PIOX_LEN
#define TP6X_PX16_SADDR					TP6X_PIOX16_SADDR
#define TP6X_PX16_EADDR					TP6X_PIOX16_EADDR
#define TP6X_PX16_LEN					TP6X_PIOX16_LEN
#define TP6X_PX_LENW					TP6X_PIOX_LENW
#define TP6X_PX16_LENW					TP6X_PIOX16_LENW


		// 8-bit parallel data bus of SIOX-A/B rev.C I/F sites
		// Allocated to LSB of 32-bit datawords.
#define TP6X_SIOXA_REVC_SADDR			0x00100000	// SIOX-A| r/w, D0..D7 , A0..A5 (->A2..A7)
#define TP6X_SIOXA_REVC_EADDR			0x001000ff
#define TP6X_SIOXB_REVC_SADDR			0x00140000	// SIOX-B| r/w, D0..D7 , A0..A5 (->A2..A7)
#define TP6X_SIOXB_REVC_EADDR			0x001400ff

#define TP6X_SIOX_REVC_LEN				0x00000100	// SIOX-A/B area length in allocated 32-bit DSP area datawords
#define TP6X_SIOXA_REVC_LEN				TP6X_SIOX_REVC_LEN
#define TP6X_SIOXB_REVC_LEN				TP6X_SIOX_REVC_LEN

#define TP6X_SIOX_REVC_LENW				0x00000040	// SIOX-A/B length in 8-bit I/F datawords (64 words)
#define TP6X_SIOXA_REVC_LENW			TP6X_SIOX_REVC_LENW
#define TP6X_SIOXB_REVC_LENW			TP6X_SIOX_REVC_LENW

			// aliases for compatibility with old TP6x utilities
#define TP6X_SXA_SADDR					TP6X_SIOXA_REVC_SADDR
#define TP6X_SXA_EADDR					TP6X_SIOXA_REVC_EADDR
#define TP6X_SXB_SADDR					TP6X_SIOXB_REVC_SADDR
#define TP6X_SXB_EADDR					TP6X_SIOXB_REVC_EADDR
#define TP6X_SX_LEN						TP6X_SIOX_REVC_LEN
#define TP6X_SXA_LEN					TP6X_SIOXA_REVC_LEN
#define TP6X_SXB_LEN					TP6X_SIOXB_REVC_LEN
#define TP6X_SX_LENW					TP6X_SIOX_REVC_LENW
#define TP6X_SXA_LENW					TP6X_SIOXA_REVC_LENW
#define TP6X_SXB_LENW					TP6X_SIOXB_REVC_LENW


		//-- AMCC S5933 PCIC ports (32-bit regs)
#define TP6X_PCIC_FIFO_ADDR				0x000c0000	// FIFO: r/w
#define TP6X_PCIC_AMBEF_ADDR			0x000c0004	// AMBEF:
#define TP6X_PCIC_AINT_ADDR				0x000c0008	// AINT:
#define TP6X_PCIC_AGCSTS_ADDR			0x000c000c	// AGCSTS:
#define TP6X_PCIC_MWAR_ADDR				0x000c0010	// MWAR:
#define TP6X_PCIC_MWTC_ADDR				0x000c0014	// MWTC:
#define TP6X_PCIC_MRAR_ADDR				0x000c0018	// MRAR:
#define TP6X_PCIC_MRTC_ADDR				0x000c001c	// MRTC:
#define TP6X_PCIC_IMBX0_ADDR			0x000c0020	// IMBX-0..3: r-only
#define TP6X_PCIC_IMBX1_ADDR			0x000c0024	
#define TP6X_PCIC_IMBX2_ADDR			0x000c0028
#define TP6X_PCIC_IMBX3_ADDR			0x000c002c	
#define TP6X_PCIC_OMBX0_ADDR			0x000c0030	// OMBX-0..3: r/w
#define TP6X_PCIC_OMBX1_ADDR			0x000c0034	
#define TP6X_PCIC_OMBX2_ADDR			0x000c0038
#define TP6X_PCIC_OMBX3_ADDR			0x000c003c

		//-- IOX control regs (D0..D3 only)
#define TP6X_FIFO_STAT_ADDR				0x000c0040	// FIFO STATUS: r-only
#define TP6X_AMWREN_ADDR				0x000c0044	// AMWREN: r/w
#define TP6X_MH_RQ_ADDR					0x000c0050	// MH-RQ (wr->EMBX): w-only
#define TP6X_PXSX_RESET_ADDR			0x000c0054	// PIOX/SIOX RESET control: r/w
#define TP6X_SYS_STAT_ADDR				0x000c0058	// SYS STATUS: r/w
#define	TP6X_WDT_RESET_ADDR				0x000c005c	// WDT-CLEAR: w-only, data ignored
#define TP6X_DEV_ID_ADDR				0x000c005c	// DEV_ID: r-only,
#define TP6X_WDT_ENABLE_ADDR			0x000c0060	// WDT-ENABLE: r/w
#define TP6X_HM_RQ0_ADDR				0x000c0064	// HM_RQ-0/1: r-only, w-clear
#define TP6X_HM_RQ1_ADDR				0x000c0068
#define TP6X_NMI_SEL_ADDR				0x000c006c	// NMI/MIRQ-0..3 Selectors: r/w
#define TP6X_EXT_INT4_SEL_ADDR			0x000c0070
#define TP6X_EXT_INT5_SEL_ADDR			0x000c0074
#define TP6X_EXT_INT6_SEL_ADDR			0x000c0078
#define TP6X_EXT_INT7_SEL_ADDR			0x000c007c

			// old API defs, not recommended for usage, provided for compatibility only
#define TP6X_PXSX_RUN_ADDR				TP6X_PXSX_RESET_ADDR
#define TP6X_MIRQ0_SEL_ADDR				TP6X_EXT_INT4_SEL_ADDR
#define TP6X_MIRQ1_SEL_ADDR				TP6X_EXT_INT5_SEL_ADDR
#define TP6X_MIRQ2_SEL_ADDR				TP6X_EXT_INT6_SEL_ADDR
#define TP6X_MIRQ3_SEL_ADDR				TP6X_EXT_INT7_SEL_ADDR

//============================================================================
#endif



#ifdef	__TP6X_SELECT_TP64XX__
//====================== TP64xx Memory/IO Areas ==============================
// DSP memory areas for TP64xx DSP systems
//
// Important note:
// ---------------
//		1. Base addresses for TP64x external memory and I/O areas are not
//		   compatible with that for TP62/TP620x/TP67 DSP systems due to incompatible
//		   'C620x/C670x and 'C64xx DSP memory maps.
//		2. There is no dual-port semaphore area (DPSEM) for TP64xx.
//		3. Host-to-Master and Master-to-Host IRQs via DPRAM are supported for
//		   256Kx32 DPRAM only and are not available for 128Kx32 DPRAM.
//		4. Relative addresses for external I/O control regs (IOX), PCIC and PIOX/SIOX I/F
//		   are also incompatible.
//		5. Although external I/O control regs and SIOX rev.C subareas of TP64xx are allocated
//		   at 32-bit DSP dataword boundaries, they are recommended to be access as
//		   16-bit UNSIGNED SHORT vars in order to exclude second fault access to MSW.
//		6. Access to AMCC S5933 PCIC and PIOX/PIOX-16 I/F are supplied with
//		   data format control in order to allow access as either two 16-bit words, or
//		   as one 32-bit data word with intermediate LSW/MSW half-word storage.
//		7. FLASH memory is allocated to 16-bit EMIF-B BCE1 area and is allocated to
//		   LSB of 16-bit words at x2 DSP address boundaries. FLASH must be accessed
//		   as LSB of 16-bit UNSIGNED SHORT vars.
//		8. Aliases are provided for each TP64xx definition in order to provide
//		   compatibility with TP62/TP620x/TP67 utilities without update of user code.
//		9. Starting fm TP6x API rev.3C, the EMIF-B@CE1 area for FLASH memory access is configured as
//		   the 8-bit ASYNC area for compatibility with future TP64xx board revisions. FLASH must be accessed
//		   as UNSIGNED CHAR (BYTE) words. TP6x API rev.3B and earlier configured the EMIF-B@CE1 area
//		   as 16-bit ASYNC and allowed access to 8-bit FLASH as LSB of 16-bit datawords. No modification of
//		   application code is required in case supplied FLASH programming utilities are used.
//		10.Starting fm TP6x API rev.3C with support for TP64xx f/w rev.1A-1 and rev.1A-2, the
//		   SBSRAM/SDRAM/SDPRAM areas defs now return the area length via the corresponding macros using
//		   the XMEM_LEN_ID_RG register. For f/w rev.1A, the maximum lengths defs are used.


	//----- SBSRAM length defs
	// SBSRAM area for C64xx can be either 1Mbytes (128Kx64), or 2Mbytes (256Kx64),
	// or 4Mbytes (512Kx64), or 8 MBytes (1Mx64).
		// length in x64 words
#define	TP64XX_SBSRAM_LEN_0K			0x00000000
#define	TP64XX_SBSRAM_LEN_128K			0x00100000
#define	TP64XX_SBSRAM_LEN_256K			0x00200000
#define	TP64XX_SBSRAM_LEN_512K			0x00400000
#define	TP64XX_SBSRAM_LEN_1M			0x00800000
#define	TP64XX_SBSRAM_LEN_MAX			TP64XX_SBSRAM_LEN_1M
		// aliases in kbytes/Mbytes
#define	TP64XX_SBSRAM_LEN_0KB			TP64XX_SBSRAM_LEN_0K
#define	TP64XX_SBSRAM_LEN_1MB			TP64XX_SBSRAM_LEN_128K
#define	TP64XX_SBSRAM_LEN_2MB			TP64XX_SBSRAM_LEN_256K
#define	TP64XX_SBSRAM_LEN_4MB			TP64XX_SBSRAM_LEN_512K
#define	TP64XX_SBSRAM_LEN_8MB			TP64XX_SBSRAM_LEN_1M
		// TP6x API compatible aliases in words & kbytes/Mbytes
#define	TP6X_SBSRAM_LEN_0K				TP64XX_SBSRAM_LEN_0K
#define	TP6X_SBSRAM_LEN_128K			TP64XX_SBSRAM_LEN_128K
#define	TP6X_SBSRAM_LEN_256K			TP64XX_SBSRAM_LEN_256K
#define	TP6X_SBSRAM_LEN_512K			TP64XX_SBSRAM_LEN_512K
#define	TP6X_SBSRAM_LEN_1M				TP64XX_SBSRAM_LEN_1M
#define	TP6X_SBSRAM_LEN_MAX				TP64XX_SBSRAM_LEN_MAX

#define	TP6X_SBSRAM_LEN_0KB				TP64XX_SBSRAM_LEN_0K
#define	TP6X_SBSRAM_LEN_1MB				TP64XX_SBSRAM_LEN_128K
#define	TP6X_SBSRAM_LEN_2MB				TP64XX_SBSRAM_LEN_256K
#define	TP6X_SBSRAM_LEN_4MB				TP64XX_SBSRAM_LEN_512K
#define	TP6X_SBSRAM_LEN_8MB				TP64XX_SBSRAM_LEN_1M


	//----- SDRAM length defs
	// SDRAM area for TP64xx can be either 32Mbytes (4Mx64) or 128Mbytes (16Mx64).
		// length in x64 words
#define	TP64XX_SDRAM_LEN_0M				0x00000000
#define	TP64XX_SDRAM_LEN_4M				0x02000000
#define	TP64XX_SDRAM_LEN_16M			0x08000000
#define	TP64XX_SDRAM_LEN_MAX			TP64XX_SDRAM_LEN_4M
		// aliases in kbytes/Mbytes
#define	TP64XX_SDRAM_LEN_0MB			TP64XX_SDRAM_LEN_0M
#define	TP64XX_SDRAM_LEN_32MB			TP64XX_SDRAM_LEN_4M
#define	TP64XX_SDRAM_LEN_128MB			TP64XX_SDRAM_LEN_16M
		// TP6x API compatible aliases in words & kbytes/Mbytes
#define	TP6X_SDRAM_LEN_0M				TP64XX_SDRAM_LEN_0M
#define	TP6X_SDRAM_LEN_4M				TP64XX_SDRAM_LEN_4M
#define	TP6X_SDRAM_LEN_16M				TP64XX_SDRAM_LEN_16M
#define	TP6X_SDRAM_LEN_MAX				TP64XX_SDRAM_LEN_MAX

#define	TP6X_SDRAM_LEN_0MB				TP64XX_SDRAM_LEN_0M
#define	TP6X_SDRAM_LEN_32MB				TP64XX_SDRAM_LEN_4M
#define	TP6X_SDRAM_LEN_128MB			TP64XX_SDRAM_LEN_16M


	//----- DPRAM length defs
	// DPRAM size for TP64xx can be either 512Kbytes (128Kx32, default) or 1Mbyte (256Kx32).
		// length in x32 words
#define	TP64XX_DPRAM_LEN_0K				0x00000000
#define	TP64XX_DPRAM_LEN_128K			0x00080000
#define	TP64XX_DPRAM_LEN_256K			0x00100000
#define	TP64XX_DPRAM_LEN_512K			0x00200000
#define	TP64XX_DPRAM_LEN_MAX			TP64XX_DPRAM_LEN_512K
		// aliases in kbytes/Mbytes
#define	TP64XX_DPRAM_LEN_0KB			TP64XX_DPRAM_LEN_0K
#define	TP64XX_DPRAM_LEN_512KB			TP64XX_DPRAM_LEN_128K
#define	TP64XX_DPRAM_LEN_1MB			TP64XX_DPRAM_LEN_256K
#define	TP64XX_DPRAM_LEN_2MB			TP64XX_DPRAM_LEN_1M
		// TP6x API compatible aliases in words & kbytes/Mbytes
#define	TP6X_DPRAM_LEN_0K				TP64XX_DPRAM_LEN_0K
#define	TP6X_DPRAM_LEN_128K				TP64XX_DPRAM_LEN_128K
#define	TP6X_DPRAM_LEN_256K				TP64XX_DPRAM_LEN_256K
#define	TP6X_DPRAM_LEN_512K				TP64XX_DPRAM_LEN_512K
#define	TP6X_DPRAM_LEN_MAX				TP64XX_DPRAM_LEN_MAX

#define	TP6X_DPRAM_LEN_0KB				TP64XX_DPRAM_LEN_0K
#define	TP6X_DPRAM_LEN_512KB			TP64XX_DPRAM_LEN_128K
#define	TP6X_DPRAM_LEN_1MB				TP64XX_DPRAM_LEN_256K
#define	TP6X_DPRAM_LEN_2MB				TP64XX_DPRAM_LEN_1M


	//----------- SBSRAM (x64) area allocation at @EMIF-A/CE-0
	// Notes:
	// 		 1. For TP64xx f/w rev.1A-1/2 the particular SBSRAM length ID can be read by DSP from XMEM_LEN_ID_RG
	//		    read-only register.
	//		 2. For TP64xx f/w rev.1A the SBSRAM length is returned as 512Kx64 by the TP6X_get_sbsram_len() function.
	// 		 2. Care must be taken, since TP64xx_SBSRAM_EADDR macro returns (TP6X_SBSRAM_SADDR-1)
	// 		    value in case SBSRAM is not installed!!!
#define	TP64XX_SBSRAM_SADDR				0x80000000                 						// SBSRAM start address
#define	TP64XX_SBSRAM_EADDR				(TP64XX_SBSRAM_SADDR+TP6X_get_sbsram_len()-1)  	// last SBSRAM address
#define	TP64XX_SBSRAM_EADDR_MAX			(TP64XX_SBSRAM_SADDR+TP64XX_SBSRAM_LEN_MAX-1) 	// maximum SBSRAM last address
#define	TP64XX_SBSRAM_LEN				TP6X_get_sbsram_len()
		// aliases for compatibility with TP6x Utilities.
#define	TP6X_SBSRAM_SADDR				TP64XX_SBSRAM_SADDR
#define	TP6X_SBSRAM_EADDR				TP64XX_SBSRAM_EADDR
#define	TP6X_SBSRAM_LEN					TP64XX_SBSRAM_LEN

	//----------- SDRAM (x64) area allocation at @EMIF-A/CE-2
	// Notes:
	// 		 1. For TP64xx f/w rev.1A-1/2 the particular SDRAM length ID can be read by DSP from XMEM_LEN_ID_RG
	//		    read-only register.
	//		 2. For TP64xx f/w rev.1A the SDRAM length is returned as 4Mx64 by the TP6X_get_sdram_len() function.
	// 		 2. Care must be taken, since TP64xx_SDRAM_EADDR macro returns (TP6X_SDRAM_SADDR-1)
	// 		    value in case SDRAM is not installed!!!
#define	TP64XX_SDRAM_SADDR				0xA0000000
#define	TP64XX_SDRAM_EADDR				(TP64XX_SDRAM_SADDR+TP6X_get_sdram_len()-1)   	// last SDRAM address
#define	TP64XX_SDRAM_EADDR_MAX			(TP64XX_SDRAM_SADDR+TP6X_SDRAM_LEN_MAX-1) 		// maximum SDRAM last address
#define	TP64XX_SDRAM_LEN				0x08000000
		// aliases for compatibility with TP6x utilities.
#define	TP6X_SDRAM_SADDR				TP64XX_SDRAM_SADDR
#define	TP6X_SDRAM_EADDR				TP64XX_SDRAM_EADDR
#define	TP6X_SDRAM_LEN					TP64XX_SDRAM_LEN

	//----------- DPRAM (x32) area allocation at @EMIF-A/CE-1
#define TP64XX_DPRAM_SADDR				0x90000000
#define	TP64XX_DPRAM_EADDR				(TP64XX_DPRAM_SADDR+TP6X_get_dpram_len()-1)   	// last DPRAM address
#define	TP64XX_DPRAM_EADDR_MAX			(TP64XX_DPRAM_SADDR+TP6X_DPRAM_LEN_MAX-1) 		// maximum DPRAM last address
#define TP64XX_DPRAM_LEN				TP6X_get_dpram_len()

		// DPRAM addresses for Host-to-DSP irq and DSP-to-Host irq
		// Note, that these addresses are valid for TP64xx boards with DPRAM 256K/512Kx32 only.
		// nevertheless, these defs do not check for valid DPRAM length and return the
		// values for even the 128Kx32 DPRAM memory in order to exclude any run-time error due
		// to access into invalid memory location.
#define	TP64XX_DPRAM_HM_RQ_ADDR			(TP64XX_DPRAM_EADDR-3)	// DPRAM address for Host-to-DSP irq
#define	TP64XX_DPRAM_MH_RQ_ADDR			(TP64XX_DPRAM_EADDR-7)	// DPRAM address for DSP-to-HOST irq

		// aliases for compatibility with TP6x Utilities.
#define TP6X_DPRAM_SADDR				TP64XX_DPRAM_SADDR
#define TP6X_DPRAM_EADDR				TP64XX_DPRAM_EADDR
#define TP6X_DPRAM_LEN					TP64XX_DPRAM_LEN
#define	TP6X_DPRAM_HM_RQ_ADDR			TP64XX_DPRAM_HM_RQ_ADDR
#define	TP6X_DPRAM_MH_RQ_ADDR			TP64XX_DPRAM_MH_RQ_ADDR


	//----------- FLASH/EPROM (x8) area @EMIF-B/CE-1
	// TP64xx h/w rev.1A (f/w rev.1A/1A-1/1A-2):
	// 		  - PLCC-32 IC socket for FLASH/EPROM memory
	// 		  - FLASH/EPROM area can be 128Kx8 (FLASH/EPROM), or 256Kx8 (EPROM only), or 512Kx8 (FLASH/EPROM),
	// 		  	or 1Mx8 (EPROM only). Note, that FLASH/EPROM length is treated as 8-bit words for TP64xx.
	//		  - no XMEM2_LEN_ID IOX register with FLASH capacity ID, i.e. no possibility to identify the installed
	//		  	FLASH/EPROM length (except for the case if AMD 5v-only FLASH is installed and FLASH WRITE is enabled).
	//
	// TP64xx h/w rev.1B (f/w rev.1B and later):
	// 		  - factory installed either of the AMD 512K/1M/2M/4M/8M/16M/32M/64Mx8 FLASH memory
	//		  - XMEM2_LEN_ID IOX register with the FLASH capacity ID
	//		  - 512K/1Mx8 FLASH memories do not use FLASH_PAGE_RG IOX register (all writes to this register are ignored and reads return 00H)
	//		  - 2M/4M/8M/16M/32M/64Mx8 FLASH memories use FLASH_PAGE_RG register (1/2/3/4/5/6-bits correspondingly) in order to select the
	//		  	1Mx8 FLASH page, which is currently accessed via the EMIF-B/CE1 area

		// FLASH memory area defs (EMIF-B/CE-1)
#define	TP64XX_FLASH_AREA_SADDR			0x64000000
#define	TP64XX_FLASH_AREA_EADDR			0x640fffff
#define	TP64XX_FLASH_AREA_LEN			0x00100000
#define	TP64XX_FLASH_AREA_ADDR_STEP		1

			// aliases for compatibility with utilities prior rev.4A
#define	TP64XX_FLASH_SADDR				TP64XX_FLASH_AREA_SADDR
#define	TP64XX_FLASH_EADDR				TP64XX_FLASH_AREA_EADDR
#define	TP64XX_FLASH_LEN				TP64XX_FLASH_AREA_LEN
#define	TP64XX_FLASH_ADDR_STEP			TP64XX_FLASH_AREA_ADDR_STEP

		// FLASH page defs for TP64xx f/w rev.1B and later
#define	TP64XX_FLASH_PAGE_SADDR			TP64XX_FLASH_AREA_SADDR
#define	TP64XX_FLASH_PAGE_EADDR			TP64XX_FLASH_AREA_EADDR
#define	TP64XX_FLASH_PAGE_LEN			TP64XX_FLASH_AREA_LEN
#define	TP64XX_FLASH_PAGE_ADDR_STEP		TP64XX_FLASH_AREA_ADDR_STEP

		// FLASH/EPROM length defs for TP64xx
#define	TP64XX_FLASH_LEN_0K				0x00000000  // FLASH not installed
#define	TP64XX_FLASH_LEN_128K			0x00020000  // TP64xx rev.1A/1A-1/1A-2 only
#define	TP64XX_FLASH_LEN_256K			0x00040000	// TP64xx rev.1A/1A-1/1A-2 only
#define	TP64XX_FLASH_LEN_512K			0x00080000  // no FLASH memory paging, direct access to full FLASH memory
#define	TP64XX_FLASH_LEN_1M				0x00100000	// no FLASH memory paging, direct access to full FLASH memory
#define	TP64XX_FLASH_LEN_2M				0x00200000	// 2x FLASH memory pages
#define	TP64XX_FLASH_LEN_4M				0x00400000	// 4x FLASH memory pages
#define	TP64XX_FLASH_LEN_8M				0x00800000	// 8x FLASH memory pages
#define	TP64XX_FLASH_LEN_16M			0x01000000	// 16x FLASH memory pages
#define	TP64XX_FLASH_LEN_32M			0x02000000	// 32x FLASH memory pages
#define	TP64XX_FLASH_LEN_64M			0x04000000	// 64x FLASH memory pages

		// aliases for compatibility with the TP6x.
#define	TP6X_FLASH_AREA_SADDR			TP64XX_FLASH_SADDR
#define	TP6X_FLASH_AREA_EADDR			TP64XX_FLASH_EADDR
#define	TP6X_FLASH_AREA_LEN				TP64XX_FLASH_LEN
#define	TP6X_FLASH_AREA_ADDR_STEP		TP64XX_FLASH_ADDR_STEP

#define	TP6X_FLASH_PAGE_SADDR			TP64XX_FLASH_SADDR
#define	TP6X_FLASH_PAGE_EADDR			TP64XX_FLASH_EADDR
#define	TP6X_FLASH_PAGE_LEN				TP64XX_FLASH_LEN
#define	TP6X_FLASH_PAGE_ADDR_STEP		TP64XX_FLASH_ADDR_STEP

			// aliases with APIs prior rev.4A
#define	TP6X_FLASH_SADDR				TP64XX_FLASH_SADDR
#define	TP6X_FLASH_EADDR				TP64XX_FLASH_EADDR
#define	TP6X_FLASH_LEN					TP64XX_FLASH_LEN
#define	TP6X_FLASH_ADDR_STEP			TP64XX_FLASH_ADDR_STEP


			// FLASH length defs for TP6x utilities
#define	TP6X_FLASH_LEN_0K				TP64XX_FLASH_LEN_0K
#define	TP6X_FLASH_LEN_128K				TP64XX_FLASH_LEN_128K
#define	TP6X_FLASH_LEN_256K				TP64XX_FLASH_LEN_256K
#define	TP6X_FLASH_LEN_512K				TP64XX_FLASH_LEN_512K
#define	TP6X_FLASH_LEN_1M				TP64XX_FLASH_LEN_1M
#define	TP6X_FLASH_LEN_2M				TP64XX_FLASH_LEN_2M
#define	TP6X_FLASH_LEN_4M				TP64XX_FLASH_LEN_4M
#define	TP6X_FLASH_LEN_8M				TP64XX_FLASH_LEN_8M
#define	TP6X_FLASH_LEN_16M				TP64XX_FLASH_LEN_16M
#define	TP6X_FLASH_LEN_32M				TP64XX_FLASH_LEN_32M
#define	TP6X_FLASH_LEN_64M				TP64XX_FLASH_LEN_64M

			// aliases with API rev.3B and earlier
#define	TP6X_FLASH0K_LEN				TP6X_FLASH_LEN_0K
#define	TP6X_FLASH128K_LEN				TP6X_FLASH_LEN_128K
#define	TP6X_FLASH256K_LEN				TP6X_FLASH_LEN_256K
#define	TP6X_FLASH512K_LEN				TP6X_FLASH_LEN_512K
#define	TP6X_FLASH1M_LEN				TP6X_FLASH_LEN_1M
#define	TP6X_FLASH2M_LEN				TP6X_FLASH_LEN_2M
#define	TP6X_FLASH4M_LEN				TP6X_FLASH_LEN_4M
#define	TP6X_FLASH8M_LEN				TP6X_FLASH_LEN_8M
#define	TP6X_FLASH16M_LEN				TP6X_FLASH_LEN_16M
#define	TP6X_FLASH32M_LEN				TP6X_FLASH_LEN_32M
#define	TP6X_FLASH64M_LEN				TP6X_FLASH_LEN_64M



	//----------- PIOX I/O area at @EMIF-B/CE-3
	// PIOX/PIOX-16 area is allocated at x32 bit boundaries and can be
	// accessed as 16-bit or 32-bit data words.
	// PIOX access data format is controlled via px_ccl_fmt[0:1] bits of
	// TP64XX_XIO_FMT_RG IOX register.
	// Note, that 32-bit PIOX I/F for TP64xx has 19 address lines and can address
	// 512K 32-bit I/F datawords only.
#define TP64XX_PIOX_SADDR				0x6C000000	// 32-bit PIOX: D0..31 (8/16/32-bit accesses_
#define TP64XX_PIOX_EADDR				0x6C1fffff
#define TP64XX_PIOX_LEN					0x00200000	// PIOX length in allocated 32-bit DSP area datawords
#define TP64XX_PIOX16_SADDR				0x6C000000	// 16-bit PIOX-16: D0..15 (LSW of 32-bit words, byte control is ignored)
#define TP64XX_PIOX16_EADDR				0x6C03ffff
#define TP64XX_PIOX16_LEN				0x00040000	// PIOX-16 length in allocated 32-bit DSP area datawords
#define TP64XX_PIOX_LENW				0x00080000	// PIOX length in 32-bit I/F data words (512K)
#define TP64XX_PIOX16_LENW				0x00010000	// PIOX-16 length in 16-bit I/F data words (64K)
		// aliases for compatibility with TP6x Utilities.
#define TP6X_PIOX_SADDR					TP64XX_PIOX_SADDR
#define TP6X_PIOX_EADDR					TP64XX_PIOX_EADDR
#define TP6X_PIOX_LEN					TP64XX_PIOX_LEN
#define TP6X_PIOX16_SADDR				TP64XX_PIOX16_SADDR
#define TP6X_PIOX16_EADDR				TP64XX_PIOX16_EADDR
#define TP6X_PIOX16_LEN					TP64XX_PIOX16_LEN
#define TP6X_PIOX_LENW					TP64XX_PIOX_LENW
#define TP6X_PIOX16_LENW				TP64XX_PIOX16_LENW
		// aliases
#define TP6X_PX_SADDR					TP6X_PIOX_SADDR
#define TP6X_PX_EADDR					TP6X_PIOX_EADDR
#define TP6X_PX_LEN						TP6X_PIOX_LEN
#define TP6X_PX16_SADDR					TP6X_PIOX16_SADDR
#define TP6X_PX16_EADDR					TP6X_PIOX16_EADDR
#define TP6X_PX16_LEN					TP6X_PIOX16_LEN
#define TP6X_PX_LENW					TP6X_PIOX_LENW
#define TP6X_PX16_LENW					TP6X_PIOX16_LENW


	//----------- SIOX_rev_C I/O sub-area at @EMIF-B/CE-0
	// 8-bit parallel data bus of SIOX-A/B rev.C I/F sites.
	// Are allocated as LSB of 32-bit datawords with byte control, and can be
	// accessed as either 8-bit or 16-bit datawords.
	// Are not recommended to access as 32-bit dataword in order to exclude splitting of
	// 32-bit word into two 16-bit cycle and second faulty 16-bit LSW access cycle.
#define TP64XX_SIOXA_REVC_SADDR			0x60100000	// SIOX-A r/w, D0..D7 , A0..A5 (->A2..A7)
#define TP64XX_SIOXA_REVC_EADDR			0x6010007f
#define TP64XX_SIOXB_REVC_SADDR			0x60180000	// SIOX-B r/w, D0..D7 , A0..A5  (->A2..A7)
#define TP64XX_SIOXB_REVC_EADDR			0x6018007f
#define TP64XX_SIOX_REVC_LEN			0x00000080 // SIOX-A/B area length in allocated 16-bit DSP area datawords
#define TP64XX_SIOXA_REVC_LEN			TP64XX_SIOX_REVC_LEN
#define TP64XX_SIOXB_REVC_LEN			TP64XX_SIOX_REVC_LEN

#define TP64XX_SIOX_REVC_LENW			0x00000040	// SIOX-A/B length in 8-bit I/F datawords (64 words)
#define TP64XX_SIOXA_REVC_LENW			TP64XX_SIOX_REVC_LENW
#define TP64XX_SIOXB_REVC_LENW			TP64XX_SIOX_REVC_LENW

		// aliases for compatibility with TP6x Utilities.
#define TP6X_SIOXA_REVC_SADDR			TP64XX_SIOXA_REVC_SADDR
#define TP6X_SIOXA_REVC_EADDR			TP64XX_SIOXA_REVC_EADDR
#define TP6X_SIOXB_REVC_SADDR			TP64XX_SIOXB_REVC_SADDR
#define TP6X_SIOXB_REVC_EADDR			TP64XX_SIOXB_REVC_EADDR
#define TP6X_SIOX_REVC_LEN				TP64XX_SIOX_REVC_LEN
#define TP6X_SIOXA_REVC_LEN				TP64XX_SIOXA_REVC_LEN
#define TP6X_SIOXB_REVC_LEN				TP64XX_SIOXB_REVC_LEN
#define TP6X_SIOX_REVC_LENW				TP64XX_SIOX_REVC_LENW
#define TP6X_SIOXA_REVC_LENW			TP64XX_SIOXA_REVC_LENW
#define TP6X_SIOXB_REVC_LENW			TP64XX_SIOXB_REVC_LENW

		// aliases for compatibility with old TP6x utilities
#define TP6X_SXA_SADDR					TP6X_SIOXA_REVC_SADDR
#define TP6X_SXA_EADDR					TP6X_SIOXA_REVC_EADDR
#define TP6X_SXB_SADDR					TP6X_SIOXB_REVC_SADDR
#define TP6X_SXB_EADDR					TP6X_SIOXB_REVC_EADDR
#define TP6X_SX_LEN						TP6X_SIOX_REVC_LEN
#define TP6X_SXA_LEN					TP6X_SIOXA_REVC_LEN
#define TP6X_SXB_LEN					TP6X_SIOXB_REVC_LEN
#define TP6X_SX_LENW					TP6X_SIOX_REVC_LENW
#define TP6X_SXA_LENW					TP6X_SIOXA_REVC_LENW
#define TP6X_SXB_LENW					TP6X_SIOXB_REVC_LENW



	//----------- AMCC S5933 PCIC I/O sub-area at @EMIF-B/CE-0
	// 32-bit AMCC PCIC sub-area.
	// PCIC 16/32-bit access data format is controlled via pcic_ccl_fmt bit of
	// TP64XX_XIO_FMT_RG IOX register.
#define TP64XX_PCIC_FIFO_ADDR			0x60000000	// FIFO: r/w
#define TP64XX_PCIC_AMBEF_ADDR			0x60000004	// AMBEF:
#define TP64XX_PCIC_AINT_ADDR			0x60000008	// AINT:
#define TP64XX_PCIC_AGCSTS_ADDR			0x6000000c	// AGCSTS:
#define TP64XX_PCIC_MWAR_ADDR			0x60000010	// MWAR:
#define TP64XX_PCIC_MWTC_ADDR			0x60000014	// MWTC:
#define TP64XX_PCIC_MRAR_ADDR			0x60000018	// MRAR:
#define TP64XX_PCIC_MRTC_ADDR			0x6000001c	// MRTC:
#define TP64XX_PCIC_IMBX0_ADDR			0x60000020	// IMBX-0..3: r-only
#define TP64XX_PCIC_IMBX1_ADDR			0x60000024
#define TP64XX_PCIC_IMBX2_ADDR			0x60000028
#define TP64XX_PCIC_IMBX3_ADDR			0x6000002c
#define TP64XX_PCIC_OMBX0_ADDR			0x60000030	// OMBX-0..3: r/w
#define TP64XX_PCIC_OMBX1_ADDR			0x60000034
#define TP64XX_PCIC_OMBX2_ADDR			0x60000038
#define TP64XX_PCIC_OMBX3_ADDR			0x6000003c
		// aliases for compatibility with TP6x Utilities.
#define TP6X_PCIC_FIFO_ADDR				TP64XX_PCIC_FIFO_ADDR
#define TP6X_PCIC_AMBEF_ADDR			TP64XX_PCIC_AMBEF_ADDR
#define TP6X_PCIC_AINT_ADDR				TP64XX_PCIC_AINT_ADDR
#define TP6X_PCIC_AGCSTS_ADDR			TP64XX_PCIC_AGCSTS_ADDR
#define TP6X_PCIC_MWAR_ADDR				TP64XX_PCIC_MWAR_ADDR
#define TP6X_PCIC_MWTC_ADDR				TP64XX_PCIC_MWTC_ADDR
#define TP6X_PCIC_MRAR_ADDR				TP64XX_PCIC_MRAR_ADDR
#define TP6X_PCIC_MRTC_ADDR				TP64XX_PCIC_MRTC_ADDR
#define TP6X_PCIC_IMBX0_ADDR			TP64XX_PCIC_IMBX0_ADDR
#define TP6X_PCIC_IMBX1_ADDR			TP64XX_PCIC_IMBX1_ADDR
#define TP6X_PCIC_IMBX2_ADDR			TP64XX_PCIC_IMBX2_ADDR
#define TP6X_PCIC_IMBX3_ADDR			TP64XX_PCIC_IMBX3_ADDR
#define TP6X_PCIC_OMBX0_ADDR			TP64XX_PCIC_OMBX0_ADDR
#define TP6X_PCIC_OMBX1_ADDR			TP64XX_PCIC_OMBX1_ADDR
#define TP6X_PCIC_OMBX2_ADDR			TP64XX_PCIC_OMBX2_ADDR
#define TP6X_PCIC_OMBX3_ADDR			TP64XX_PCIC_OMBX3_ADDR



	//----------- IOX control registers sub-area at @EMIF-B/CE-0
	// IOX control registers of TP64xx are allocated as 8-bit LSB of 32-bit datawords,
	// and can be accessed as either 8-bit and 16-bit datawords.
	// Are not recommended to access as 32-bit datawords in order to exclude splitting of
	// 32-bit word into two 16-bit cycles and avoid faulty 16-bit MSW access cycle.
#define TP64XX_FIFO_STAT_ADDR			0x60000040	// FIFO STATUS: r-only
#define TP64XX_AMWREN_ADDR				0x60000044	// AMWREN: r/w
#define TP64XX_XMEM_LEN_ID_ADDR			0x60000048	// (f/w rev.1A-1/2) r-only SBSRAM/SDRAM/DPRAM capacity ID
#define TP64XX_XIO_FMT_ADDR				0x6000004C	// XIO_FMT: r/w, (TP64xx only!!)
#define TP64XX_MH_RQ_ADDR				0x60000050	// MH-RQ (wr->EMBX): w-only
#define TP64XX_PXSX_RESET_ADDR			0x60000054	// PIOX/SIOX RESET control: r/w
#define TP64XX_SYS_STAT_ADDR			0x60000058	// SYS STATUS: r/w
#define	TP64XX_WDT_RESET_ADDR			0x6000005c	// WDT-CLEAR: w-only, data ignored
#define TP64XX_DEV_ID_ADDR				0x6000005c	// DEV_ID: r-only
#define TP64XX_WDT_ENABLE_ADDR			0x60000060	// WDT-ENABLE: r/w
#define TP64XX_HM_RQ0_ADDR				0x60000064	// HM_RQ-0/1: r-only, w-clear
#define TP64XX_HM_RQ1_ADDR				0x60000068
#define TP64XX_NMI_SEL_ADDR				0x6000006c	// NMI/MIRQ-0..3 Selectors: r/w
#define TP64XX_EXT_INT4_SEL_ADDR		0x60000070
#define TP64XX_EXT_INT5_SEL_ADDR		0x60000074
#define TP64XX_EXT_INT6_SEL_ADDR		0x60000078
#define TP64XX_EXT_INT7_SEL_ADDR		0x6000007c
		// IOX registers added in the f/w rev.1B and later
#define TP64XX_FLASH_PAGE_ADDR		  	0x60080000	// (f/w rev.1B) r/w: FLASH page selector register
#define TP64XX_XMEM2_LEN_ID_ADDR		0x60080004	// (f/w rev.1B) r-only: FLASH memory capacity ID

		// aliases for compatibility with TP6x Utilities.
#define TP6X_FIFO_STAT_ADDR				TP64XX_FIFO_STAT_ADDR
#define TP6X_AMWREN_ADDR				TP64XX_AMWREN_ADDR
#define TP6X_XMEM_LEN_ID_ADDR			TP64XX_XMEM_LEN_ID_ADDR // TP64xx only with f/w rev.1A-1/2 and later
#define TP6X_XIO_FMT_ADDR				TP64XX_XIO_FMT_ADDR		// TP64xx only
#define TP6X_MH_RQ_ADDR					TP64XX_MH_RQ_ADDR
#define TP6X_PXSX_RESET_ADDR			TP64XX_PXSX_RESET_ADDR
#define TP6X_SYS_STAT_ADDR				TP64XX_SYS_STAT_ADDR
#define	TP6X_WDT_RESET_ADDR				TP64XX_WDT_RESET_ADDR
#define TP6X_DEV_ID_ADDR				TP64XX_DEV_ID_ADDR
#define TP6X_WDT_ENABLE_ADDR			TP64XX_WDT_ENABLE_ADDR
#define TP6X_HM_RQ0_ADDR				TP64XX_HM_RQ0_ADDR
#define TP6X_HM_RQ1_ADDR				TP64XX_HM_RQ1_ADDR
#define TP6X_NMI_SEL_ADDR				TP64XX_NMI_SEL_ADDR
#define TP6X_EXT_INT4_SEL_ADDR			TP64XX_EXT_INT4_SEL_ADDR
#define TP6X_EXT_INT5_SEL_ADDR			TP64XX_EXT_INT5_SEL_ADDR
#define TP6X_EXT_INT6_SEL_ADDR			TP64XX_EXT_INT6_SEL_ADDR
#define TP6X_EXT_INT7_SEL_ADDR			TP64XX_EXT_INT7_SEL_ADDR
#define TP6X_FLASH_PAGE_ADDR		  	TP64XX_FLASH_PAGE_ADDR     // TP64xx only with f/w rev.1B and later
#define TP6X_XMEM2_LEN_ID_ADDR			TP64XX_XMEM2_LEN_ID_ADDR   // TP64xx only with f/w rev.1B and later

			// old API defs, not recommended for usage, provided for compatibility only
#define TP64XX_PXSX_RUN_ADDR			TP64XX_PXSX_RESET_ADDR
#define TP64XX_MIRQ0_SEL_ADDR			TP64XX_EXT_INT4_SEL_ADDR
#define TP64XX_MIRQ1_SEL_ADDR			TP64XX_EXT_INT5_SEL_ADDR
#define TP64XX_MIRQ2_SEL_ADDR			TP64XX_EXT_INT6_SEL_ADDR
#define TP64XX_MIRQ3_SEL_ADDR			TP64XX_EXT_INT7_SEL_ADDR

#define TP6X_PXSX_RUN_ADDR				TP64XX_PXSX_RESET_ADDR
#define TP6X_MIRQ0_SEL_ADDR				TP64XX_EXT_INT4_SEL_ADDR
#define TP6X_MIRQ1_SEL_ADDR				TP64XX_EXT_INT5_SEL_ADDR
#define TP6X_MIRQ2_SEL_ADDR				TP64XX_EXT_INT6_SEL_ADDR
#define TP6X_MIRQ3_SEL_ADDR				TP64XX_EXT_INT7_SEL_ADDR

//================================================================
#endif




//===== recommended TP6x/TP64xx EMIF configuration defs ==========
// Recommended TP6x EMIF configuration settings are provided for
// reference only and are not recommended for direct setting.
// These EMIF settings are automatically set by TP6X_init_dsp_emif()
// extended TP6x API function (see below), which must be invoked from
// user DSP application in the very beginning of MAIN() C-function.
//
// TP6x API Rev.3B:
// ~~~~~~~~~~~~~~~~
//		The EMIF setting values have been updated to match
//		high-speed SBSRAM/SDRAM memory installed onto TORNADO-P6x boards.
//
// TP6x API Rev.3C:
// ~~~~~~~~~~~~~~~~
// 		1. EMIF settings for TP64xx boards have been extended to support f/w rev.1A-1 and rev.1A-2.
//		2. EMIF-B@CE1 area (FLASH) settings for TP64xx boards have been modified to 8-bit ASYNC.


	//-- old TP62/TP67 EMIF settings, which are provided for
	//   compatibility with old software only. These settings will run
	//   on TP6202/TP6203 with limited functionality, and are not compatible
	//   with TP64xx.
#define TP6X_EMIF_GCR_DATA				0x3078      // 1/2 CLK SBSRAM
#define TP6X_EMIF_CE0_DATA				0x31b3c623	// DPRAM, PIOX and UTIL area (stb=6, setup=3,hold=3)
#define TP6X_EMIF_CE1_DATA				0x8638d823	// FLASH area (stb=24, setup=8,hold=3)
#define TP6X_EMIF_CE2_DATA				0x40		// SBSRAM
#define TP6X_EMIF_CE3_DATA				0x30		// SDRAM
#define TP6X_EMIF_SDRAM_DATA			0x7116000	// Trc=70ns, Trp=20ns, Trcd=20ns for Micron -8E SDRAM
#define	TP6X_EMIF_SDRAM_TIMER_DATA		1253		// REFR period is shortest for TP62/TP67

	//-- new EMIF settings for TP62 (200MHz DSP clock, 100MHz EMIF clock)
#define TP62_EMIF_GCR_DATA				0x3078      // 1/2 CLK SBSRAM
#define TP62_EMIF_GCR_SBSRAM0WS_DATA	0x307C      // 1x CLK SBSRAM
#define TP62_EMIF_CE0_DATA				0x31b3c623	// DPRAM, PIOX and UTIL area (stb=6, setup=3,hold=3)
#define TP62_EMIF_CE1_DATA				0x8638d823	// FLASH area (stb=24, setup=8,hold=3)
#define TP62_EMIF_CE2_DATA				0x40		// SBSRAM
#define TP62_EMIF_CE3_DATA				0x30		// SDRAM
#define TP62_EMIF_SDRAM_DATA			0x7116000	// Trc=70ns, Trp=20ns, Trcd=20ns for Micron -8E SDRAM
#define	TP62_EMIF_SDRAM_TIMER_DATA		1500		// REFR period = 64mS/4K_rows/10us

	//-- new EMIF settings for TP67 (167MHz DSP clock, 83.5MHz EMIF clock)
#define TP67_EMIF_GCR_DATA				0x3078      // 1/2 CLK SBSRAM
#define TP67_EMIF_GCR_SBSRAM0WS_DATA	0x307C      // 1x CLK SBSRAM
#define TP67_EMIF_CE0_DATA				0x2162c522	// DPRAM, PIOX and UTIL area (stb=5, setup=2,hold=2)
#define TP67_EMIF_CE1_DATA				0x8638d823	// FLASH area (stb=24, setup=8,hold=3)
#define TP67_EMIF_CE2_DATA				0x40		// SBSRAM
#define TP67_EMIF_CE3_DATA				0x30		// SDRAM
#define TP67_EMIF_SDRAM_DATA			0x7116000	// Trc=70ns, Trp=20ns, Trcd=20ns for Micron -8E SDRAM
#define	TP67_EMIF_SDRAM_TIMER_DATA		1253		// REFR period = 64mS/4K_rows/10us

	//-- TP6202 EMIF settings (250MHz DSP clock, 125MHz EMIF clock)
	//  note!!! EMIF CE2 and CE3 registers shall be configured identically
#define TP6202_EMIF_GCR_DATA			0x3078      // set as default
#define TP6202_EMIF_CE0_DATA			0x31f3c723	// DPRAM, PIOX and UTIL area (stb=7, setup=3,hold=3)
#define TP6202_EMIF_CE1_DATA			0x87b8de23	// FLASH area (stb=30, setup=8,hold=3)
#define TP6202_EMIF_CE3_SDRAM_DATA		0x30		// enable CE3 as SDRAM
#define TP6202_EMIF_CE3_SBSRAM_DATA		0x40		// enable CE3 as SBSRAM (reserved)
#define TP6202_EMIF_CE2_SDRAM_DATA		0x30		// enable CE2 as SDRAM
#define TP6202_EMIF_CE2_SBSRAM_DATA		0x40		// enable CE2 as SBSRAM
#define TP6202_EMIF_SDRAM_DATA			0x7228000	// Trc=70ns, Trp=20ns, Trcd=20ns for Micron -8E SDRAM
#define	TP6202_EMIF_SDRAM_TIMER_DATA	1875		// REFR period = 64mS/4K_rows/8us

	//-- TP6203 EMIF settings (300MHz DSP clock, 150MHz EMIF clock)
	//  note!!! EMIF CE2 and CE3 registers shall be configured identically
#define TP6203_EMIF_GCR_DATA			0x3078      // set as default
#define TP6203_EMIF_CE0_DATA			0x4274c923	// DPRAM, PIOX and UTIL area (stb=9, setup=4,hold=3)
#define TP6203_EMIF_CE1_DATA			0xc93ce423	// FLASH area (stb=36, setup=12,hold=3)
#define TP6203_EMIF_CE2_SDRAM_DATA		0x30		// enable CE2 as SDRAM
#define TP6203_EMIF_CE2_SBSRAM_DATA		0x40		// enable CE2 as SBSRAM
#define TP6203_EMIF_CE3_SDRAM_DATA		0x30		// enable CE3 as SDRAM
#define TP6203_EMIF_CE3_SBSRAM_DATA		0x40		// enable CE3 as SBSRAM (reserved)
#define TP6203_EMIF_SDRAM_DATA			0x7228000	// Trc=60ns, Trp=15ns, Trcd=15ns for Micron 150MHz SDRAM
#define	TP6203_EMIF_SDRAM_TIMER_DATA	2272		// REFR period = 64mS/4K_rows/6.6us


	//-- TP64xx EMIF settings
	//	 Notes:
	//   -------
	//		1. TP64xx f/w rev.1A-1 and rev.1A-2 are supported starting fm TP6x API rev.3C.
	//		2. Starting fm TP6x API rev.3C, the EMIF-B@CE1 area for FLASH memory access is configured as
	//		   the 8-bit ASYNC area for compatibility with future TP64xx board revisions. FLASH must be accessed
	//		   as UNSIGNED CHAR (BYTE) words. TP6x API rev.3B and earlier configured the EMIF-B@CE1 area
	//		   as 16-bit ASYNC and allowed access to 8-bit FLASH as LSB of 16-bit datawords. No modification of
	//		   application code is required in case supplied FLASH programming utilities are used.

		//~~~ common EMIF registers settings for all f/w revisions of TP64xx boards
		// 	  	1. CE-0/1/2 areas are used/valid for EMIF-A, and CE-0/1/3 areas are used/valid for EMIF-B
		// 		2. EMIF-A_clk=100MHz (f/w rev.1A) , 125MHz (f/w rev.1A-1) and 133MHz (f/w rev.1A-2)
#define TP64XX_EMIFA_GCR_DATA			0x0001207c  // default GCR setting
#define TP64XX_EMIFA_CE0_DATA			0xe0		// configure CE0 as 64-bit SBSRAM
#define TP64XX_EMIFA_CE0_SEC_DATA		0x00000002	// CE-0 secondary ext
#define TP64XX_EMIFA_CE1_DATA			0x40		// configure CE0 as 32-bit SDPRAM
#define TP64XX_EMIFA_CE1_SEC_DATA		0x00000002	// CE-0 secondary ext
#define TP64XX_EMIFA_CE2_DATA			0xd0		// configure CE2 as 64-bit SDRAM
#define TP64XX_EMIFA_CE2_SEC_DATA		0x00000002	// CE-2 secondary ext
#define TP64XX_EMIFA_CE3_DATA			0xffffff03	// reserved, leave as default
#define TP64XX_EMIFB_GCR_DATA			0x0001207c  // default GCR setting
#define TP64XX_EMIFB_CE0_DATA_PRESET	0xffffff13	// pre-setup setting of EMIF-B CE-0 space configuration
													// (required to set 16-bit ASYNC space in EMIF-B CE-0 and perform correct control regs decoding for
													//  further configuraion of EMIF-B CE-0 space register upon the f/w revision, DSP speed and silicon rev)
#define TP64XX_EMIFB_CE2_DATA			0xffffff03	// reserved, leave as default

		//~~~ register specific settings upon the f/w revision, DSP speed and SDRAM capacity
			// EMIF settings for TP64xx f/w rev.1A with 600MHz DSP, EMIF-A/B_clk=CPU_CLK/6=100MHz@10ns
#define TP64XX_EMIFA_SDRAM16M_DATA_REV1A_600MHZ			0x63115000	// 128MB: Trc=60ns, Trp=15ns, Trcd=15ns for Micron -8ns SDRAM
#define TP64XX_EMIFA_SDRAM4M_DATA_REV1A_600MHZ			0x57115000	// 32MB:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron -8ns SDRAM
#define TP64XX_EMIFA_SDRAM_EXT_DATA_REV1A_600MHZ		0x00014d29	// SDRAM ext
#define	TP64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A_600MHZ	781			// REFR period = 64mS@8192rows => 7.81us @ 100MHz(10ns)
#define	TP64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A_600MHZ	1562		// REFR period = 64mS@4096rows => 15.62us @ 100MHz(10ns)
#define TP64XX_EMIFB_CE0_DATA_REV1A_600MHZ				0x10d14311	// IOX/SIOX_revC/PCIC area (16-bit ASYNC, stb=3, setup=1,hold=1, TA=1)
#define TP64XX_EMIFB_CE1_DATA_REV1A_600MHZ				0x22924a01	// FLASH area (8-bit async,stb=10, setup=2,hold=1, TA=1)
#define TP64XX_EMIFB_CE3_DATA_REV1A_600MHZ				0x10d14311	// PIOX area (16-bit ASYNC, stb=3, setup=1,hold=1, TA=1)

			// EMIF settings for TP64xx f/w rev.1A-1 with 600/720/1000MHz DSP, EMIF-A/B_clk=external=125MHz@8ns
#define TP64XX_EMIFA_SDRAM16M_DATA_REV1A1				0x63117000	// 16Mx64: Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TP64XX_EMIFA_SDRAM4M_DATA_REV1A1				0x57117000	// 4Mx64:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TP64XX_EMIFA_SDRAM_EXT_DATA_REV1A1				0x00014d29	// SDRAM ext
#define	TP64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A1			976			// REFR period = 64mS@8192rows => 7.81us @ 125MHz(8ns)
#define	TP64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A1			1952		// REFR period = 64mS@4096rows => 15.62us @ 125MHz(8ns)
#define TP64XX_EMIFB_CE0_DATA_REV1A1					0x21124411	// IOX/SIOX_revC/PCICarea (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)
#define TP64XX_EMIFB_CE1_DATA_REV1A1					0x23524d01	// FLASH area (8-bit,stb=13, setup=2,hold=1)
#define TP64XX_EMIFB_CE3_DATA_REV1A1					0x21124411	// PIOX area (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)

			// EMIF settings for E64xx f/w rev.1A-2 with 600/720/1000MHz DSP, EMIF-A/B_clk=external=133MHz@7.5ns
#define TP64XX_EMIFA_SDRAM16M_DATA_REV1A2				0x63117000	// 16Mx64: Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TP64XX_EMIFA_SDRAM4M_DATA_REV1A2				0x57117000	// 4Mx64:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TP64XX_EMIFA_SDRAM_EXT_DATA_REV1A2				0x00014d29	// SDRAM ext
#define	TP64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A2			1041		// REFR period = 64mS@8192rows => 7.81us @ 125MHz(7.5ns)
#define	TP64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A2			2082		// REFR period = 64mS@4096rows => 15.62us @ 125MHz(7.5ns)
#define TP64XX_EMIFB_CE0_DATA_REV1A2					0x21124411	// IOX/SIOX_revC/PCIC area (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)
#define TP64XX_EMIFB_CE1_DATA_REV1A2					0x23d24e01	// FLASH area (8-bit,100ns access, stb=14, setup=2,hold=1, TA=1)
#define TP64XX_EMIFB_CE3_DATA_REV1A2					0x21124411	// PIOX area (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)

			// EMIF settings for E64xx f/w rev.1B with 600/720/1000MHz DSP, EMIF-A/B_clk=external=133MHz@7.5ns
//#define TP64XX_EMIFA_SDRAM16M_DATA_REV1B				0x63117000	// 16Mx64: Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
//#define TP64XX_EMIFA_SDRAM4M_DATA_REV1B					0x57117000	// 4Mx64:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
//#define TP64XX_EMIFA_SDRAM_EXT_DATA_REV1B				0x00014d29	// SDRAM ext
#define TP64XX_EMIFA_SDRAM16M_DATA_REV1B				0x63228000	// 16Mx64 SDRAM: Trc=66ns, Trp=20ns, Trcd=20ns for Micron -75 SDRAM
#define TP64XX_EMIFA_SDRAM4M_DATA_REV1B					0x57228000	// 4Mx32 SDRAM:  Trc=66ns, Trp=20ns, Trcd=20ns for Micron -75 SDRAM
#define TP64XX_EMIFA_SDRAM_EXT_DATA_REV1B				0x00014d2b	// SDRAM ext for Micron -75 SDRAM: Tras=44ns, CAS=3clk, Trrd=15ns, Twr=15ns
#define	TP64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1B			1041		// REFR period = 64mS@8192rows => 7.81us @ 125MHz(7.5ns)
#define	TP64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1B			2082		// REFR period = 64mS@4096rows => 15.62us @ 125MHz(7.5ns)
#define TP64XX_EMIFB_CE0_DATA_REV1B						0x21124411	// IOX/SIOX_revC/PCIC area (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)
#define TP64XX_EMIFB_CE1_DATA_REV1B						0x24125001	// FLASH area (8-bit,120ns access, stb=16, setup=2,hold=1, TA=1)
#define TP64XX_EMIFB_CE3_DATA_REV1B						0x21124411	// PIOX area (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)

//================================================================



//=================== TP6x peripherals defs ======================

//----- DPSEM defs (TP62/TP620x/TP67 only)
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
#define	TP6X_DPSEM_BITMASK					0x1		// DPSEM data bitmask and values
#define	TP6X_DPSEM_OFF						0x1
#define	TP6X_DPSEM_ON						0x0
#endif


//----- data read bitmask for IOX control registers
// IOX registers are valid thru D0..D3 bits for TP6x and D0..D7 bits for TP64xx.
// IOX data register format for TP64xx is upward compatible with TP6x.
// (*) denotes default bit setting on DSP reset condition.
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
#define	TP6X_IOX_DATA_BITMASK				0xf
#endif
#ifdef	__TP6X_SELECT_TP64XX__
#define	TP64XX_IOX_DATA_BITMASK				0xff
#define	TP6X_IOX_DATA_BITMASK				TP64XX_IOX_DATA_BITMASK
#endif

	//-------------- IOX registers bits defs --------------
		//~~~ FIFO STATUS register (r-only)
		//    (includes PCIC FIFO status bits for quick access without need to read of PCIC AGCSTS register)
			// bit masks
#define	TP6X_WFIFO_FULL_BITMASK				1	// (r): PCIC W-FIFO full indicator
#define	TP6X_WFIFO_EMPTY_BITMASK			2	// (r): PCIC W-FIFO empty indicator
#define	TP6X_RFIFO_EMPTY_BITMASK			4	// (r): PCIC R-FIFO empty indicator
#define	TP6X_RFIFO_FULL_BITMASK				8	// (r): PCIC R-FIFO full indicator

			// bit shifts for _USET/_UGET macros
#define	TP6X_WFIFO_FULL_BITSHIFT			0
#define	TP6X_WFIFO_EMPTY_BITSHIFT			1
#define	TP6X_RFIFO_EMPTY_BITSHIFT			2
#define	TP6X_RFIFO_FULL_BITSHIFT			3

			// individual bit values
#define	TP6X_WFIFO_FULL_ON					1
#define	TP6X_WFIFO_FULL_OFF					0
#define	TP6X_WFIFO_EMPTY_ON					2
#define	TP6X_WFIFO_EMPTY_OFF				0
#define	TP6X_RFIFO_EMPTY_ON					4
#define	TP6X_RFIFO_EMPTY_OFF				0
#define	TP6X_RFIFO_FULL_ON					8
#define	TP6X_RFIFO_FULL_OFF					0


		//~~~ AMWREN register (r/w)
		//    (includes PCI-bus mastering enable control)
			// bit masks
#define	TP6X_AMWEN_BITMASK					1	// (r/w): enable control for PCI-bus mastering write transfers
#define	TP6X_AMREN_BITMASK					2	// (r/w): enable control for PCI-bus mastering read transfers
#define	TP6X_AMEN_BITMASK					8	// (r):   enable indicator for PCI-bus mastering, set by host PC s/w

			// bit shifts for _USET/_UGET macros
#define	TP6X_AMWEN_BITSHIFT					0
#define	TP6X_AMREN_BITSHIFT					1
#define	TP6X_AMEN_BITSHIFT					3

			// individual bit values
#define	TP6X_AMWEN_ON						1
#define	TP6X_AMWEN_OFF						0	// (*)
#define	TP6X_AMREN_ON						2
#define	TP6X_AMREN_OFF						0	// (*)
#define	TP6X_AMEN_ON						8
#define	TP6X_AMEN_OFF						0


#ifdef	__TP6X_SELECT_TP64XX__
		//~~~ XIO FMT register (TP64xx only, r/w)
		//    (includes control for PIOX and PCIC data access formats, and C64xx DSP clcok freq ID (f/w rev.1A-1/2 only))
		    // bit masks
#define	TP64XX_PX_FMT_BITMASK				0x3		// (r/w): PIOX data access format control
#define	TP64XX_PCIC_FMT_BITMASK				0x4		// (r/w): PCIC data access format control
#define	TP64XX_DSP_CLK_FREQ_ID_BITMASK		0x30	// f/w rev.1A-1/2 (r): C64xx DSP clock frequency ID code bitmask
				// TP6x API aliases
#define	TP6X_PX_FMT_BITMASK					TP64XX_PX_FMT_BITMASK
#define	TP6X_PCIC_FMT_BITMASK				TP64XX_PCIC_FMT_BITMASK
#define	TP6X_DSP_CLK_FREQ_ID_BITMASK		TP64XX_DSP_CLK_FREQ_ID_BITMASK

			// individual bit values
#define	TP64XX_PX_FMT_W16					0	// (*) 	8-bit and 16-bit cycles, 32-bit cycles are translated into two 16-bit cycles (PIOX/PIOX-16)
#define	TP64XX_PX_FMT_W32					1	// 		32-bit access cycles only (PIOX/PIOX-16)
#define	TP64XX_PX_FMT_MSW16_IGNORED			2	// 		16-bit cycles with MSW ignored (PIOX-16 only)
				// aliases
#define	TP64XX_PX_FMT_PX16					TP64XX_PX_FMT_MSW16_IGNORED
				// TP6x API aliases
#define	TP6X_PX_FMT_W16						TP64XX_PX_FMT_W16
#define	TP6X_PX_FMT_W32						TP64XX_PX_FMT_W32
#define	TP6X_PX_FMT_MSW16_IGNORED			TP64XX_PX_FMT_MSW16_IGNORED

#define	TP64XX_PCIC_FMT_W16					0	// 		8-bit and 16-bit cycles, 32-bit cycles are translated into two 16-bit cycles
#define	TP64XX_PCIC_FMT_W32					4	// (*) 	32-bit/8-bit access cycles only
				// TP6x API aliases
#define	TP6X_PCIC_FMT_W16					TP64XX_PCIC_FMT_W16
#define	TP6X_PCIC_FMT_W32					TP64XX_PCIC_FMT_W32

#define	TP64XX_DSP_CLK_FREQ_ID_600MHZ		0x00	// C64xx DSP with 600MHz clock
#define	TP64XX_DSP_CLK_FREQ_ID_720MHZ		0x10	// C64xx DSP with 720MHz clock
#define	TP64XX_DSP_CLK_FREQ_ID_1000MHZ		0x20	// C64xx DSP with 1GHz clock
				// TP6x API aliases
#define	TP6X_DSP_CLK_FREQ_ID_600MHZ			TP64XX_DSP_CLK_FREQ_ID_600MHZ
#define	TP6X_DSP_CLK_FREQ_ID_720MHZ			TP64XX_DSP_CLK_FREQ_ID_720MHZ
#define	TP6X_DSP_CLK_FREQ_ID_1000MHZ		TP64XX_DSP_CLK_FREQ_ID_1000MHZ
#endif


#ifdef	__TP6X_SELECT_TP64XX__
		//~~~ XMEM LEN ID register (TP64xx with f/w rev.1A-1 and rev.1A-2 only, r)
		//    (includes IDs for on-board SBSRAM/SDRAM/DPRAM memories)
		    // bit masks
#define	TP64XX_SBSRAM_LEN_ID_BITMASK		0x03	// SBSRAM capacity ID bitmask
#define	TP64XX_SDRAM_LEN_ID_BITMASK			0x0c	// SDRAM capacity ID bitmask
#define	TP64XX_DPRAM_LEN_ID_BITMASK			0x30	// DPRAM capacity ID bitmask
				// TP6x aliases
#define	TP6X_SBSRAM_LEN_ID_BITMASK			TP64XX_SBSRAM_LEN_ID_BITMASK
#define	TP6X_SDRAM_LEN_ID_BITMASK			TP64XX_SDRAM_LEN_ID_BITMASK
#define	TP6X_DPRAM_LEN_ID_BITMASK			TP64XX_DPRAM_LEN_ID_BITMASK

		// SBSRAM length ID
#define	TP64XX_SBSRAM_LEN_ID_NONE			0		// SBSRAM is not installed
#define	TP64XX_SBSRAM_LEN_ID_0K				TP64XX_SBSRAM_LEN_ID_NONE		  // alias for nont installed
#define	TP64XX_SBSRAM_LEN_ID_128K			1		// SBSRAM capacity is 128Kx64
#define	TP64XX_SBSRAM_LEN_ID_256K			2		// SBSRAM capacity is 256Kx64
#define	TP64XX_SBSRAM_LEN_ID_512K			3		// SBSRAM capacity is 512Kx64
			// aliases (for length in bytes)
#define	TP64XX_SBSRAM_LEN_ID_0KB			TP64XX_SBSRAM_LEN_ID_0K
#define	TP64XX_SBSRAM_LEN_ID_512KB			TP64XX_SBSRAM_LEN_ID_128K
#define	TP64XX_SBSRAM_LEN_ID_1MB			TP64XX_SBSRAM_LEN_ID_256K
#define	TP64XX_SBSRAM_LEN_ID_2MB			TP64XX_SBSRAM_LEN_ID_512K

			// TP6x API aliases
#define	TP6X_SBSRAM_LEN_ID_NONE				TP64XX_SBSRAM_LEN_ID_NONE
#define	TP6X_SBSRAM_LEN_ID_0K				TP64XX_SBSRAM_LEN_ID_0K
#define	TP6X_SBSRAM_LEN_ID_128K				TP64XX_SBSRAM_LEN_ID_128K
#define	TP6X_SBSRAM_LEN_ID_256K				TP64XX_SBSRAM_LEN_ID_256K
#define	TP6X_SBSRAM_LEN_ID_512K				TP64XX_SBSRAM_LEN_ID_512K
				// aliases (for length in bytes)
#define	TP6X_SBSRAM_LEN_ID_0KB				TP6X_SBSRAM_LEN_ID_0K
#define	TP6X_SBSRAM_LEN_ID_512KB			TP6X_SBSRAM_LEN_ID_128K
#define	TP6X_SBSRAM_LEN_ID_1MB				TP6X_SBSRAM_LEN_ID_256K
#define	TP6X_SBSRAM_LEN_ID_2MB				TP6X_SBSRAM_LEN_ID_512K


		// SDRAM length ID
#define	TP64XX_SDRAM_LEN_ID_NONE			0		// SDRAM is not installed
#define	TP64XX_SDRAM_LEN_ID_0M				TP64XX_SDRAM_LEN_ID_NONE		// alias for not installed
#define	TP64XX_SDRAM_LEN_ID_4M				4		// SDRAM capacity is 4Mx64
#define	TP64XX_SDRAM_LEN_ID_16M				8		// SDRAM capacity is 16Mx64
			// aliases (for length in bytes)
#define	TP64XX_SDRAM_LEN_ID_0MB				TP64XX_SDRAM_LEN_ID_0M
#define	TP64XX_SDRAM_LEN_ID_32MB			TP64XX_SDRAM_LEN_ID_4M
#define	TP64XX_SDRAM_LEN_ID_128MB			TP64XX_SDRAM_LEN_ID_16M

			// TP6x API aliases
#define	TP6X_SDRAM_LEN_ID_NONE				TP64XX_SDRAM_LEN_ID_NONE
#define	TP6X_SDRAM_LEN_ID_0M				TP64XX_SDRAM_LEN_ID_0M
#define	TP6X_SDRAM_LEN_ID_4M				TP64XX_SDRAM_LEN_ID_4M
#define	TP6X_SDRAM_LEN_ID_16M				TP64XX_SDRAM_LEN_ID_16M
				// aliases (for length in bytes)
#define	TP6X_SDRAM_LEN_ID_0MB				TP6X_SDRAM_LEN_ID_0M
#define	TP6X_SDRAM_LEN_ID_32MB				TP6X_SDRAM_LEN_ID_4M
#define	TP6X_SDRAM_LEN_ID_128MB				TP6X_SDRAM_LEN_ID_16M


		// DPRAM length ID
#define	TP64XX_DPRAM_LEN_ID_NONE			0		// DPRAM is not installed
#define	TP64XX_DPRAM_LEN_ID_0K				TP64XX_DPRAM_LEN_ID_NONE		// alias for not installed
#define	TP64XX_DPRAM_LEN_ID_128K			0x10	// DPRAM capacity is 128Kx64
#define	TP64XX_DPRAM_LEN_ID_256K			0x20	// DPRAM capacity is 256Kx64
#define	TP64XX_DPRAM_LEN_ID_512K			0x30	// DPRAM capacity is 512Kx64
			// aliases (for length in bytes)
#define	TP64XX_DPRAM_LEN_ID_0KB				TP64XX_DPRAM_LEN_ID_0K
#define	TP64XX_DPRAM_LEN_ID_512KB			TP64XX_DPRAM_LEN_ID_128K
#define	TP64XX_DPRAM_LEN_ID_1MB				TP64XX_DPRAM_LEN_ID_256K
#define	TP64XX_DPRAM_LEN_ID_2MB				TP64XX_DPRAM_LEN_ID_512K

			// TP6x API aliases
#define	TP6X_DPRAM_LEN_ID_NONE				TP64XX_DPRAM_LEN_ID_NONE
#define	TP6X_DPRAM_LEN_ID_0K				TP64XX_DPRAM_LEN_ID_0K
#define	TP6X_DPRAM_LEN_ID_128K				TP64XX_DPRAM_LEN_ID_128K
#define	TP6X_DPRAM_LEN_ID_256K				TP64XX_DPRAM_LEN_ID_256K
#define	TP6X_DPRAM_LEN_ID_512K				TP64XX_DPRAM_LEN_ID_512K
				// aliases (for length in bytes)
#define	TP6X_DPRAM_LEN_ID_0KB				TP6X_DPRAM_LEN_ID_0K
#define	TP6X_DPRAM_LEN_ID_512KB				TP6X_DPRAM_LEN_ID_128K
#define	TP6X_DPRAM_LEN_ID_1MB				TP6X_DPRAM_LEN_ID_256K
#define	TP6X_DPRAM_LEN_ID_2MB				TP6X_DPRAM_LEN_ID_512K
#endif


		//~~~ WDT enable register (r/w)
#define	TP6X_WDT_ENABLE_BITMASK				1	// (r/w): WDT enable control

			// individual bit values
#define	TP6X_WDT_ENABLE_OFF					0	// (*)
#define	TP6X_WDT_ENABLE_ON					1


		//~~~ SYS STAT register (r-only, r/w for TP6202/TP6203)
		//    (includes system status/configuration)
		    // new bit masks, recommended for usage
#define	TP6X_PCI_PRESENTS_BITMASK			1	// (r): host PCI-bus presence indicator
#define	TP6X_XDEV_ID_EN_BITMASK				2	// (r): extended board ID enable: TP62/TP67 (=0) or TP6202/TP6203/TP64xx/etc (=1)
#define	TP6X_XMEM_SEL_BITMASK				4	// (r/w, TP6202/TP6203): external memory selector: SBSRAM (=0), SDRAM (=1)
#define	TP6X_HOST_MRESET_CONTROL_BITMASK	8	// (r): enable indicator for host control for MRESET
#ifdef	__TP6X_SELECT_TP64XX__
#define	TP64XX_UTOPIA_EN_BITMASK			0x80// (r, TP64xx): indicator for UTOPIA enable
#define	TP64XX_BMODE_BITMASK				0x30// (r, TP64xx): DSP bootmode indicator
#define	TP64XX_XIF_BITMASK					TP64XX_UTOPIA_EN_BITMASK
			// aliases
#define	TP6X_UTOPIA_EN_BITMASK				TP64XX_UTOPIA_EN_BITMASK
#define	TP6X_XIF_BITMASK					TP64XX_UTOPIA_EN_BITMASK
#define	TP6X_BMODE_BITMASK					TP64XX_BMODE_BITMASK
#endif

		    // old API bit masks, not recommended for usage, provided for compatibility only
#define	TP6X_SYS_STAT_PCI_ON_BITMASK		TP6X_PCI_PRESENTS_BITMASK
#define	TP6X_SYS_STAT_XDEV_ID_BITMASK		TP6X_XDEV_ID_EN_BITMASK
#define	TP6X_SYS_STAT_XMEM_BITMASK			TP6X_XMEM_SEL_BITMASK
#define	TP6X_SYS_STAT_HRESET_BITMASK		TP6X_HOST_MRESET_CONTROL_BITMASK

			// bit shifts for _USET/_UGET macros
#define	TP6X_PCI_PRESENTS_BITSHIFT			0
#define	TP6X_XDEV_ID_EN_BITSHIFT			1
#define	TP6X_XMEM_SEL_BITSHIFT				2
#define	TP6X_HOST_MRESET_CONTROL_BITSHIFT	3
#ifdef	__TP6X_SELECT_TP64XX__
#define	TP64XX_UTOPIA_EN_BITSHIFT			7
			// aliases
#define	TP6X_UTOPIA_EN_BITSHIFT				TP64XX_UTOPIA_EN_BITSHIFT
#endif

			// individual bit values
#define	TP6X_PCI_PRESENTS_ON				1		// 		host PCI presents
#define	TP6X_PCI_PRESENTS_OFF				0		// 		host PCI doe not present
#define	TP6X_XDEV_ID_EN_ON					2		// 		extended board ID is enabled (TP6202/TP6203/TP64xx/etc), use XDEV_ID_RG register to identify DSP/board type
#define	TP6X_XDEV_ID_EN_OFF					0		// 		extended board ID is disabled (TP62/TP67), use DSP on-chip CSR register to identify DSP/board type
#define	TP6X_XMEM_SEL_SDRAM					4		// 		external SDRAM is selected for TP6202/TP6203
#define	TP6X_XMEM_SEL_SBSRAM				0		// (*) 	external SBSRAM is selected for TP6202/TP6203
#define	TP6X_HOST_MRESET_CONTROL_ON			8		// 		MRESET is controlled by HOST PC
#define	TP6X_HOST_MRESET_CONTROL_OFF		0		// 		MRESET is not controlled by HOST PC

#ifdef	__TP6X_SELECT_TP64XX__
#define	TP64XX_UTOPIA_EN_ON					0x80	// 		UTOPIA I/F is enabled for TP64xx, McBSP1 port is disabled
#define	TP64XX_UTOPIA_EN_OFF				0		// 		UTOPIA I/F is disabled for TP64xx, McBSP1 port is enabled
			// aliases for UTOPIA I/F selector
#define	TP64XX_MCBSP1_EN_OFF				TP64XX_UTOPIA_EN_ON
#define	TP64XX_MCBSP1_EN_ON					TP64XX_UTOPIA_EN_OFF
#define	TP64XX_XIF_UTOPIA					TP64XX_UTOPIA_EN_ON
#define	TP64XX_XIF_MCBSP1					TP64XX_UTOPIA_EN_OFF

#define	TP64XX_BMODE_NONE					0		// TP64xx DSP botmode: no-boot
#define	TP64XX_BMODE_HPI					0x10	// TP64xx DSP botmode: boot from HPI
#define	TP64XX_BMODE_FLASH					0x20	// TP64xx DSP botmode: boot from FLASH

			// aliases
#define	TP6X_UTOPIA_EN_ON					TP64XX_UTOPIA_EN_ON
#define	TP6X_UTOPIA_EN_OFF					TP64XX_UTOPIA_EN_OFF
#define	TP6X_MCBSP1_EN_OFF					TP64XX_MCBSP1_EN_OFF
#define	TP6X_MCBSP1_EN_ON					TP64XX_MCBSP1_EN_ON
#define	TP6X_XIF_UTOPIA						TP64XX_UTOPIA_EN_ON
#define	TP6X_XIF_MCBSP1						TP64XX_UTOPIA_EN_OFF
#define	TP6X_BMODE_NONE						TP64XX_BMODE_NONE
#define	TP6X_BMODE_HPI						TP64XX_BMODE_HPI
#define	TP6X_BMODE_FLASH					TP64XX_BMODE_FLASH
#endif

			// old API definitions, not recommended for usage, provided for compatibility only
				// ext memory type selectors for TP6202/TP6203
#define	TP6X_XMEM_SBSRAM					TP6X_XMEM_SEL_SBSRAM
#define	TP6X_XMEM_SDRAM						TP6X_XMEM_SEL_SDRAM
				// ext Dev ID register enable
#define	TP6X_XDEV_ID_OFF					TP6X_XDEV_ID_EN_OFF
#define	TP6X_XDEV_ID_ON						TP6X_XDEV_ID_EN_ON


		//~~~ EXT_INT4..7(MIRQ-0..3) and NMI source selector registers
		//    (include source selector bits for corresponding DSP external interrupts)
#define	TP6X_EXT_INT_SEL_OFF				0x0	// (*)	no IRQ source, power-on default

#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
	// TP62xx/TP67 feature XIRQ-0..3 interrupts, which are mapped as the following:
	//	- XIRQ-0 goes to the SIOX-A rev.B IRQ-0 and SIOX-A rev.C IRQ
	//	- XIRQ-1 goes to the SIOX-A rev.B IRQ-1 and SIOX-B rev.C IRQ
	//	- XIRQ-2 goes to the SIOX-A rev.B IRQ-2 and PIOX IRQ-0
	//	- XIRQ-3 goes to the PIOX IRQ-1
#define	TP6X_EXT_INT_SEL_XIRQ0				0x1	// 		TP62/TP620x/TP67: shared SIOX-A rev.B and SIOX-A rev.C IRQ-0
#define	TP6X_EXT_INT_SEL_XIRQ1				0x2	// 		TP62/TP620x/TP67: shared SIOX-A rev.B and SIOX-B rev.C IRQ-1
#define	TP6X_EXT_INT_SEL_XIRQ2				0x3	// 		TP62/TP620x/TP67: shared SIOX-A rev.B and PIOX IRQ-0
#define	TP6X_EXT_INT_SEL_XIRQ3				0x4	// 		TP62/TP620x/TP67: PIOX IRQ-0
				// aliases for shared IRQ selectors for TP62/TP620x/TP67 only
#define	TP6X_EXT_INT_SEL_PXIRQ0				TP6X_EXT_INT_SEL_XIRQ2	// PIOX IRQ-0 maps to the XIRQ-2
#define	TP6X_EXT_INT_SEL_PXIRQ1				TP6X_EXT_INT_SEL_XIRQ3	// PIOX IRQ-1 maps to the XIRQ-3
#define	TP6X_EXT_INT_SEL_SXIRQ0				TP6X_EXT_INT_SEL_XIRQ0
#define	TP6X_EXT_INT_SEL_SXIRQ1				TP6X_EXT_INT_SEL_XIRQ1
#define	TP6X_EXT_INT_SEL_SXIRQ2				TP6X_EXT_INT_SEL_XIRQ2
#endif
#ifdef	__TP6X_SELECT_TP64XX__
#define	TP64XX_EXT_INT_SEL_PXIRQ0			0x1	// 		TP64xx: dedicated PIOX IRQ-0
#define	TP64XX_EXT_INT_SEL_PXIRQ1			0x2	// 		TP64xx: dedicated PIOX IRQ-1
#define	TP64XX_EXT_INT_SEL_PXIRQ2			0x3	// 		TP64xx: dedicated PIOX IRQ-2
#define	TP64XX_EXT_INT_SEL_PXIRQ3			0x4	// 		TP64xx: dedicated PIOX IRQ-3
				// aliases
#define	TP6X_EXT_INT_SEL_PXIRQ0				TP64XX_EXT_INT_SEL_PXIRQ0
#define	TP6X_EXT_INT_SEL_PXIRQ1				TP64XX_EXT_INT_SEL_PXIRQ1
#define	TP6X_EXT_INT_SEL_PXIRQ2				TP64XX_EXT_INT_SEL_PXIRQ2
#define	TP6X_EXT_INT_SEL_PXIRQ3				TP64XX_EXT_INT_SEL_PXIRQ3
#endif
				// DPRAM Host->DSP IRQ location is not supported
				// for TP64xx with 128Kx32 DPRAM only
#if (defined(__TP6X_SELECT_TP62_TP67_TP620X__) || (defined(__TP6X_SELECT_TP64XX__) && defined(__TP6X_DPRAM256K__)))
#define	TP6X_EXT_INT_SEL_MDPIRQ				0x5	// 		H-to-M DPRAM IRQ via corresponding H-to-M IRQ doorbell DPRAM location (TP62/TP620x/TP67, and TP64xx with 256K DPRAM)
#endif
#define	TP6X_EXT_INT_SEL_HM_RQ0				0x6	// 		Host-to-Master #0 IRQ via 4-bit HM_RQ0 doorbell register
#define	TP6X_EXT_INT_SEL_HM_RQ1				0x7	// 		Host-to-Master #1 IRQ via 4-bit HM_RQ1 doorbell register
#define	TP6X_EXT_INT_SEL_PCIC				0x8	// 		IRQ from S5933 PCIC
#define	TP6X_EXT_INT_SEL_WFIFO				0x9	// 		IRQ from W-FIFO (w-fifo is not full)
#define	TP6X_EXT_INT_SEL_RFIFO				0xa	// 		IRQ from R-FIFO (r-fifo is not empty)
#define	TP6X_EXT_INT_SEL_WDT				0xb	// 		IRQ from WDT expired
#define	TP6X_EXT_INT_SEL_RSRV1				0xc	// 		reserved
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
#define	TP6X_EXT_INT_SEL_RSRV2				0xd	// 		reserved for TP62/TP620x/TP67
#define	TP6X_EXT_INT_SEL_RSRV3				0xe	// 		reserved for TP62/TP620x/TP67
#define	TP6X_EXT_INT_SEL_RSRV4				0xf	// 		reserved for TP62/TP620x/TP67
#endif
#ifdef	__TP6X_SELECT_TP64XX__
#define	TP64XX_EXT_INT_SEL_SXIRQ0			0xd	// 		TP64xx: dedicated SIOX IRQ-0
#define	TP64XX_EXT_INT_SEL_SXIRQ1			0xe	// 		TP64xx: dedicated SIOX IRQ-1
#define	TP64XX_EXT_INT_SEL_SXIRQ2			0xf	// 		TP64xx: dedicated SIOX IRQ-2
				// aliases
#define	TP6X_EXT_INT_SEL_SXIRQ0				TP64XX_EXT_INT_SEL_SXIRQ0
#define	TP6X_EXT_INT_SEL_SXIRQ1				TP64XX_EXT_INT_SEL_SXIRQ1
#define	TP6X_EXT_INT_SEL_SXIRQ2				TP64XX_EXT_INT_SEL_SXIRQ2
#endif

				// old API definitions, provided for compatibility only
#define	TP6X_MIRQ_SEL_OFF					TP6X_EXT_INT_SEL_OFF
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
#define	TP6X_MIRQ_SEL_XIRQ0					TP6X_EXT_INT_SEL_XIRQ0
#define	TP6X_MIRQ_SEL_XIRQ1					TP6X_EXT_INT_SEL_XIRQ1
#define	TP6X_MIRQ_SEL_XIRQ2					TP6X_EXT_INT_SEL_XIRQ2
#define	TP6X_MIRQ_SEL_XIRQ3					TP6X_EXT_INT_SEL_XIRQ3
#endif
				// DPRAM Host->DSP IRQ location is not supported
				// for TP64xx with 128Kx32 DPRAM only
#if (defined(__TP6X_SELECT_TP62_TP67_TP620X__) || (defined(__TP6X_SELECT_TP64XX__) && defined(__TP6X_DPRAM256K__)))
#define	TP6X_MIRQ_SEL_MDPIRQ				TP6X_EXT_INT_SEL_MDPIRQ
#endif
#define	TP6X_MIRQ_SEL_HM_RQ0				TP6X_EXT_INT_SEL_HM_RQ0
#define	TP6X_MIRQ_SEL_HM_RQ1				TP6X_EXT_INT_SEL_HM_RQ1
#define	TP6X_MIRQ_SEL_PCIC					TP6X_EXT_INT_SEL_PCIC
#define	TP6X_MIRQ_SEL_WFIFO					TP6X_EXT_INT_SEL_WFIFO
#define	TP6X_MIRQ_SEL_RFIFO					TP6X_EXT_INT_SEL_RFIFO
#define	TP6X_MIRQ_SEL_WDT					TP6X_EXT_INT_SEL_WDT
#define	TP6X_MIRQ_SEL_RSRV1					TP6X_EXT_INT_SEL_RSRV1
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
#define	TP6X_MIRQ_SEL_RSRV2					TP6X_EXT_INT_SEL_RSRV2
#define	TP6X_MIRQ_SEL_RSRV3					TP6X_EXT_INT_SEL_RSRV3
#define	TP6X_MIRQ_SEL_RSRV4					TP6X_EXT_INT_SEL_RSRV4
#endif



		//~~~ PIOX/SIOX Reset Control register (r/w)
		//	 (includes control bits for SIOX and PIOX reset control)
			// bit masks
#define	TP6X_PX_RESET_BITMASK				1	// (r/w): PIOX/PIOX-16 reset control
#define	TP6X_SXA_RESET_BITMASK				2	// (r/w): SIOX-A rev.C reset control
#define	TP6X_SXB_RESET_BITMASK				4	// (r/w): SIOX-B rev.C reset control

			// bit shifts for _USET/_UGET macros
#define	TP6X_PX_RESET_BITSHIFT				0
#define	TP6X_SXA_RESET_BITSHIFT				1
#define	TP6X_SXB_RESET_BITSHIFT				2

			// individual bit values
#define	TP6X_PX_RESET_ON					0	// (*)
#define	TP6X_PX_RESET_OFF					1

#define	TP6X_SXA_RESET_ON					0	// (*)
#define	TP6X_SXA_RESET_OFF					2

#define	TP6X_SXB_RESET_ON					0	// (*)
#define	TP6X_SXB_RESET_OFF					4

			// unified RUN/RESET defs
#define	TP6X_PXSX_RESET_OFF					1
#define	TP6X_PXSX_RESET_ON					0


				// old API definitions, provided for compatibility only
#define	TP6X_PX_RUN_BITMASK					TP6X_PX_RESET_BITMASK
#define	TP6X_PX_RESET						TP6X_PX_RESET_ON
#define	TP6X_PX_RUN							TP6X_PX_RESET_OFF
#define	TP6X_SXA_RUN_BITMASK				TP6X_SXA_RESET_BITMASK
#define	TP6X_SXA_RESET						TP6X_SXA_RESET_ON
#define	TP6X_SXA_RUN						TP6X_SXA_RESET_OFF
#define	TP6X_SXB_RUN_BITMASK				TP6X_SXB_RESET_BITMASK
#define	TP6X_SXB_RESET						TP6X_SXB_RESET_ON
#define	TP6X_SXB_RUN						TP6X_SXB_RESET_OFF
#define	TP6X_PXSX_RUN						TP6X_PXSX_RESET_OFF
#define	TP6X_PXSX_RESET						TP6X_PXSX_RESET_ON


	//~~~ TP6X_DEV_ID register (r)
	//	  (contains extended device ID data, which must be used for TP6202/TP6203/TP64xx/etc
	//	   boards in order to identify board type via DSP user s/w.	TP62/TP67 boards can be
	//	   recognized via bit XDEV_ID_EN==0 (D1) of SYS_STAT register. TP64xx boards also
	//	   provide board/firmwire revision ID code, which is available via bits D4..D7.)
#define	TP6X_DEV_ID_BITMASK					0xf

#define	TP6202_DEV_ID						2		// TP6202 device ID
#define	TP6203_DEV_ID						3		// TP6203 device ID
#define	TP6414_DEV_ID						8		// TP6414 device ID
#define	TP6415_DEV_ID						9		// TP6415 device ID
#define	TP6416_DEV_ID						0xa 	// TP6416 device ID

#ifdef	__TP6X_SELECT_TP64XX__
		// TP64xx boards provide board&f/w revision ID in bits #4..7 of DEV_ID_RG register
#define	TP64XX_REV_ID_BITMASK				0xf0
			// aliases
#define	TP6X_REV_ID_BITMASK					TP64XX_REV_ID_BITMASK

#define	TP64XX_REV_ID_REV1A					0x10	// TP64xx only: f/w rev.1A (600MHz DSP, EMIF clock 100MHz, no DSP clock ID, no XMEM_LEN_ID_RG register)
#define	TP64XX_REV_ID_REV1A1				0x20	// TP64xx only: f/w rev.1A-1 (600/720/1000MHz DSP, EMIF clock 125MHz, DSP clock ID, XMEM_LEN_ID_RG register)
#define	TP64XX_REV_ID_REV1A2				0x30	// TP64xx only: f/w rev.1A-2 (600/720/1000MHz DSP, EMIF clock 133MHz, DSP clock ID, XMEM_LEN_ID_RG register)

#define	TP64XX_REV_ID_REV1B					0x40	// TP64xx only: f/w rev.1B (h/w rev.1B, 600/720/1000MHz DSP, EMIF clock 133MHz, DSP clock ID,
													// 		  		XMEM_LEN_ID_RG/FLAH_LEN_ID_RG/FLASH_PAGE_RG registers)
			// aliases with TP6x API
#define	TP6X_REV_ID_REV1A					TP64XX_REV_ID_REV1A
#define	TP6X_REV_ID_REV1A1					TP64XX_REV_ID_REV1A1
#define	TP6X_REV_ID_REV1A2					TP64XX_REV_ID_REV1A2
#define	TP6X_REV_ID_REV1B					TP64XX_REV_ID_REV1B
			// aliases with old API
#define	TP64XX_REV_ID_1A					TP64XX_REV_ID_REV1A
#define	TP64XX_REV_ID_1A1					TP64XX_REV_ID_REV1A1
#define	TP64XX_REV_ID_1A2					TP64XX_REV_ID_REV1A2
#define	TP64XX_REV_ID_1B					TP64XX_REV_ID_REV1B
#define	TP6X_REV_ID_1A						TP64XX_REV_ID_REV1A
#define	TP6X_REV_ID_1A1						TP64XX_REV_ID_REV1A1
#define	TP6X_REV_ID_1A2						TP64XX_REV_ID_REV1A2
#define	TP6X_REV_ID_1B						TP64XX_REV_ID_REV1B
#endif



#ifdef	__TP6X_SELECT_TP64XX__
		//~~~ XMEM2_LEN_ID register (TP64xx with f/w rev.1B and later only, r)
		//    (includes IDs for on-board FLASH memory)
		    // bit masks
#define	TP64XX_FLASH_LEN_ID_BITMASK		 	0x0f	// FLASH capacity ID bitmask
				// TP6x aliases
#define	TP6X_FLASH_LEN_ID_BITMASK			TP64XX_FLASH_LEN_ID_BITMASK

		// FLASH length ID
#define	TP64XX_FLASH_LEN_ID_NONE			0		// FLASH memory is not installed
#define	TP64XX_FLASH_LEN_ID_0K				TP64XX_FLASH_LEN_ID_NONE   // alias for not installed
#define	TP64XX_FLASH_LEN_ID_0M				TP64XX_FLASH_LEN_ID_NONE   // alias for not installed
#define	TP64XX_FLASH_LEN_ID_512K			1		// FLASH capacity is 512Kx8
#define	TP64XX_FLASH_LEN_ID_1M				2		// FLASH capacity is 1Mx8
#define	TP64XX_FLASH_LEN_ID_2M				3		// FLASH capacity is 2Mx8
#define	TP64XX_FLASH_LEN_ID_4M				4		// FLASH capacity is 4Mx8
#define	TP64XX_FLASH_LEN_ID_8M				5		// FLASH capacity is 8Mx8
#define	TP64XX_FLASH_LEN_ID_16M				6		// FLASH capacity is 16Mx8
#define	TP64XX_FLASH_LEN_ID_32M				7		// FLASH capacity is 32Mx8
#define	TP64XX_FLASH_LEN_ID_64M				8		// FLASH capacity is 64Mx8

			// aliases (for length in bytes)
#define	TP64XX_FLASH_LEN_ID_0KB				TP64XX_FLASH_LEN_ID_0K
#define	TP64XX_FLASH_LEN_ID_0MB				TP64XX_FLASH_LEN_ID_0M
#define	TP64XX_FLASH_LEN_ID_512KB			TP64XX_FLASH_LEN_ID_512K
#define	TP64XX_FLASH_LEN_ID_1MB				TP64XX_FLASH_LEN_ID_1M
#define	TP64XX_FLASH_LEN_ID_2MB				TP64XX_FLASH_LEN_ID_2M
#define	TP64XX_FLASH_LEN_ID_4MB				TP64XX_FLASH_LEN_ID_4M
#define	TP64XX_FLASH_LEN_ID_8MB				TP64XX_FLASH_LEN_ID_8M
#define	TP64XX_FLASH_LEN_ID_16MB			TP64XX_FLASH_LEN_ID_16M
#define	TP64XX_FLASH_LEN_ID_32MB			TP64XX_FLASH_LEN_ID_32M
#define	TP64XX_FLASH_LEN_ID_64MB			TP64XX_FLASH_LEN_ID_64M

			// TP6x API aliases
#define	TP6X_FLASH_LEN_ID_NONE				TP64XX_FLASH_LEN_ID_NONE
#define	TP6X_FLASH_LEN_ID_0K				TP64XX_FLASH_LEN_ID_0K
#define	TP6X_FLASH_LEN_ID_0M				TP64XX_FLASH_LEN_ID_0M
#define	TP6X_FLASH_LEN_ID_512K				TP64XX_FLASH_LEN_ID_512K
#define	TP6X_FLASH_LEN_ID_1M				TP64XX_FLASH_LEN_ID_1M
#define	TP6X_FLASH_LEN_ID_2M				TP64XX_FLASH_LEN_ID_2M
#define	TP6X_FLASH_LEN_ID_4M				TP64XX_FLASH_LEN_ID_4M
#define	TP6X_FLASH_LEN_ID_8M				TP64XX_FLASH_LEN_ID_8M
#define	TP6X_FLASH_LEN_ID_16M				TP64XX_FLASH_LEN_ID_16M
#define	TP6X_FLASH_LEN_ID_32M				TP64XX_FLASH_LEN_ID_32M
#define	TP6X_FLASH_LEN_ID_64M				TP64XX_FLASH_LEN_ID_64M
				// aliases (for length in bytes)
#define	TP6X_FLASH_LEN_ID_0KB				TP64XX_FLASH_LEN_ID_0K
#define	TP6X_FLASH_LEN_ID_0MB				TP64XX_FLASH_LEN_ID_0M
#define	TP6X_FLASH_LEN_ID_512KB				TP64XX_FLASH_LEN_ID_512K
#define	TP6X_FLASH_LEN_ID_1MB				TP64XX_FLASH_LEN_ID_1M
#define	TP6X_FLASH_LEN_ID_2MB				TP64XX_FLASH_LEN_ID_2M
#define	TP6X_FLASH_LEN_ID_4MB				TP64XX_FLASH_LEN_ID_4M
#define	TP6X_FLASH_LEN_ID_8MB				TP64XX_FLASH_LEN_ID_8M
#define	TP6X_FLASH_LEN_ID_16MB				TP64XX_FLASH_LEN_ID_16M
#define	TP6X_FLASH_LEN_ID_32MB				TP64XX_FLASH_LEN_ID_32M
#define	TP6X_FLASH_LEN_ID_64MB				TP64XX_FLASH_LEN_ID_64M


		//~~~ FLASH PAGE register (TP64xx with f/w rev.1B and later only, r/w)
		    // bit masks
#define	TP64XX_FLASH_PAGE_BITMASK		 	0x3f	// FLASH page selector bitmask
				// TP6x aliases
#define	TP6X_FLASH_PAGE_BITMASK				TP64XX_FLASH_PAGE_BITMASK

#endif

//================================================================


//================== Access macros & Functions ===================
/*
 	TP6x utilities rev.3A and later offer unified utility macros/functions synopsis:
	  	- TP6X_set_..._rg(..)    - sets the control register contents
		- TP6X_get_..._rg()      - returns contents of control register
  		- TP6X_set_...(..)       - sets bit of control register (using particular bit set value)
	  	- TP6X_get_...()         - returns bit state of control register (using particular bit set value)
  		- TP6X_uset_...(ON/OFF)  - sets bit of control register using unified ON/OFF values
	  	- TP6X_uget_...()        - returns bit state of control register using unified ON/OFF values
 	Aliases for register read-back macros and for some write macros
 	are provided for syntax compatibility with previous release TORNADO-P6x DSP utilities.

 	Notes:
 	------
 	1.	Definition of some macros is defined upon the TORNADO-Pxxx board type. For example,
 	   	DPSEM access macros are defined for TP62/TP620x/TP67 boards only, and are not defined
 		for TP64xx boards. In order to force C-compiler to generate error message when he
 		will find platform incompatible macro in the user code, this macro is defined to the
 		'MACRO_IS_UNDEFINED' value in this header value for incompatible TORNADO-Pxxx platform(s).
 		'MACRO_IS_UNDEFINED' value itself is not defined, so it will cause C-compiler to generate
 		and error message.

	2.	Some API v.2A and earlier macros confuse with the macros synopsis of new API v.3A
		and later. These confusing macros are:
						TP6X_set_amwen(v)
						TP6X_set_amren(v)
						TP6X_amwen()
						TP6X_amren()
						TP6X_amen()
		The confusion is that in the new API v.3A macros synopsis, these macros are used to
		set/get specific bit values not aligned to unified ON/OFF values, whereas in the old
		API v.2A these macros are equivalent to the corresponding _USET_/_UGET_ macros of API v.3A.
		In order to resolve this confusion without any need for modification of old user code, which
		has been written using TP6x API v.2A and earlier utilities, the run-time compiler key
		__TP6X_API_V2_COMPATIBILITY__ must be used. If __TP6X_API_V2_COMPATIBILITY__
		run-time compiler key is defined in user code prior inclusion of this header file,
		then API v.2A and earlier confusing macros will be provided for 100% functional
		compatibility with API v.2A utilities, otherwise these confusing macros will be provided
		in accordance with new API v.3A synopsis. Default is NO COMPATIBILITY with API v.2A
		utilities.
*/


//------------ I/O data type selector for different TP6x DSP systems ---------
/*
 	TP62/TP620x/TP67 DSP systems have all I/O peripherals externally connected via
	32-bit data bus, so they can be accessed as 32-bit UNSIGNED,
	16-bit UNSIGNED SHORT and 8-bit UNSIGNED CHAR.

 	TP64xx DSP systems have all I/O peripherals (IOX registers, PCIC, PIOX and
	SIOX_revC I/F sites) connected to EMIF-B, so these can be generally accessed
	as 8/16/32-bit variable. However, when accessing IOX registers and SIOX_revC I/F site
	using 32-bit UNSIGNED INT, the access cycle is splitted into two 16-bit cycles
	with the LSW cycle only containing valid data. 16-bit MSW cycle is just ignore and
	actually results in the time loss, so it can be omitted. The recommended vars for
	accessing IOX registers and SIOX_revC I/F site are either 8-/16-bit UNSIGNED INT.
	Instead, PIOX and PCIC accesses provide on-board resources to buffer 16-bit LSW/MSW
	data in order to perform true 32-bit access cycles to PCIC h/w and PIOX DCM.

	The declared __TP6X_IO_DATA_TYPE__ is used to unify below macros and functions for accessing
	the IOX registers, and to make particular selection at the run-time compilation time.

	The declared __TP6X_FLASH_DATA_TYPE__ has been introduces since TP6x API rev.3C and is used
	to unify below macros and functions for accessing the FLASH memory, and to make particular
	selection at the run-time compilation time.
*/

#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
typedef	unsigned							__TP6X_IOX_DATA_TYPE__;
typedef	unsigned							__TP6X_FLASH_DATA_TYPE__;	// introduced since API rev.3C
#endif

#ifdef	__TP6X_SELECT_TP64XX__
typedef	unsigned short						__TP6X_IOX_DATA_TYPE__;
typedef	unsigned char						__TP6X_FLASH_DATA_TYPE__;	// introduced since API rev.3C
#endif
//----------------------------------------------------------------------------


//------- Macros for DPRAM/DPSEM areas access
// Supplied DPRAM address must be in bytes count.
#define TP6X_set_dpram(a,v)					(((unsigned *)TP6X_DPRAM_SADDR)[a%TP6X_DPRAM_LEN])=(v)
#define TP6X_get_dpram(a)					(((unsigned *)TP6X_DPRAM_SADDR)[a%TP6X_DPRAM_LEN])
	// aliases for old API compatibility
#define TP6X_dpram(a)						TP6X_get_dpram(a)

	// DPRAM Host->DSP and DSP->Host IRQ access macros
	// Note, that these DPRAM locations are not supported for TP64xx with 128Kx32 DPRAM, and
	// accessing these DPRAM location will not result in any DPRAM IRQ asserted.
	// User application must check for the DPRAM length using the TP6X_get_dpram_len() macros
	// before accessing the Host->DSP and DSP->Host IRQ DPRAM locations.
#define TP6X_set_dpram_mh_rq(v)				(*(unsigned *) TP6X_DPRAM_MH_RQ_ADDR)=(v)
#define TP6X_get_dpram_mh_rq()				(*(unsigned *) TP6X_DPRAM_MH_RQ_ADDR)
#define TP6X_set_dpram_hm_rq(v)				(*(unsigned *) TP6X_DPRAM_HM_RQ_ADDR)=(v)
#define TP6X_get_dpram_hm_rq()				(*(unsigned *) TP6X_DPRAM_HM_RQ_ADDR)
		// aliases for old API compatibility
#define TP6X_dpram_mh_rq()					TP6X_get_dpram_mh_rq()
#define TP6X_dpram_hm_rq()					TP6X_get_dpram_hm_rq()

	// DPSEM access functions are supported for TP62/TP620x/TP67 only
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
#define TP6X_set_dpsem(n,v)					(*(unsigned *) (TP6X_DPSEM_SADDR+((n%8)*4))) = ((v)&TP6X_DPSEM_BITMASK)
#define TP6X_get_dpsem(n)					((*(unsigned *) (TP6X_DPSEM_SADDR+((n%8)*4))) &TP6X_DPSEM_BITMASK)
#else
	// have compiler to generate error message for TP64xx platform
#define TP6X_set_dpsem(n,v)					MACRO_IS_UNDEFINED
#define TP6X_get_dpsem(n)					MACRO_IS_UNDEFINED
#endif
		// aliases for old API compatibility
#define TP6X_dpsem(n)						TP6X_get_dpsem(n)



//------- Macros for IOX registers access
	//~~~ DSP->Host request
#define TP6X_set_mh_rq()					(*(__TP6X_IOX_DATA_TYPE__ *) TP6X_MH_RQ_ADDR)=0
		// direct bit set/get macros
#define	TP6X_SET_MH_RQ						TP6X_set_mh_rq()

	//~~~ WDR reset control
#define TP6X_set_wdt_reset()				(*(__TP6X_IOX_DATA_TYPE__ *) TP6X_WDT_RESET_ADDR)=0
		// alias for compatibility with old API
#define TP6X_wdt_reset()					TP6X_set_wdt_reset()
		// direct bit-specific set/get macros
#define	TP6X_SET_WDT_RESET					TP6X_set_wdt_reset()

	//~~~ WDR enable control
#define TP6X_set_wdt_enable_rg(v)			(*(__TP6X_IOX_DATA_TYPE__ *) TP6X_WDT_ENABLE_ADDR)=((__TP6X_IOX_DATA_TYPE__)(v))
#define TP6X_get_wdt_enable_rg()			((*(__TP6X_IOX_DATA_TYPE__ *) TP6X_WDT_ENABLE_ADDR) & TP6X_IOX_DATA_BITMASK)
		// aliases
#define TP6X_wdt_enable_rg()				TP6X_get_wdt_enable_rg()

		// dedicated bit-specific macros with V = {TP6X_WDT_ENABLE_ON,TP6X_WDT_ENABLE_OFF}
#define TP6X_set_wdt_enable(v)				TP6X_set_wdt_enable_rg(v)
#define TP6X_get_wdt_enable()				(TP6X_get_wdt_enable_rg() & TP6X_WDT_ENABLE_BITMASK)
		// aliases for compatibility with old API
#define TP6X_wdt_enable()					TP6X_get_wdt_enable()

		// Unified macros with V = {TP6X_ON,TP6X_OFF}
#define	TP6X_uset_wdt_enable(v)				TP6X_set_wdt_enable(v)
#define	TP6X_uget_wdt_enable()				TP6X_get_wdt_enable()

		// direct bit-specific set/get macros
#define	TP6X_SET_WDT_ENABLE_OFF				TP6X_set_wdt_enable(TP6X_WDT_ENABLE_OFF)
#define	TP6X_SET_WDT_ENABLE_ON				TP6X_set_wdt_enable(TP6X_WDT_ENABLE_ON)
#define	TP6X_WDT_ENABLE_IS_OFF				(TP6X_get_wdt_enable()==TP6X_WDT_ENABLE_OFF)
#define	TP6X_WDT_ENABLE_IS_ON				(TP6X_get_wdt_enable()==TP6X_WDT_ENABLE_ON)


	//~~~ AMWREN register
#define TP6X_set_amwren_rg(v)				(*(__TP6X_IOX_DATA_TYPE__ *) TP6X_AMWREN_ADDR)=((__TP6X_IOX_DATA_TYPE__)(v))
#define TP6X_get_amwren_rg()				((*(__TP6X_IOX_DATA_TYPE__ *) TP6X_AMWREN_ADDR) & TP6X_IOX_DATA_BITMASK)
		// aliases for compatibility with old API
#define TP6X_set_amwren(v)					TP6X_set_amwren_rg(v)
#define TP6X_amwren()						TP6X_get_amwren_rg()
#define TP6X_amwren_rg()					TP6X_get_amwren_rg()

		// dedicated bit macros with V = {TP6X_AMWEN_ON,TP6X_AMWEN_OFF,..}
#define	TP6X_set_amwen_(v)					TP6X_set_amwren_rg((TP6X_get_amwren_rg() & ~TP6X_AMWEN_BITMASK) | ((v) & TP6X_AMWEN_BITMASK))
#define	TP6X_set_amren_(v)					TP6X_set_amwren_rg((TP6X_get_amwren_rg() & ~TP6X_AMREN_BITMASK) | ((v) & TP6X_AMREN_BITMASK))
#define TP6X_get_amwen()					(TP6X_get_amwren_rg() & TP6X_AMWEN_BITMASK)
#define TP6X_get_amren()					(TP6X_get_amwren_rg() & TP6X_AMREN_BITMASK)
#define TP6X_get_amen()						(TP6X_get_amwren_rg() & TP6X_AMEN_BITMASK)

		// unified bit macros with with V = {TP6X_ON,TP6X_OFF}
#define	TP6X_uset_amwen(v)					TP6X_set_amwen_(v)
#define	TP6X_uset_amren(v)					TP6X_set_amren_(v<<TP6X_AMREN_BITSHIFT)
#define TP6X_uget_amwen()					TP6X_get_amwen()
#define TP6X_uget_amren()					(TP6X_get_amren() >>TP6X_AMREN_BITSHIFT)
#define TP6X_uget_amen()					(TP6X_get_amen() >>TP6X_AMEN_BITSHIFT)

		// direct bit-specific set/get macros
#define	TP6X_SET_AMWEN_ON					TP6X_set_amwen_(TP6X_AMWEN_ON)
#define	TP6X_SET_AMWEN_OFF					TP6X_set_amwen_(TP6X_AMWEN_OFF)
#define	TP6X_SET_AMREN_ON					TP6X_set_amren_(TP6X_AMREN_ON)
#define	TP6X_SET_AMREN_OFF					TP6X_set_amren_(TP6X_AMREN_OFF)

#define	TP6X_AMWEN_IS_ON					(TP6X_get_amwen()==TP6X_AMWEN_ON)
#define	TP6X_AMWEN_IS_OFF					(TP6X_get_amwen()==TP6X_AMWEN_OFF)
#define	TP6X_AMREN_IS_ON					(TP6X_get_amren()==TP6X_AMREN_ON)
#define	TP6X_AMREN_IS_OFF					(TP6X_get_amren()==TP6X_AMREN_OFF)
#define	TP6X_AMEN_IS_ON						(TP6X_get_amen()==TP6X_AMEN_ON)
#define	TP6X_AMEN_IS_OFF					(TP6X_get_amen()==TP6X_AMEN_OFF)

			// aliases for compatibility with old API
#ifdef __TP6X_API_V2_COMPATIBILITY__
				// API v.2A functional compatible macros
#define	TP6X_set_amwen(v)					TP6X_uset_amwen(v)
#define	TP6X_set_amren(v)					TP6X_uset_amren(v)
#define TP6X_amwen()						TP6X_uget_amwen()
#define TP6X_amren()						TP6X_uget_amren()
#define TP6X_amen()							TP6X_uget_amen()
#else
				// API v.2A functional non-compatible macros (default)
#define	TP6X_set_amwen(v)					TP6X_set_amwen_(v)
#define	TP6X_set_amren(v)					TP6X_set_amren_(v)
#define TP6X_amwen()						TP6X_get_amwen()
#define TP6X_amren()						TP6X_get_amren()
#define TP6X_amen()							TP6X_get_amen()
#endif



	//~~~ PCIC FIFO status rg
#define TP6X_get_fifo_stat_rg()				((*(__TP6X_IOX_DATA_TYPE__ *) TP6X_FIFO_STAT_ADDR) & TP6X_IOX_DATA_BITMASK)
		// aliases
#define TP6X_fifo_stat()					TP6X_get_fifo_stat_rg()
#define TP6X_fifo_stat_rg()					TP6X_get_fifo_stat_rg()

		// dedicated bit macros with V = {TP6X_WFIFO_FULL_ON,...}
#define TP6X_get_wfifo_full()				(TP6X_get_fifo_stat_rg() & TP6X_WFIFO_FULL_BITMASK)
#define TP6X_get_wfifo_empty()				(TP6X_get_fifo_stat_rg() & TP6X_WFIFO_EMPTY_BITMASK)
#define TP6X_get_rfifo_full()				(TP6X_get_fifo_stat_rg() & TP6X_RFIFO_FULL_BITMASK)
#define TP6X_get_rfifo_empty()				(TP6X_get_fifo_stat_rg() & TP6X_RFIFO_EMPTY_BITMASK)
			// aliases
#define TP6X_wfifo_full()					TP6X_get_wfifo_full()
#define TP6X_wfifo_empty()					TP6X_get_wfifo_empty()
#define TP6X_rfifo_full()					TP6X_get_rfifo_full()
#define TP6X_rfifo_empty()					TP6X_get_rfifo_empty()

		// unified bit macros with with V = {TP6X_ON,TP6X_OFF}
#define TP6X_uget_wfifo_full()				TP6X_get_wfifo_full()
#define TP6X_uget_wfifo_empty()				(TP6X_get_wfifo_empty() >> TP6X_WFIFO_EMPTY_BITSHIFT)
#define TP6X_uget_rfifo_full()				(TP6X_get_rfifo_full() >> TP6X_RFIFO_EMPTY_BITSHIFT)
#define TP6X_uget_rfifo_empty()				(TP6X_get_rfifo_empty() >> TP6X_RFIFO_FULL_BITSHIFT)

		// direct bit-specific set/get macros
#define	TP6X_WFIFO_FULL_IS_ON				(TP6X_get_wfifo_full()==TP6X_WFIFO_FULL_ON)
#define	TP6X_WFIFO_FULL_IS_OFF				(TP6X_get_wfifo_full()==TP6X_WFIFO_FULL_OFF)
#define	TP6X_WFIFO_EMPTY_IS_ON				(TP6X_get_wfifo_empty()==TP6X_WFIFO_EMPTY_ON)
#define	TP6X_WFIFO_EMPTY_IS_OFF				(TP6X_get_wfifo_empty()==TP6X_WFIFO_EMPTY_OFF)

#define	TP6X_RFIFO_FULL_IS_ON				(TP6X_get_rfifo_full()==TP6X_RFIFO_FULL_ON)
#define	TP6X_RFIFO_FULL_IS_OFF				(TP6X_get_rfifo_full()==TP6X_RFIFO_FULL_OFF)
#define	TP6X_RFIFO_EMPTY_IS_ON				(TP6X_get_rfifo_empty()==TP6X_RFIFO_EMPTY_ON)
#define	TP6X_RFIFO_EMPTY_IS_OFF				(TP6X_get_rfifo_empty()==TP6X_RFIFO_EMPTY_OFF)



	//~~~ SYS STATUS register
#define TP6X_get_sys_stat_rg()				((*(__TP6X_IOX_DATA_TYPE__ *) TP6X_SYS_STAT_ADDR) & TP6X_IOX_DATA_BITMASK)
#define TP6X_set_sys_stat_rg(v)				(*(__TP6X_IOX_DATA_TYPE__ *) TP6X_SYS_STAT_ADDR)=((__TP6X_IOX_DATA_TYPE__)(v))
		// aliases for old API compatibility
#define TP6X_set_sys_stat(v)				TP6X_set_sys_stat_rg(v)
#define TP6X_sys_stat()						TP6X_get_sys_stat_rg()
#define TP6X_sys_stat_rg()					TP6X_get_sys_stat_rg()

		// dedicated bit macros with V = {TP6X_PCI_PRESENTS_ON,...}
#define TP6X_get_pci_presents()				(TP6X_get_sys_stat_rg() & TP6X_PCI_PRESENTS_BITMASK)
#define TP6X_get_xdev_id_en()				(TP6X_get_sys_stat_rg() & TP6X_XDEV_ID_EN_BITMASK)
#define TP6X_get_host_mreset_control()		(TP6X_get_sys_stat_rg() & TP6X_HOST_MRESET_CONTROL_BITMASK)
#define TP6X_get_xmem_sel()					(TP6X_get_sys_stat_rg() & TP6X_XMEM_SEL_BITMASK)
#define TP6X_set_xmem_sel(v)				TP6X_set_sys_stat_rg(v)
#ifdef	__TP6X_SELECT_TP64XX__
#define TP6X_get_utopia_en()				(TP6X_get_sys_stat_rg() & TP64XX_UTOPIA_EN_BITMASK)
#define TP6X_get_bmode()					(TP6X_get_sys_stat_rg() & TP64XX_BMODE_BITMASK)
#endif
			// aliases for old API compatibility
#define TP6X_sys_stat_pci_on()				TP6X_get_pci_presents()
#define TP6X_sys_stat_hmreset()				TP6X_get_host_mreset_control()
#define TP6X_sys_stat_xmem() 				TP6X_get_xmem_sel()
#define TP6X_sys_stat_xdev_id() 			TP6X_get_xdev_id_en()
#define TP6X_pci_on()						TP6X_get_pci_presents()
#define TP6X_hmreset()						TP6X_get_host_mreset_control()
#define TP6X_xmem() 						TP6X_get_xmem_sel()
#define TP6X_xdev_id()						TP6X_get_xdev_id_en()
#define TP6X_set_xmem(v)					TP6X_set_xmem_sel(v)
#define TP6X_pci_presents()					TP6X_get_pci_presents()
#define TP6X_host_mreset_control()			TP6X_get_host_mreset_control()
#define TP6X_xdev_id_en()					TP6X_get_xdev_id_en()
#define TP6X_xmem_sel()						TP6X_get_xmem_sel()
#ifdef	__TP6X_SELECT_TP64XX__
#define TP6X_utopia_en()					TP6X_get_utopia_en()
#define TP6X_bmode()						TP6X_get_bmode()
#endif

		// unified bit macros with with V = {TP6X_ON,TP6X_OFF}
#define TP6X_uget_pci_presents()			TP6X_get_pci_presents()
#define TP6X_uget_host_mreset_control()		(TP6X_get_host_mreset_control()	>> TP6X_HOST_MRESET_CONTROL_BITSHIFT)
#define TP6X_uget_xdev_id_en()				(TP6X_get_xdev_id_en()>>TP6X_XDEV_ID_EN_BITSHIFT)
#ifdef	__TP6X_SELECT_TP64XX__
#define TP6X_uget_utopia_en()				(TP6X_get_utopia_en()>>TP6X_UTOPIA_EN_BITSHIFT)
#endif

		// direct bit-specific set/get macros
#define	TP6X_PCI_PRESENTS_IS_ON				(TP6X_get_pci_presents()==TP6X_PCI_PRESENTS_ON)
#define	TP6X_PCI_PRESENTS_IS_OFF			(TP6X_get_pci_presents()==TP6X_PCI_PRESENTS_OFF)
#define	TP6X_HOST_MRESET_CONTROL_IS_ON		(TP6X_get_host_mreset_control()==TP6X_HOST_MRESET_CONTROL_ON)
#define	TP6X_HOST_MRESET_CONTROL_IS_OFF		(TP6X_get_host_mreset_control()==TP6X_HOST_MRESET_CONTROL_OFF)
#define TP6X_XDEV_ID_EN_IS_ON				(TP6X_get_xdev_id_en()==TP6X_XDEV_ID_EN_ON)
#define TP6X_XDEV_ID_EN_IS_OFF				(TP6X_get_xdev_id_en()==TP6X_XDEV_ID_EN_OFF)
#define TP6X_XMEM_SEL_IS_SBSRAM				(TP6X_get_xmem_sel()==TP6X_XMEM_SEL_SBSRAM)
#define TP6X_XMEM_SEL_IS_SDRAM				(TP6X_get_xmem_sel()==TP6X_XMEM_SEL_SDRAM)
#define TP6X_SET_XMEM_SEL_SBSRAM			TP6X_set_xmem_sel(TP6X_XMEM_SEL_SBSRAM)
#define TP6X_SET_XMEM_SEL_SDRAM				TP6X_set_xmem_sel(TP6X_XMEM_SEL_SDRAM)
#ifdef	__TP6X_SELECT_TP64XX__
#define TP6X_UTOPIA_EN_IS_OFF				(TP6X_get_utopia_en()==TP64XX_UTOPIA_EN_OFF)
#define TP6X_UTOPIA_EN_IS_ON				(TP6X_get_utopia_en()==TP64XX_UTOPIA_EN_ON)
#define TP6X_BMODE_IS_NONE					(TP6X_get_bmode()==TP64xX_BMODE_NONE)
#define TP6X_BMODE_IS_FLASH					(TP6X_get_bmode()==TP64XX_BMODE_FLASH)
#define TP6X_BMODE_IS_HPI					(TP6X_get_bmode()==TP64XX_BMODE_HPI)
			// aliases
#define TP6X_MCBSP1_EN_IS_ON				TP6X_UTOPIA_EN_IS_OFF
#define TP6X_MCBSP1_EN_IS_OFF				TP6X_UTOPIA_EN_IS_ON
#define TP6X_XIF_IS_MCBSP1					TP6X_UTOPIA_EN_IS_OFF
#define TP6X_XIF_IS_UTOPIA					TP6X_UTOPIA_EN_IS_ON
#endif

		// aliases for old API compatibility
#define TP6X_set_xmem_sbsram()				TP6X_set_xmem_sel(TP6X_XMEM_SBSRAM)
#define TP6X_set_xmem_sdram()				TP6X_set_xmem_sel(TP6X_XMEM_SDRAM)


	//~~~ Host->DSP request #0/#1 registers
	// (no unified macros are available)
#define TP6X_get_hm_rq0_rg()				((*(__TP6X_IOX_DATA_TYPE__ *) TP6X_HM_RQ0_ADDR) & TP6X_IOX_DATA_BITMASK)
#define TP6X_get_hm_rq1_rg()				((*(__TP6X_IOX_DATA_TYPE__ *) TP6X_HM_RQ1_ADDR) & TP6X_IOX_DATA_BITMASK)
#define TP6X_clr_hm_rq0_rg()				(*(__TP6X_IOX_DATA_TYPE__ *) TP6X_HM_RQ0_ADDR)=0
#define TP6X_clr_hm_rq1_rg()				(*(__TP6X_IOX_DATA_TYPE__ *) TP6X_HM_RQ1_ADDR)=0
		// aliases for old API compatibility
#define TP6X_hm_rq0()						TP6X_get_hm_rq0_rg()
#define TP6X_hm_rq1()						TP6X_get_hm_rq1_rg()
#define TP6X_hm_rq0_rg()					TP6X_get_hm_rq0_rg()
#define TP6X_hm_rq1_rg()					TP6X_get_hm_rq1_rg()
#define TP6X_clr_hm_rq0()					TP6X_clr_hm_rq0_rg()
#define TP6X_clr_hm_rq1()					TP6X_clr_hm_rq1_rg()

		// direct set/get macros
#define TP6X_CLR_HM_RQ0						TP6X_clr_hm_rq0_rg()
#define TP6X_CLR_HM_RQ1						TP6X_clr_hm_rq1_rg()


	//~~~ PIOX/SIOX reset control IOX register
#define TP6X_set_pxsx_reset_rg(v)			(*(__TP6X_IOX_DATA_TYPE__ *)  TP6X_PXSX_RESET_ADDR)=((__TP6X_IOX_DATA_TYPE__)(v))
#define TP6X_get_pxsx_reset_rg()			((*(__TP6X_IOX_DATA_TYPE__ *) TP6X_PXSX_RESET_ADDR) & TP6X_IOX_DATA_BITMASK)
		// aliases
#define TP6X_set_pxsx_reset(v)				TP6X_set_pxsx_reset_rg(v)
#define TP6X_pxsx_reset()					TP6X_get_pxsx_reset_rg()
#define TP6X_pxsx_reset_rg()				TP6X_get_pxsx_reset_rg()

		// dedicated bit-specific macros with V = {TP6X_{PX/SXA/SXB}_RESET_ON,TP6X_{PX/SXA/SXB}_RESET_OFF}
#define TP6X_set_px_reset(v)				TP6X_set_pxsx_reset_rg((TP6X_get_pxsx_reset_rg() & ~TP6X_PX_RESET_BITMASK) | ((v) & TP6X_PX_RESET_BITMASK))
#define TP6X_get_px_reset()					(TP6X_get_pxsx_reset_rg() & TP6X_PX_RESET_BITMASK)
#define TP6X_set_sxa_reset(v)				TP6X_set_pxsx_reset_rg((TP6X_get_pxsx_reset_rg() & ~TP6X_SXA_RESET_BITMASK) | ((v) & TP6X_SXA_RESET_BITMASK))
#define TP6X_get_sxa_reset()				(TP6X_get_pxsx_reset_rg() & TP6X_SXA_RESET_BITMASK)
#define TP6X_set_sxb_reset(v)				TP6X_set_pxsx_reset_rg((TP6X_get_pxsx_reset_rg() & ~TP6X_SXB_RESET_BITMASK) | ((v) & TP6X_SXB_RESET_BITMASK))
#define TP6X_get_sxb_reset()				(TP6X_get_pxsx_reset_rg() & TP6X_SXB_RESET_BITMASK)

		// Unified macros with V = {TP6X_PXSX_RESET_ON,TP6X_PXSX_RESET_OFF}
#define	TP6X_uset_px_reset(v)				TP6X_set_px_reset(v)
#define	TP6X_uget_px_reset()				TP6X_get_px_reset()
#define	TP6X_uset_sxa_reset(v)				TP6X_set_sxa_reset((v<<TP6X_SXA_RESET_BITSHIFT))
#define	TP6X_uget_sxa_reset()				(TP6X_get_sxa_reset()>>TP6X_SXA_RESET_BITSHIFT)
#define	TP6X_uset_sxb_reset(v)				TP6X_set_sxb_reset((v<<TP6X_SXB_RESET_BITSHIFT))
#define	TP6X_uget_sxb_reset()				(TP6X_get_sxb_reset()>>TP6X_SXB_RESET_BITSHIFT)

		// direct bit-specific set/get macros
#define	TP6X_SET_PX_RESET_OFF				TP6X_set_px_reset(TP6X_PX_RESET_OFF)
#define	TP6X_SET_PX_RESET_ON				TP6X_set_px_reset(TP6X_PX_RESET_ON)
#define	TP6X_PX_RESET_IS_OFF				(TP6X_get_px_reset()==TP6X_PX_RESET_OFF)
#define	TP6X_PX_RESET_IS_ON					(TP6X_get_px_reset()==TP6X_PX_RESET_ON)

#define	TP6X_SET_SXA_RESET_OFF				TP6X_set_sxa_reset(TP6X_SXA_RESET_OFF)
#define	TP6X_SET_SXA_RESET_ON				TP6X_set_sxa_reset(TP6X_SXA_RESET_ON)
#define	TP6X_SXA_RESET_IS_OFF				(TP6X_get_sxa_reset()==TP6X_SXA_RESET_OFF)
#define	TP6X_SXA_RESET_IS_ON				(TP6X_get_sxa_reset()==TP6X_SXA_RESET_ON)

#define	TP6X_SET_SXB_RESET_OFF				TP6X_set_sxb_reset(TP6X_SXB_RESET_OFF)
#define	TP6X_SET_SXB_RESET_ON				TP6X_set_sxb_reset(TP6X_SXB_RESET_ON)
#define	TP6X_SXB_RESET_IS_OFF				(TP6X_get_sxb_reset()==TP6X_SXB_RESET_OFF)
#define	TP6X_SXB_RESET_IS_ON				(TP6X_get_sxb_reset()==TP6X_SXB_RESET_ON)

#define	TP6X_SET_PXSX_RESET_OFF				TP6X_set_pxsx_reset_rg(TP6X_PX_RESET_OFF|TP6X_SXA_RESET_OFF|TP6X_SXB_RESET_OFF)
#define	TP6X_SET_PXSX_RESET_ON				TP6X_set_pxsx_reset_rg(TP6X_PX_RESET_ON |TP6X_SXA_RESET_ON |TP6X_SXB_RESET_ON)

		// aliases for compatibility with old API
		// Note, that old bit set macros are equivalent to new _USET_/_UGET_ macros (!!!).
		// Old PX/SX reset (run) control macros used TP6X_PXSX_RESET/TP6X_PXSX_RUN unified defs.
#define TP6X_set_pxsx_run(v)				TP6X_set_pxsx_reset_rg(v)
#define TP6X_pxsx_run()						TP6X_get_pxsx_reset_rg()
#define TP6X_set_px_run(v)					TP6X_uset_px_reset(v)
#define TP6X_px_run()						TP6X_uget_px_reset()
#define TP6X_set_sxa_run(v)					TP6X_uset_sxa_reset(v)
#define TP6X_sxa_run()						TP6X_uget_sxa_reset()
#define TP6X_set_sxb_run(v)					TP6X_uset_sxb_reset(v)
#define TP6X_sxb_run()						TP6X_uget_sxb_reset()


	//~~~ device ID register (also revision ID register for TP64xx boards)
#define TP6X_get_dev_id_rg()				((*(__TP6X_IOX_DATA_TYPE__ *) TP6X_DEV_ID_ADDR) & TP6X_IOX_DATA_BITMASK)

		// dedicated bit specific macros
#define TP6X_get_dev_id()					(TP6X_get_dev_id_rg() & TP6X_DEV_ID_BITMASK)
#ifdef	__TP6X_SELECT_TP64XX__
#define TP6X_get_rev_id()					(TP6X_get_dev_id_rg() & TP6X_REV_ID_BITMASK)
#endif
			// aliases with old API
#define TP6X_dev_id()						TP6X_get_dev_id()
#define TP6X_dev_id_rg()					TP6X_get_dev_id_rg()
#define TP6X_rev_id()						TP6X_get_rev_id()

		// direct board ID macros
		// (a variety of board ID combinations are provided in order to detect board family)
#define TP62_BOARD							(TP6X_XDEV_ID_EN_IS_OFF && ((CSR&0x02000000)==0))
#define TP67_BOARD							(TP6X_XDEV_ID_EN_IS_OFF && ((CSR&0x02000000)!=0))
#define TP62_TP67_BOARD						TP6X_XDEV_ID_EN_IS_OFF
#define TP6202_BOARD						(TP6X_XDEV_ID_EN_IS_ON && (TP6X_get_dev_id()==TP6202_DEV_ID))
#define TP6203_BOARD						(TP6X_XDEV_ID_EN_IS_ON && (TP6X_get_dev_id()==TP6203_DEV_ID))
#define TP6202_TP6203_BOARD					(TP6202_BOARD || TP6203_BOARD)
#define TP620X_BOARD						TP6202_TP6203_BOARD
#define TP62_TP620X_TP67_BOARD				(TP62_TP67_BOARD || TP6202_TP6203_BOARD)
#define TP6414_BOARD						(TP6X_XDEV_ID_EN_IS_ON && (TP6X_get_dev_id()==TP6414_DEV_ID))
#define TP6415_BOARD						(TP6X_XDEV_ID_EN_IS_ON && (TP6X_get_dev_id()==TP6415_DEV_ID))
#define TP6416_BOARD						(TP6X_XDEV_ID_EN_IS_ON && (TP6X_get_dev_id()==TP6416_DEV_ID))
#define TP6415_TP6416_BOARD					(TP6415_BOARD || TP6416_BOARD)
#define TP64XX_BOARD						(TP6414_BOARD || TP6415_BOARD || TP6416_BOARD)

			// revision ID macros are supported for TP64xx boards only
#ifdef	__TP6X_SELECT_TP64XX__
#define TP64XX_REV_ID_IS_REV1A				(TP6X_get_rev_id()==TP64XX_REV_ID_REV1A)
#define TP64XX_REV_ID_IS_REV1A1				(TP6X_get_rev_id()==TP64XX_REV_ID_REV1A1)
#define TP64XX_REV_ID_IS_REV1A2				(TP6X_get_rev_id()==TP64XX_REV_ID_REV1A2)
#define TP64XX_REV_ID_IS_REV1B				(TP6X_get_rev_id()==TP64XX_REV_ID_REV1B)
				// aliases with TP6x API
#define TP6X_REV_ID_IS_REV1A				TP64XX_REV_ID_IS_REV1A
#define TP6X_REV_ID_IS_REV1A1				TP64XX_REV_ID_IS_REV1A1
#define TP6X_REV_ID_IS_REV1A2				TP64XX_REV_ID_IS_REV1A2
#define TP6X_REV_ID_IS_REV1B				TP64XX_REV_ID_IS_REV1B
				// aliases with API rev.3B and earlier
#define TP64XX_REV_ID_IS_1A					TP64XX_REV_ID_IS_REV1A
#define TP64XX_REV_ID_IS_1A1				TP64XX_REV_ID_IS_REV1A1
#define TP64XX_REV_ID_IS_1A2				TP64XX_REV_ID_IS_REV1A2
#define TP64XX_REV_ID_IS_1B					TP64XX_REV_ID_IS_REV1B
#define TP6X_REV_ID_IS_1A					TP64XX_REV_ID_IS_REV1A
#define TP6X_REV_ID_IS_1A1					TP64XX_REV_ID_IS_REV1A1
#define TP6X_REV_ID_IS_1A2					TP64XX_REV_ID_IS_REV1A2
#define TP6X_REV_ID_IS_1B					TP64XX_REV_ID_IS_REV1B

			// combined deviceID/revisionID macros for TP64xx boards
#define TP64XX_REV1A_BOARD					(TP64XX_BOARD && TP64XX_REV_ID_IS_1A)
#define TP64XX_REV1A1_BOARD					(TP64XX_BOARD && TP64XX_REV_ID_IS_1A1)
#define TP64XX_REV1A2_BOARD					(TP64XX_BOARD && TP64XX_REV_ID_IS_1A2)
#define TP64XX_REV1A1_REV1A2_BOARD			(TP64XX_REV1A1_BOARD || TP64XX_REV1A2_BOARD)
#define TP64XX_REV1A_REV1A1_REV1A2_BOARD	(TP64XX_REV1A_BOARD  || TP64XX_REV1A1_BOARD || TP64XX_REV1A2_BOARD)
#define TP64XX_REV1B_BOARD					(TP64XX_BOARD && TP64XX_REV_ID_IS_1B)


				// combinatorial macros to check for the presence of the XMEM_LEN_ID and FLASH_LEN_ID registers and of the DSP clock ID
				// notes:
				//		- SBSRAM/SDRAM/DPRAM/FLASH mem ID availability macros have been added since s/w utilities rev.4A
#define TP64XX_DSP_CLK_ID_IS_AVAILABLE			!TP64XX_REV_ID_IS_REV1A

#define TP64XX_XMEM_LEN_ID_IS_AVAILABLE			!TP64XX_REV_ID_IS_REV1A
#define TP64XX_SBSRAM_LEN_ID_IS_AVAILABLE		TP64XX_XMEM_LEN_ID_IS_AVAILABLE
#define TP64XX_SDRAM_LEN_ID_IS_AVAILABLE		TP64XX_XMEM_LEN_ID_IS_AVAILABLE
#define TP64XX_DPRAM_LEN_ID_IS_AVAILABLE		TP64XX_XMEM_LEN_ID_IS_AVAILABLE

#define TP64XX_XMEM2_LEN_ID_IS_AVAILABLE		TP64XX_REV_ID_IS_REV1B
#define TP64XX_FLASH_LEN_ID_IS_AVAILABLE		TP64XX_XMEM2_LEN_ID_IS_AVAILABLE

#define TP64XX_DSP_CLK_ID_IS_NOT_AVAILABLE	   	!TP64XX_DSP_CLK_ID_IS_AVAILABLE

#define TP64XX_XMEM_LEN_ID_IS_NOT_AVAILABLE	   	!TP64XX_XMEM_LEN_ID_IS_AVAILABLE
#define TP64XX_SBSRAM_LEN_ID_IS_NOT_AVAILABLE	!TP64XX_SBSRAM_LEN_ID_IS_AVAILABLE
#define TP64XX_SDRAM_LEN_ID_IS_NOT_AVAILABLE	!TP64XX_SDRAM_LEN_ID_IS_AVAILABLE
#define TP64XX_DPRAM_LEN_ID_IS_NOT_AVAILABLE	!TP64XX_DPRAM_LEN_ID_IS_AVAILABLE

#define TP64XX_XMEM2_LEN_ID_IS_NOT_AVAILABLE   	!TP64XX_XMEM2_LEN_ID_IS_AVAILABLE
#define TP64XX_FLASH_LEN_ID_IS_NOT_AVAILABLE   	!TP64XX_FLASH_LEN_ID_IS_AVAILABLE

						// TP6x compatible macros
#define TP6X_DSP_CLK_ID_IS_AVAILABLE			TP64XX_DSP_CLK_ID_IS_AVAILABLE

#define TP6X_XMEM_LEN_ID_IS_AVAILABLE			TP64XX_XMEM_LEN_ID_IS_AVAILABLE
#define TP6X_SBSRAM_LEN_ID_IS_AVAILABLE			TP64XX_SBSRAM_LEN_ID_IS_AVAILABLE
#define TP6X_SDRAM_LEN_ID_IS_AVAILABLE			TP64XX_SDRAM_LEN_ID_IS_AVAILABLE
#define TP6X_DPRAM_LEN_ID_IS_AVAILABLE			TP64XX_DPRAM_LEN_ID_IS_AVAILABLE

#define TP6X_XMEM2_LEN_ID_IS_AVAILABLE			TP64XX_XMEM2_LEN_ID_IS_AVAILABLE
#define TP6X_FLASH_LEN_ID_IS_AVAILABLE			TP64XX_FLASH_LEN_ID_IS_AVAILABLE

#define TP6X_DSP_CLK_ID_IS_NOT_AVAILABLE		TP64XX_DSP_CLK_ID_IS_NOT_AVAILABLE

#define TP6X_XMEM_LEN_ID_IS_NOT_AVAILABLE		TP64XX_XMEM_LEN_ID_IS_NOT_AVAILABLE
#define TP6X_SBSRAM_LEN_ID_IS_NOT_AVAILABLE		TP64XX_SBSRAM_LEN_ID_IS_NOT_AVAILABLE
#define TP6X_SDRAM_LEN_ID_IS_NOT_AVAILABLE		TP64XX_SDRAM_LEN_ID_IS_NOT_AVAILABLE
#define TP6X_DPRAM_LEN_ID_IS_NOT_AVAILABLE		TP64XX_DPRAM_LEN_ID_IS_NOT_AVAILABLE

#define TP6X_XMEM2_LEN_ID_IS_NOT_AVAILABLE  	TP64XX_XMEM2_LEN_ID_IS_NOT_AVAILABLE
#define TP6X_FLASH_LEN_ID_IS_NOT_AVAILABLE  	TP64XX_FLASH_LEN_ID_IS_NOT_AVAILABLE

#endif



	//~~~ PIOX/PCIC data access format register (TP64xx only)
	//    No unified _USET_/_UGET_ macros are provided.
#ifdef	__TP6X_SELECT_TP64XX__
#define TP6X_set_xio_fmt_rg(v)				(*(__TP6X_IOX_DATA_TYPE__ *)  TP64XX_XIO_FMT_ADDR)=((__TP6X_IOX_DATA_TYPE__)(v))
#define TP6X_get_xio_fmt_rg()				((*(__TP6X_IOX_DATA_TYPE__ *) TP64XX_XIO_FMT_ADDR) & TP6X_IOX_DATA_BITMASK)
		// aliases
#define TP6X_set_xio_fmt(v)					TP6X_set_xio_fmt_rg(v)
#define TP6X_xio_fmt()						TP6X_get_xio_fmt_rg()
#define TP6X_xio_fmt_rg()					TP6X_get_xio_fmt_rg()

		// dedicated bit-specific macros with V = {TP64XX_PX_FMT_W16...}
#define TP6X_set_px_fmt(v)					TP6X_set_xio_fmt_rg((TP6X_get_xio_fmt_rg() & ~TP64XX_PX_FMT_BITMASK) | ((v) & TP64XX_PX_FMT_BITMASK))
#define TP6X_get_px_fmt()					(TP6X_get_xio_fmt_rg() & TP64XX_PX_FMT_BITMASK)
#define TP6X_set_pcic_fmt(v)				TP6X_set_xio_fmt_rg((TP6X_get_xio_fmt_rg() & ~TP64XX_PCIC_FMT_BITMASK) | ((v) & TP64XX_PCIC_FMT_BITMASK))
#define TP6X_get_pcic_fmt()					(TP6X_get_xio_fmt_rg() & TP64XX_PCIC_FMT_BITMASK)

		// direct bit-specific set/get macros
#define	TP6X_SET_PX_FMT_W16					TP6X_set_px_fmt(TP64XX_PX_FMT_W16)
#define	TP6X_SET_PX_FMT_W32					TP6X_set_px_fmt(TP64XX_PX_FMT_W32)
#define	TP6X_SET_PX_FMT_MSW16_IGNORED		TP6X_set_px_fmt(TP64XX_PX_FMT_MSW16_IGNORED)
#define	TP6X_PX_FMT_IS_W16					(TP6X_get_px_fmt()==TP6X_PX_FMT_W16)
#define	TP6X_PX_FMT_IS_W32					(TP6X_get_px_fmt()==TP6X_PX_FMT_W32)
#define	TP6X_PX_FMT_IS_MSW16_IGNORED		(TP6X_get_px_fmt()==TP6X_PX_FMT_MSW16_IGNORED)
#define	TP6X_SET_PCIC_FMT_W16				TP6X_set_pcic_fmt(TP64XX_PCIC_FMT_W16)
#define	TP6X_SET_PCIC_FMT_W32				TP6X_set_pcic_fmt(TP64XX_PCIC_FMT_W32)
#define	TP6X_PCIC_FMT_IS_W16				(TP6X_get_pcic_fmt()==TP6X_PCIC_FMT_W16)
#define	TP6X_PCIC_FMT_IS_W32				(TP6X_get_pcic_fmt()==TP6X_PCIC_FMT_W32)
			// aliases
#define	TP6X_SET_PX_FMT_PX16				TP6X_SET_PX_FMT_MSW16_IGNORED
#define	TP6X_PX_FMT_IS_PX16					TP6X_PX_FMT_IS_MSW16_IGNORED

		// DSP clock frequency ID read macros (TP64xx f/w rev.1A-1 and rev.1A-2)
		// This macros can be also used with TP64xx f/w rev.1A, which will return 600MHz DSP ID
		// for this f/w, and this match the issue that this f/w has been used with 600MHz DSP only.
#define TP6X_get_dsp_clk_freq_id()			(TP6X_get_xio_fmt_rg() & TP64XX_DSP_CLK_FREQ_ID_BITMASK)

			// DSP clock ID check macros
#define TP64XX_DSP_CLK_FREQ_IS_600MHZ		(TP64XX_REV_ID_IS_REV1A || ((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_dsp_clk_freq_id()==TP64XX_DSP_CLK_FREQ_ID_600MHZ)))
#define TP64XX_DSP_CLK_FREQ_IS_720MHZ		((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_dsp_clk_freq_id()==TP64XX_DSP_CLK_FREQ_ID_720MHZ))
#define TP64XX_DSP_CLK_FREQ_IS_1000MHZ		((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_dsp_clk_freq_id()==TP64XX_DSP_CLK_FREQ_ID_1000MHZ))
			// aliases with TP6x API
#define TP6X_DSP_CLK_FREQ_IS_600MHZ			TP64XX_DSP_CLK_FREQ_IS_600MHZ
#define TP6X_DSP_CLK_FREQ_IS_720MHZ			TP64XX_DSP_CLK_FREQ_IS_720MHZ
#define TP6X_DSP_CLK_FREQ_IS_1000MHZ		TP64XX_DSP_CLK_FREQ_IS_1000MHZ
#endif


	//---- universal macro, which returns DSP clock frequency (floating point value in Hz) upon the board type and DSP clock frequency ID
	// This macros is different for TP6x boards with C620x/C6701 DSPs and for TP64xx boards with C64xx DSPs.
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
#define	TP6X_dsp_clk_freq()				(TP62_BOARD ? TP62_DSP_CLK_FREQ:\
											(TP67_BOARD ? TP67_DSP_CLK_FREQ:\
												(TP6202_BOARD ? TP6202_DSP_CLK_FREQ:TP6203_DSP_CLK_FREQ)))
#endif
#ifdef	__TP6X_SELECT_TP64XX__
#define	TP6X_dsp_clk_freq()				(TP64XX_DSP_CLK_FREQ_IS_600MHZ ? TP64XX_DSP_CLK_FREQ_600MHZ:\
											( TP64XX_DSP_CLK_FREQ_IS_720MHZ ? TP64XX_DSP_CLK_FREQ_720MHZ:TP64XX_DSP_CLK_FREQ_1000MHZ))
#endif



	//---- SBSRAM/SDRAM/DPRAM capacity ID macros (TP64xx boards with f/w rev.1A-1 and rev.1A-2 only)
	//     Notes:
	//		1. For TP64xx f/w rev.1A boards, the TP6X_sdram_len() macros returns the SDRAM length
	//		   upon the setting of the  __TP6X_SDRAM128M__ compiler key.
	//		2. For TP64xx f/w rev.1A boards w/o XMEM_LEN_ID register, the TP6X_sbsram_len() macros
	//		   always returns the 512Kx32 SBSRAM length.
	//		3. For TP64xx f/w rev.1A boards w/o XMEM_LEN_ID register, the TP6X_dpram_len() macros
	//		   always returns the 128Kx32 DPRAM length.
	//		4. For TP64xx rev.1A-1/2 boards, the TP6X_sdram_len(), TP6X_sbsram_len() and TP6X_dpram_len()
	//		   macros return the SDRAM/SBSRAM/DPRAM  length upon the corresponding field of XMEM_LEN_ID register.
	//		5. For rev.1A boards, reading the XMEM_LEN_ID register always returns the '0' values for both
	//		   SDRAM_LEN_ID/SBSRAM_LEN_ID/DPRAM_LEN_ID fields, that correspond to the not-installed
	//		   SBSRAM/SDRAM/DPRAM memories.

#ifdef	__TP6X_SELECT_TP64XX__

	// XMEM_LEN_ID register is available for f/w rev.1A-1 and rev.1A-2 only (!!!)
#define TP6X_get_xmem_len_id_rg()			((*(__TP6X_IOX_DATA_TYPE__ *) TP64XX_XMEM_LEN_ID_ADDR) & TP6X_IOX_DATA_BITMASK)

		// dedicated bit specific macros
#define TP6X_get_sbsram_len_id()			(TP6X_get_xmem_len_id_rg() & TP6X_SBSRAM_LEN_ID_BITMASK)
#define TP6X_get_sdram_len_id()				(TP6X_get_xmem_len_id_rg() & TP6X_SDRAM_LEN_ID_BITMASK)
#define TP6X_get_dpram_len_id()				(TP6X_get_xmem_len_id_rg() & TP6X_DPRAM_LEN_ID_BITMASK)

		// condition check macros (for use in IF() and other conditional operators)
			//	For TP64xx f/w rev.1A boards, the SBSRAM length condition check macros return the true value
			//  for the 512Kx32 SBSRAM length only
#define TP6X_SBSRAM_LEN_IS_0K				((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_sbsram_len_id()==TP6X_SBSRAM_LEN_ID_0K))
#define TP6X_SBSRAM_LEN_IS_128K				((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_sbsram_len_id()==TP6X_SBSRAM_LEN_ID_128K))
#define TP6X_SBSRAM_LEN_IS_256K				((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_sbsram_len_id()==TP6X_SBSRAM_LEN_ID_256K))
#define TP6X_SBSRAM_LEN_IS_512K				(TP64XX_REV_ID_IS_REV1A || ((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_sbsram_len_id()==TP6X_SBSRAM_LEN_ID_512K)))
#define TP6X_SBSRAM_IS_NOT_INSTALLED		TP6X_SBSRAM_LEN_IS_0K
#define TP6X_SBSRAM_IS_INSTALLED			!TP6X_SBSRAM_IS_NOT_INSTALLED

			//	For TP64xx f/w rev.1A boards, the SDRAM length condition check macros return the true value for the 4M and 16M length
			//	upon the setting of the __TP6X_SDRAM128M__ compiler key.
#define TP6X_SDRAM_LEN_IS_0M				((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_sdram_len_id()==TP6X_SDRAM_LEN_ID_0M))

#ifdef	__TP6X_SDRAM128M__
#define TP6X_SDRAM_LEN_IS_4M				((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_sdram_len_id()==TP6X_SDRAM_LEN_ID_4M))
#define TP6X_SDRAM_LEN_IS_16M				(TP64XX_REV_ID_IS_REV1A || ((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_sdram_len_id()==TP6X_SDRAM_LEN_ID_16M)))
#else
#define TP6X_SDRAM_LEN_IS_4M				(TP64XX_REV_ID_IS_REV1A || ((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_sdram_len_id()==TP6X_SDRAM_LEN_ID_4M)))
#define TP6X_SDRAM_LEN_IS_16M				((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_sdram_len_id()==TP6X_SDRAM_LEN_ID_16M))
#endif

#define TP6X_SDRAM_IS_NOT_INSTALLED			TP6X_SDRAM_LEN_IS_0M
#define TP6X_SDRAM_IS_INSTALLED				!TP6X_SDRAM_IS_NOT_INSTALLED


			//	For TP64xx f/w rev.1A boards, the DPRAM length condition check macros return the true
			//  value for 128Kx32 DPRAM length only.
#define TP6X_DPRAM_LEN_IS_0K				((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_dpram_len_id()==TP6X_DPRAM_LEN_ID_0K))
#define TP6X_DPRAM_LEN_IS_128K				(TP64XX_REV_ID_IS_REV1A || ((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_dpram_len_id()==TP6X_DPRAM_LEN_ID_128K)))
#define TP6X_DPRAM_LEN_IS_256K				((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_dpram_len_id()==TP6X_DPRAM_LEN_ID_256K))
#define TP6X_DPRAM_LEN_IS_512K				((!TP64XX_REV_ID_IS_REV1A) && (TP6X_get_dpram_len_id()==TP6X_DPRAM_LEN_ID_512K))
#define TP6X_DPRAM_IS_NOT_INSTALLED			TP6X_DPRAM_LEN_IS_0K
#define TP6X_DPRAM_IS_INSTALLED				!TP6X_DPRAM_IS_NOT_INSTALLED


			// aliases for length in bytes
#define TP6X_SBSRAM_LEN_IS_0KB				TP6X_SBSRAM_LEN_IS_0K
#define TP6X_SBSRAM_LEN_IS_1MB				TP6X_SBSRAM_LEN_IS_128K
#define TP6X_SBSRAM_LEN_IS_2MB				TP6X_SBSRAM_LEN_IS_256K
#define TP6X_SBSRAM_LEN_IS_4MB				TP6X_SBSRAM_LEN_IS_512K

#define TP6X_SDRAM_LEN_IS_0MB				TP6X_SDRAM_LEN_IS_0M
#define TP6X_SDRAM_LEN_IS_32MB				TP6X_SDRAM_LEN_IS_4M
#define TP6X_SDRAM_LEN_IS_128MB				TP6X_SDRAM_LEN_IS_16M

#define TP6X_DPRAM_LEN_IS_0KB				TP6X_DPRAM_LEN_IS_0K
#define TP6X_DPRAM_LEN_IS_512KB				TP6X_DPRAM_LEN_IS_128K
#define TP6X_DPRAM_LEN_IS_1MB				TP6X_DPRAM_LEN_IS_256K
#define TP6X_DPRAM_LEN_IS_2MB				TP6X_DPRAM_LEN_IS_512K


		// macros returning true SBSRAM/SDRAM/DPRAM memories length (in bytes)
#define TP6X_get_sbsram_len()     			(TP6X_SBSRAM_LEN_IS_0K ? TP6X_SBSRAM_LEN_0K:\
													(TP6X_SBSRAM_LEN_IS_128K ? TP6X_SBSRAM_LEN_128K:\
														(TP6X_SBSRAM_LEN_IS_256K ? TP6X_SBSRAM_LEN_256K:TP6X_SBSRAM_LEN_512K )))

#define TP6X_get_sdram_len()     			(TP6X_SDRAM_LEN_IS_0M ? TP6X_SDRAM_LEN_0M:\
													(TP6X_SDRAM_LEN_IS_4M ? TP6X_SDRAM_LEN_4M:TP6X_SDRAM_LEN_16M))

#define TP6X_get_dpram_len()     			(TP6X_DPRAM_LEN_IS_0K ? TP6X_DPRAM_LEN_0K:\
													(TP6X_DPRAM_LEN_IS_128K ? TP6X_DPRAM_LEN_128K:\
														(TP6X_DPRAM_LEN_IS_256K ? TP6X_DPRAM_LEN_256K:TP6X_DPRAM_LEN_512K )))
#endif


//---- SBSRAM/SDRAM/DPRA< length return macro for TP62/TP67/TP6202/TP6203 boards
//     These are the false macros returning maximum possible values for SBSRAM (1Mx32)
//	   and 4Mx32 for SDRAM. The length of DPRAM is returened upon the setting of the __TP6X_DPRAM64K__
//	   compiler key.
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
#define TP6X_get_sbsram_len()     			TP6X_SBSRAM_LEN
#define TP6X_get_sdram_len()     			TP6X_SDRAM_LEN
#define TP6X_get_dpram_len()     			TP6X_DPRAM_LEN
#endif




	//~~~ EXT_INT4..EXT_INT7 and NMI IRQ source selector IOX registers
	//    Provided macros include both individual INT selectors SET/GET macros and
	//    universal SET/GET macro, which uses INT_SEL ID to select particular EXT_INTx/NMI.

		// individual INT selectors access macros
#define TP6X_set_ext_int4_sel_rg(v)			(*(__TP6X_IOX_DATA_TYPE__ *)  TP6X_EXT_INT4_SEL_ADDR)=((__TP6X_IOX_DATA_TYPE__)(v))
#define TP6X_get_ext_int4_sel_rg()			((*(__TP6X_IOX_DATA_TYPE__ *) TP6X_EXT_INT4_SEL_ADDR) & TP6X_IOX_DATA_BITMASK)
#define TP6X_set_ext_int5_sel_rg(v)			(*(__TP6X_IOX_DATA_TYPE__ *)  TP6X_EXT_INT5_SEL_ADDR)=((__TP6X_IOX_DATA_TYPE__)(v))
#define TP6X_get_ext_int5_sel_rg()			((*(__TP6X_IOX_DATA_TYPE__ *) TP6X_EXT_INT5_SEL_ADDR) & TP6X_IOX_DATA_BITMASK)
#define TP6X_set_ext_int6_sel_rg(v)			(*(__TP6X_IOX_DATA_TYPE__ *)  TP6X_EXT_INT6_SEL_ADDR)=((__TP6X_IOX_DATA_TYPE__)(v))
#define TP6X_get_ext_int6_sel_rg()			((*(__TP6X_IOX_DATA_TYPE__ *) TP6X_EXT_INT6_SEL_ADDR) & TP6X_IOX_DATA_BITMASK)
#define TP6X_set_ext_int7_sel_rg(v)			(*(__TP6X_IOX_DATA_TYPE__ *)  TP6X_EXT_INT7_SEL_ADDR)=((__TP6X_IOX_DATA_TYPE__)(v))
#define TP6X_get_ext_int7_sel_rg()			((*(__TP6X_IOX_DATA_TYPE__ *) TP6X_EXT_INT7_SEL_ADDR) & TP6X_IOX_DATA_BITMASK)
#define TP6X_set_nmi_sel_rg(v)				(*(__TP6X_IOX_DATA_TYPE__ *)  TP6X_NMI_SEL_ADDR)=((__TP6X_IOX_DATA_TYPE__)v)
#define TP6X_get_nmi_sel_rg()				((*(__TP6X_IOX_DATA_TYPE__ *) TP6X_NMI_SEL_ADDR) & TP6X_IOX_DATA_BITMASK)

	// dedicated bit-specific macros
#define	TP6X_set_ext_int4_sel(v)			TP6X_set_ext_int4_sel_rg(v)
#define TP6X_get_ext_int4_sel()				TP6X_get_ext_int4_sel_rg()
#define	TP6X_set_ext_int5_sel(v)			TP6X_set_ext_int5_sel_rg(v)
#define TP6X_get_ext_int5_sel()				TP6X_get_ext_int5_sel_rg()
#define	TP6X_set_ext_int6_sel(v)			TP6X_set_ext_int6_sel_rg(v)
#define TP6X_get_ext_int6_sel()				TP6X_get_ext_int6_sel_rg()
#define	TP6X_set_ext_int7_sel(v)			TP6X_set_ext_int7_sel_rg(v)
#define TP6X_get_ext_int7_sel()				TP6X_get_ext_int7_sel_rg()
#define	TP6X_set_nmi_sel(v)					TP6X_set_nmi_sel_rg(v)
#define TP6X_get_nmi_sel()					TP6X_get_nmi_sel_rg()

	// aliases and style-compatible aliases for compatibility with old API
#define TP6X_ext_int4_sel()					TP6X_get_ext_int4_sel_rg()
#define TP6X_ext_int5_sel()					TP6X_get_ext_int5_sel_rg()
#define TP6X_ext_int6_sel()					TP6X_get_ext_int6_sel_rg()
#define TP6X_ext_int7_sel()					TP6X_get_ext_int7_sel_rg()
#define TP6X_nmi_sel()						TP6X_get_nmi_sel_rg()


	// universal macros to set/get EXT_INT/EXT_NMI selector regs using EXT_INT ID
			// IDs for universal EXT_INT/NMI selectors control macro
#define	TP6X_EXT_INT4_SEL					0
#define	TP6X_EXT_INT5_SEL					1
#define	TP6X_EXT_INT6_SEL					2
#define	TP6X_EXT_INT7_SEL					3
#define	TP6X_NMI_SEL						4
				//aliases
#define	TP6X_EXT_INT4						TP6X_EXT_INT4_SEL
#define	TP6X_EXT_INT5						TP6X_EXT_INT5_SEL
#define	TP6X_EXT_INT6						TP6X_EXT_INT6_SEL
#define	TP6X_EXT_INT7						TP6X_EXT_INT7_SEL
#define	TP6X_NMI							TP6X_NMI_SEL

			// access macros with input params: IRQ={TP6X_EXT_INTx_SEL/TP6X_NMI_SEL}, V={TP6X_EXT_INT_SEL_..}
#define	TP6X_set_ext_int_sel_rg(irq,v)		if(irq==0) TP6X_set_ext_int4_sel_rg(v); \
										  	 else if(irq==1) TP6X_set_ext_int5_sel_rg(v);\
										      else if(irq==2) TP6X_set_ext_int6_sel_rg(v);\
										       else if(irq==3) TP6X_set_ext_int7_sel_rg(v);\
											    else TP6X_set_nmi_sel_rg(v)
#define	TP6X_get_ext_int_sel_rg(irq)		((irq==0)? TP6X_get_ext_int4_sel(): \
										  	 ((irq==1)? TP6X_get_ext_int5_sel():\
										      ((irq==2)? TP6X_get_ext_int6_sel():\
										       ((irq==3)? TP6X_get_ext_int7_sel():TP6X_get_nmi_sel()))))
		// macros aliases
#define	TP6X_set_ext_int_sel(irq,v)			TP6X_set_ext_int_sel_rg(irq,v)
#define	TP6X_get_ext_int_sel(irq)			TP6X_get_ext_int_sel_rg(irq)

		// direct universal set/get macros
#define	TP6X_SET_EXT_INT_SEL_OFF(irq)		TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_OFF)
#define	TP6X_SET_EXT_INT_SEL_SXIRQ0(irq)	TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_SXIRQ0)		// macros with _SXIRQx appear different upon the board type
#define	TP6X_SET_EXT_INT_SEL_SXIRQ1(irq)	TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_SXIRQ1)
#define	TP6X_SET_EXT_INT_SEL_SXIRQ2(irq)	TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_SXIRQ2)
#define	TP6X_SET_EXT_INT_SEL_PXIRQ0(irq)	TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_PXIRQ0)		// macros with _PXIRQx appear different upon the board type
#define	TP6X_SET_EXT_INT_SEL_PXIRQ1(irq)	TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_PXIRQ1)
#define	TP6X_SET_EXT_INT_SEL_PXIRQ2(irq)	TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_PXIRQ2)
#define	TP6X_SET_EXT_INT_SEL_PXIRQ3(irq)	TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_PXIRQ3)
#define	TP6X_SET_EXT_INT_SEL_MDPIRQ(irq)	TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_MDPIRQ)
#define	TP6X_SET_EXT_INT_SEL_HM_RQ0(irq)	TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_HM_RQ0)
#define	TP6X_SET_EXT_INT_SEL_HM_RQ1(irq)	TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_HM_RQ1)
#define	TP6X_SET_EXT_INT_SEL_PCIC(irq)		TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_PCIC)
#define	TP6X_SET_EXT_INT_SEL_RFIFO(irq)		TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_RFIFO)
#define	TP6X_SET_EXT_INT_SEL_WFIFO(irq)		TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_WFIFO)
#define	TP6X_SET_EXT_INT_SEL_WDT(irq)		TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_WDT)

			// _IS_OFF direct get macro must consider all possible reserved INT selector settings
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
				// TP62/TP620x/TP67 definition (RSRV1..RSRV4 codes must be processed)
#define	TP6X_EXT_INT_SEL_IS_OFF(irq)		(    TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_OFF 	\
											  || TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_RSRV1	\
											  || TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_RSRV2	\
											  || TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_RSRV3	\
											  || TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_RSRV4	\
											)
#else
				// TP64xx must use RSRV1 only and _MDPIRQ for DPRAM256K only
#define	TP6X_EXT_INT_SEL_IS_OFF(irq)		(    TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_OFF 	\
											  || TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_RSRV1	\
											)
#endif

#define	TP6X_EXT_INT_SEL_IS_SXIRQ0(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_SXIRQ0)	// macros with _SXIRQx appear different upon the board type
#define	TP6X_EXT_INT_SEL_IS_SXIRQ1(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_SXIRQ1)
#define	TP6X_EXT_INT_SEL_IS_SXIRQ2(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_SXIRQ2)
#define	TP6X_EXT_INT_SEL_IS_PXIRQ0(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_PXIRQ0)	// macros with _PXIRQx appear different upon the board type
#define	TP6X_EXT_INT_SEL_IS_PXIRQ1(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_PXIRQ1)
#define	TP6X_EXT_INT_SEL_IS_PXIRQ2(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_PXIRQ2)
#define	TP6X_EXT_INT_SEL_IS_PXIRQ3(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_PXIRQ3)
#define	TP6X_EXT_INT_SEL_IS_MDPIRQ(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_MDPIRQ)
#define	TP6X_EXT_INT_SEL_IS_HM_RQ0(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_HM_RQ0)
#define	TP6X_EXT_INT_SEL_IS_HM_RQ1(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_HM_RQ1)
#define	TP6X_EXT_INT_SEL_IS_PCIC(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_PCIC)
#define	TP6X_EXT_INT_SEL_IS_RFIFO(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_RFIFO)
#define	TP6X_EXT_INT_SEL_IS_WFIFO(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_WFIFO)
#define	TP6X_EXT_INT_SEL_IS_WDT(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_WDT)

			// macros for TP62/TP620x/TP67 boards only
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
#define	TP6X_SET_EXT_INT_SEL_XIRQ0(irq)		TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_XIRQ0)
#define	TP6X_SET_EXT_INT_SEL_XIRQ1(irq)		TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_XIRQ1)
#define	TP6X_SET_EXT_INT_SEL_XIRQ2(irq)		TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_XIRQ2)
#define	TP6X_SET_EXT_INT_SEL_XIRQ3(irq)		TP6X_set_ext_int_sel(irq,TP6X_EXT_INT_SEL_XIRQ3)

#define	TP6X_EXT_INT_SEL_IS_XIRQ0(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_XIRQ0)
#define	TP6X_EXT_INT_SEL_IS_XIRQ1(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_XIRQ1)
#define	TP6X_EXT_INT_SEL_IS_XIRQ2(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_XIRQ2)
#define	TP6X_EXT_INT_SEL_IS_XIRQ3(irq)		(TP6X_get_ext_int_sel(irq)==TP6X_EXT_INT_SEL_XIRQ3)
#endif

			// old API defs and aliases for compatibility with old API
#define	TP6X_MIRQ0							TP6X_EXT_INT4_SEL
#define	TP6X_MIRQ1							TP6X_EXT_INT5_SEL
#define	TP6X_MIRQ2							TP6X_EXT_INT6_SEL
#define	TP6X_MIRQ3							TP6X_EXT_INT7_SEL

#define TP6X_set_mnmi_sel(v)				TP6X_set_nmi_sel(v)
#define TP6X_mnmi_sel()						TP6X_get_nmi_sel()
#define TP6X_set_mirq_sel(n,v)				TP6X_set_ext_int_sel_rg(n,v)
#define TP6X_mirq_sel(n)					TP6X_get_ext_int_sel_rg(n)


//------ Macros for PIOX and SIOX_rev_C access
	// PIOX/PIOX-16 access
#define TP6X_set_piox_rg(a,v)				(*(unsigned *) (TP6X_PIOX_SADDR + ((a&0x7ffff)<<2)))=(unsigned)(v)
#define TP6X_get_piox_rg(a)					(*(unsigned *) (TP6X_PIOX_SADDR + ((a&0x7ffff)<<2)))

#define TP6X_set_piox_lsw_rg(a,v)			(*(unsigned short *) (TP6X_PIOX_SADDR + ((a&0x7ffff)<<2)))=(unsigned short)((v)&0xffff)
#define TP6X_set_piox_msw_rg(a,v)			(*(unsigned short *) (TP6X_PIOX_SADDR + 2+ ((a&0x7ffff)<<2)))=(unsigned short)(((v)>>16)&0xffff)
#define TP6X_get_piox_lsw_rg(a)				(*(unsigned short *) (TP6X_PIOX_SADDR + ((a&0x7ffff)<<2)))
#define TP6X_get_piox_msw_rg(a)				(*(unsigned short *) (TP6X_PIOX_SADDR + 2+ ((a&0x7ffff)<<2)))

#define TP6X_set_piox16_rg(a,v)				(*(__TP6X_IOX_DATA_TYPE__ *) (TP6X_PIOX_SADDR + ((a&0xffff)<<2)))=(((__TP6X_IOX_DATA_TYPE__) (v)) & 0xffff)
#define TP6X_get_piox16_rg(a)				((*(__TP6X_IOX_DATA_TYPE__ *) (TP6X_PIOX_SADDR + ((a&0xffff)<<2))) & 0xffff)

	// SIOX_rev_C access
#define TP6X_set_sioxa_revc_rg(a,v)			(*(__TP6X_IOX_DATA_TYPE__ *) (TP6X_SIOXA_REVC_SADDR + ((a&0x3f)<<2)))=(((__TP6X_IOX_DATA_TYPE__) (v)) & 0xff)
#define TP6X_get_sioxa_revc_rg(a)			((*(__TP6X_IOX_DATA_TYPE__ *) (TP6X_SIOXA_REVC_SADDR + ((a&0x3f)<<2))) & 0xff)

#define TP6X_set_sioxb_revc_rg(a,v)			(*(__TP6X_IOX_DATA_TYPE__ *) (TP6X_SIOXB_REVC_SADDR + ((a&0x3f)<<2)))=(((__TP6X_IOX_DATA_TYPE__) (v)) & 0xff)
#define TP6X_get_sioxb_revc_rg(a)			((*(__TP6X_IOX_DATA_TYPE__ *) (TP6X_SIOXB_REVC_SADDR + ((a&0x3f)<<2))) & 0xff)

	// aliases
#define TP6X_set_px_rg(a,v)					TP6X_set_piox_rg(a,v)
#define TP6X_get_px_rg(a)					TP6X_get_piox_rg(a)
#define TP6X_set_px16_rg(a,v)				TP6X_set_piox16_rg(a,v)
#define TP6X_get_px16_rg(a)					TP6X_get_piox16_rg(a)
#define TP6X_set_sxa_revc_rg(a,v)			TP6X_set_sioxa_revc_rg(a,v)
#define TP6X_get_sxa_revc_rg(a)				TP6X_get_sioxa_revc_rg(a)
#define TP6X_set_sxb_revc_rg(a,v)			TP6X_set_sioxb_revc_rg(a,v)
#define TP6X_get_sxb_revc_rg(a)				TP6X_get_sioxb_revc_rg(a)




	//---- XMEM2_LEN_ID and FLASH_PAGE registers
	//	   (FLASH capacity ID and FLASH PAGE selector macros for TP64xx boards with f/w rev.1B and later only)
	//     Notes:
	//		1. For TP64xx f/w rev.1A/1A-1/1A-2 boards w/o XMEM2_LEN_ID register, the TP6X_get_flash_len() macros
	//		   always returns the 512Kx8 valua, which is the maximum FLASH PLCC-32 memory size available (typically
	//		   shipped with all TORNADO boards).
	//		2. For TP64xx rev.1B boards, the TP6X_get_flash_len() macros returns installed FLASH length.
	//		3. For TP64xx f/w rev.1B boards, the number of valid bits of the FLASH_PAGE register is defined upon the
	//		   installed FLASH memory capacity. All writes to the FLASH_PAGE register are ignored for 512K/1Mx8 FLASH
	//		   memories, and write to invalid bits is also ignored for 2M..32Mx8 FLASH memorries. reading from invalid
	//		   bits always returns the '0' value.
	//		4. For rev.1A/1A-1/1A-2 boards, reading the XMEM2_LEN_ID and FLASH_PAGE registers always returns the '0' values.

#ifdef	__TP6X_SELECT_TP64XX__

	// XMEM2_LEN_ID register (available for f/w rev.1A-1 and rev.1A-2 only !!!)
#define TP6X_get_xmem2_len_id_rg()			((*(__TP6X_IOX_DATA_TYPE__ *) TP64XX_XMEM2_LEN_ID_ADDR) & TP6X_IOX_DATA_BITMASK)

		// dedicated bit specific macros
#define TP6X_get_flash_len_id()				(TP6X_get_xmem2_len_id_rg() & TP6X_FLASH_LEN_ID_BITMASK)

		// condition check macros (for use in IF() and other conditional operators)
			//	For TP64xx f/w rev.1A/1A-1/1A-2 boards, the FLASH length condition check macros returns the true value
			//  for the 512Kx8 FLASH length only
#define TP6X_FLASH_LEN_IS_0K				(TP64XX_XMEM2_LEN_ID_IS_AVAILABLE && (TP6X_get_flash_len_id()==TP6X_FLASH_LEN_ID_0K))
#define TP6X_FLASH_LEN_IS_512K				(!TP64XX_XMEM2_LEN_ID_IS_AVAILABLE || (TP64XX_XMEM2_LEN_ID_IS_AVAILABLE && (TP6X_get_flash_len_id()==TP6X_FLASH_LEN_ID_512K)))
#define TP6X_FLASH_LEN_IS_1M				(TP64XX_XMEM2_LEN_ID_IS_AVAILABLE && (TP6X_get_flash_len_id()==TP6X_FLASH_LEN_ID_1M))
#define TP6X_FLASH_LEN_IS_2M				(TP64XX_XMEM2_LEN_ID_IS_AVAILABLE && (TP6X_get_flash_len_id()==TP6X_FLASH_LEN_ID_2M))
#define TP6X_FLASH_LEN_IS_4M				(TP64XX_XMEM2_LEN_ID_IS_AVAILABLE && (TP6X_get_flash_len_id()==TP6X_FLASH_LEN_ID_4M))
#define TP6X_FLASH_LEN_IS_8M				(TP64XX_XMEM2_LEN_ID_IS_AVAILABLE && (TP6X_get_flash_len_id()==TP6X_FLASH_LEN_ID_8M))
#define TP6X_FLASH_LEN_IS_16M				(TP64XX_XMEM2_LEN_ID_IS_AVAILABLE && (TP6X_get_flash_len_id()==TP6X_FLASH_LEN_ID_16M))
#define TP6X_FLASH_LEN_IS_32M				(TP64XX_XMEM2_LEN_ID_IS_AVAILABLE && (TP6X_get_flash_len_id()==TP6X_FLASH_LEN_ID_32M))
#define TP6X_FLASH_LEN_IS_64M				(TP64XX_XMEM2_LEN_ID_IS_AVAILABLE && (TP6X_get_flash_len_id()==TP6X_FLASH_LEN_ID_64M))

#define TP6X_FLASH_IS_NOT_INSTALLED			TP6X_FLASH_LEN_IS_0K
#define TP6X_FLASH_IS_INSTALLED				!TP6X_FLASH_IS_NOT_INSTALLED


			// aliases for length in bytes
#define TP6X_FLASH_LEN_IS_0KB				TP6X_FLASH_LEN_IS_0K
#define TP6X_FLASH_LEN_IS_512KB				TP6X_FLASH_LEN_IS_512K
#define TP6X_FLASH_LEN_IS_1MB				TP6X_FLASH_LEN_IS_1M
#define TP6X_FLASH_LEN_IS_2MB				TP6X_FLASH_LEN_IS_2M
#define TP6X_FLASH_LEN_IS_4MB				TP6X_FLASH_LEN_IS_4M
#define TP6X_FLASH_LEN_IS_8MB				TP6X_FLASH_LEN_IS_8M
#define TP6X_FLASH_LEN_IS_16MB				TP6X_FLASH_LEN_IS_16M
#define TP6X_FLASH_LEN_IS_32MB				TP6X_FLASH_LEN_IS_32M
#define TP6X_FLASH_LEN_IS_64MB				TP6X_FLASH_LEN_IS_64M


		// macros returning true FLASH memory length (in bytes)
#define TP6X_get_flash_len()     			(TP6X_FLASH_LEN_IS_0K ? TP6X_FLASH_LEN_0K:\
													(TP6X_FLASH_LEN_IS_512K ? TP6X_FLASH_LEN_512K:\
														(TP6X_FLASH_LEN_IS_1M ? TP6X_FLASH_LEN_1M:\
															(TP6X_FLASH_LEN_IS_2M ? TP6X_FLASH_LEN_2M:\
																(TP6X_FLASH_LEN_IS_4M ? TP6X_FLASH_LEN_4M:\
																	(TP6X_FLASH_LEN_IS_8M ? TP6X_FLASH_LEN_8M:\
																		(TP6X_FLASH_LEN_IS_16M ? TP6X_FLASH_LEN_16M:\
																			(TP6X_FLASH_LEN_IS_32M ? TP6X_FLASH_LEN_32M:TP6X_FLASH_LEN_64M))))))))






	// FLASH PAGE register (available for f/w rev.1A-1 and rev.1A-2 only !!!)
#define TP6X_set_flash_page_rg(v)			(*(__TP6X_IOX_DATA_TYPE__ *)  TP6X_FLASH_PAGE_ADDR)=((__TP6X_IOX_DATA_TYPE__)(v))
#define TP6X_get_flash_page_rg()			((*(__TP6X_IOX_DATA_TYPE__ *) TP6X_FLASH_PAGE_ADDR) & TP6X_IOX_DATA_BITMASK)

		// dedicated bit-specific macros with V = {0..flash_page#}
#define TP6X_set_flash_page(v)				TP6X_set_flash_page_rg(v)
#define TP6X_get_flash_page()				TP6X_get_flash_page_rg()

		// dedicated FLASH PAGE set/get macros aligned to the absolute FLASH address
#define TP6X_set_flash_page_via_addr(v)		TP6X_set_flash_page(v/TP6X_FLASH_PAGE_LEN)
#define TP6X_get_flash_page_start_addr()	(TP6X_get_flash_page() * TP6X_FLASH_PAGE_LEN)

#endif
//======================================================================


//====================== HPI port specific macros ======================
#define	_C6x_HPIC_ADDR						0x01880000	// TP62/TP67/TP64xx
#define	_C6x_XBHC_ADDR						0x0188000c	// TP6202/TP6203

#define	_C6X_HPIC_DSPINT_BITMASK			0x2
#define	_C6X_HPI_DSPINT_BITMASK				0x2
#define	_C6201_C6701_HPIC_HINT_BITMASK		0x4	// TP62/TP67/TP64xx only
#define	_C6201_C6701_HPI_HINT_BITMASK		0x4	// TP62/TP67/TP64xx only
#define	_C6201_C6701_C64XX_HPIC_HINT_BITMASK	0x4

#define TP6X_hpic_dspint()					(((*(unsigned *) ((TP62_TP67_BOARD || TP64XX_BOARD)? _C6x_HPIC_ADDR:_C6x_XBHC_ADDR))>>1) &0x1)
#define TP6X_hpic_hint()					(((*(unsigned *) _C6x_HPIC_ADDR)>>2) &0x1)
#define TP6X_clr_hpic_dspint()				(*(unsigned *) ((TP62_TP67_BOARD || TP64XX_BOARD) ? _C6x_HPIC_ADDR:_C6x_XBHC_ADDR))= _C6X_HPI_DSPINT_BITMASK
#define TP6X_set_hpic_hint()				(*(unsigned *) _C6x_HPIC_ADDR)=_C6201_C6701_C64XX_HPIC_HINT_BITMASK

	// old function names aliases
#define TP6X_dspint()						TP6X_hpic_dspint()
#define TP6X_hint()							TP6X_hpic_hint()
#define TP6X_clr_dspint()					TP6X_clr_hpic_dspint()
#define TP6X_set_hint()						TP6X_set_hpic_hint()
//======================================================================


//========== Macros & Definitions for AMCC S5933 PCIC access ===========

	// AMCC S5933 PCIC IMBX/OMBX ID for access macros
#define	TP6X_PCIC_OMBX0								0	// OMBX selector IDs for macros
#define	TP6X_PCIC_OMBX1								1
#define	TP6X_PCIC_OMBX2								2
#define	TP6X_PCIC_OMBX3								3	

#define	TP6X_PCIC_IMBX0								0	// IMBX selector IDs for macros
#define	TP6X_PCIC_IMBX1								1	
#define	TP6X_PCIC_IMBX2								2	
#define	TP6X_PCIC_IMBX3								3


	// AMBEF register bits/bitmasks defs
		// IMBX STATUS bitmasks
#define TP6X_PCIC_AMBEF_IMBX0_BYTE0_STAT_BITMASK	0x1
#define TP6X_PCIC_AMBEF_IMBX0_BYTE1_STAT_BITMASK	0x2
#define TP6X_PCIC_AMBEF_IMBX0_BYTE2_STAT_BITMASK	0x3
#define TP6X_PCIC_AMBEF_IMBX0_BYTE3_STAT_BITMASK	0x8
#define TP6X_PCIC_AMBEF_IMBX1_BYTE0_STAT_BITMASK	0x10
#define TP6X_PCIC_AMBEF_IMBX1_BYTE1_STAT_BITMASK	0x20
#define TP6X_PCIC_AMBEF_IMBX1_BYTE2_STAT_BITMASK	0x30
#define TP6X_PCIC_AMBEF_IMBX1_BYTE3_STAT_BITMASK	0x80
#define TP6X_PCIC_AMBEF_IMBX2_BYTE0_STAT_BITMASK	0x100
#define TP6X_PCIC_AMBEF_IMBX2_BYTE1_STAT_BITMASK	0x200
#define TP6X_PCIC_AMBEF_IMBX2_BYTE2_STAT_BITMASK	0x300
#define TP6X_PCIC_AMBEF_IMBX2_BYTE3_STAT_BITMASK	0x800
#define TP6X_PCIC_AMBEF_IMBX3_BYTE0_STAT_BITMASK	0x1000
#define TP6X_PCIC_AMBEF_IMBX3_BYTE1_STAT_BITMASK	0x2000
#define TP6X_PCIC_AMBEF_IMBX3_BYTE2_STAT_BITMASK	0x3000
#define TP6X_PCIC_AMBEF_IMBX3_BYTE3_STAT_BITMASK	0x8000

		// IMBX FULL/EMPTY flags
#define TP6X_PCIC_AMBEF_IMBX0_BYTE0_FULL			0x1
#define TP6X_PCIC_AMBEF_IMBX0_BYTE1_FULL			0x2
#define TP6X_PCIC_AMBEF_IMBX0_BYTE2_FULL			0x3
#define TP6X_PCIC_AMBEF_IMBX0_BYTE3_FULL			0x8
#define TP6X_PCIC_AMBEF_IMBX1_BYTE0_FULL			0x10
#define TP6X_PCIC_AMBEF_IMBX1_BYTE1_FULL			0x20
#define TP6X_PCIC_AMBEF_IMBX1_BYTE2_FULL			0x30
#define TP6X_PCIC_AMBEF_IMBX1_BYTE3_FULL			0x80
#define TP6X_PCIC_AMBEF_IMBX2_BYTE0_FULL			0x100
#define TP6X_PCIC_AMBEF_IMBX2_BYTE1_FULL			0x200
#define TP6X_PCIC_AMBEF_IMBX2_BYTE2_FULL			0x300
#define TP6X_PCIC_AMBEF_IMBX2_BYTE3_FULL			0x800
#define TP6X_PCIC_AMBEF_IMBX3_BYTE0_FULL			0x1000
#define TP6X_PCIC_AMBEF_IMBX3_BYTE1_FULL			0x2000
#define TP6X_PCIC_AMBEF_IMBX3_BYTE2_FULL			0x3000
#define TP6X_PCIC_AMBEF_IMBX3_BYTE3_FULL			0x8000

#define TP6X_PCIC_AMBEF_IMBX0_BYTE0_EMPTY			0
#define TP6X_PCIC_AMBEF_IMBX0_BYTE1_EMPTY			0
#define TP6X_PCIC_AMBEF_IMBX0_BYTE2_EMPTY			0
#define TP6X_PCIC_AMBEF_IMBX0_BYTE3_EMPTY			0
#define TP6X_PCIC_AMBEF_IMBX1_BYTE0_EMPTY			0
#define TP6X_PCIC_AMBEF_IMBX1_BYTE1_EMPTY			0
#define TP6X_PCIC_AMBEF_IMBX1_BYTE2_EMPTY			0
#define TP6X_PCIC_AMBEF_IMBX1_BYTE3_EMPTY			0
#define TP6X_PCIC_AMBEF_IMBX2_BYTE0_EMPTY			0
#define TP6X_PCIC_AMBEF_IMBX2_BYTE1_EMPTY			0
#define TP6X_PCIC_AMBEF_IMBX2_BYTE2_EMPTY			0
#define TP6X_PCIC_AMBEF_IMBX2_BYTE3_EMPTY			0
#define TP6X_PCIC_AMBEF_IMBX3_BYTE0_EMPTY			0
#define TP6X_PCIC_AMBEF_IMBX3_BYTE1_EMPTY			0
#define TP6X_PCIC_AMBEF_IMBX3_BYTE2_EMPTY			0
#define TP6X_PCIC_AMBEF_IMBX3_BYTE3_EMPTY			0

		// OMBX FULL flags bitmasks
#define TP6X_PCIC_AMBEF_OMBX0_BYTE0_STAT_BITMASK	0x10000
#define TP6X_PCIC_AMBEF_OMBX0_BYTE1_STAT_BITMASK	0x20000
#define TP6X_PCIC_AMBEF_OMBX0_BYTE2_STAT_BITMASK	0x30000
#define TP6X_PCIC_AMBEF_OMBX0_BYTE3_STAT_BITMASK	0x80000
#define TP6X_PCIC_AMBEF_OMBX1_BYTE0_STAT_BITMASK	0x100000
#define TP6X_PCIC_AMBEF_OMBX1_BYTE1_STAT_BITMASK	0x200000
#define TP6X_PCIC_AMBEF_OMBX1_BYTE2_STAT_BITMASK	0x300000
#define TP6X_PCIC_AMBEF_OMBX1_BYTE3_STAT_BITMASK	0x800000
#define TP6X_PCIC_AMBEF_OMBX2_BYTE0_STAT_BITMASK	0x1000000
#define TP6X_PCIC_AMBEF_OMBX2_BYTE1_STAT_BITMASK	0x2000000
#define TP6X_PCIC_AMBEF_OMBX2_BYTE2_STAT_BITMASK	0x3000000
#define TP6X_PCIC_AMBEF_OMBX2_BYTE3_STAT_BITMASK	0x8000000
#define TP6X_PCIC_AMBEF_OMBX3_BYTE0_STAT_BITMASK	0x10000000
#define TP6X_PCIC_AMBEF_OMBX3_BYTE1_STAT_BITMASK	0x20000000
#define TP6X_PCIC_AMBEF_OMBX3_BYTE2_STAT_BITMASK	0x30000000
#define TP6X_PCIC_AMBEF_OMBX3_BYTE3_STAT_BITMASK	0x80000000

		// OMBX FULL/EMPTY flags
#define TP6X_PCIC_AMBEF_OMBX0_BYTE0_FULL			0x10000
#define TP6X_PCIC_AMBEF_OMBX0_BYTE1_FULL			0x20000
#define TP6X_PCIC_AMBEF_OMBX0_BYTE2_FULL			0x30000
#define TP6X_PCIC_AMBEF_OMBX0_BYTE3_FULL			0x80000
#define TP6X_PCIC_AMBEF_OMBX1_BYTE0_FULL			0x100000
#define TP6X_PCIC_AMBEF_OMBX1_BYTE1_FULL			0x200000
#define TP6X_PCIC_AMBEF_OMBX1_BYTE2_FULL			0x300000
#define TP6X_PCIC_AMBEF_OMBX1_BYTE3_FULL			0x800000
#define TP6X_PCIC_AMBEF_OMBX2_BYTE0_FULL			0x1000000
#define TP6X_PCIC_AMBEF_OMBX2_BYTE1_FULL			0x2000000
#define TP6X_PCIC_AMBEF_OMBX2_BYTE2_FULL			0x3000000
#define TP6X_PCIC_AMBEF_OMBX2_BYTE3_FULL			0x8000000
#define TP6X_PCIC_AMBEF_OMBX3_BYTE0_FULL			0x10000000
#define TP6X_PCIC_AMBEF_OMBX3_BYTE1_FULL			0x20000000
#define TP6X_PCIC_AMBEF_OMBX3_BYTE2_FULL			0x30000000
#define TP6X_PCIC_AMBEF_OMBX3_BYTE3_FULL			0x80000000

#define TP6X_PCIC_AMBEF_OMBX0_BYTE0_EMPTY			0
#define TP6X_PCIC_AMBEF_OMBX0_BYTE1_EMPTY			0
#define TP6X_PCIC_AMBEF_OMBX0_BYTE2_EMPTY			0
#define TP6X_PCIC_AMBEF_OMBX0_BYTE3_EMPTY			0
#define TP6X_PCIC_AMBEF_OMBX1_BYTE0_EMPTY			0
#define TP6X_PCIC_AMBEF_OMBX1_BYTE1_EMPTY			0
#define TP6X_PCIC_AMBEF_OMBX1_BYTE2_EMPTY			0
#define TP6X_PCIC_AMBEF_OMBX1_BYTE3_EMPTY			0
#define TP6X_PCIC_AMBEF_OMBX2_BYTE0_EMPTY			0
#define TP6X_PCIC_AMBEF_OMBX2_BYTE1_EMPTY			0
#define TP6X_PCIC_AMBEF_OMBX2_BYTE2_EMPTY			0
#define TP6X_PCIC_AMBEF_OMBX2_BYTE3_EMPTY			0
#define TP6X_PCIC_AMBEF_OMBX3_BYTE0_EMPTY			0
#define TP6X_PCIC_AMBEF_OMBX3_BYTE1_EMPTY			0
#define TP6X_PCIC_AMBEF_OMBX3_BYTE2_EMPTY			0
#define TP6X_PCIC_AMBEF_OMBX3_BYTE3_EMPTY			0


	// AINT register bits/bitmasks defs
#define TP6X_PCIC_AINT_IRQ_BITMASK					0x00800000	// summary output PCI IRQ bitmask

		// particular source IRQ bitmasks
#define TP6X_PCIC_AINT_AM_ERR_IRQ_BITMASK			0x00200000	// IRQ on Add-on PCI-bus mastering error
#define TP6X_PCIC_AINT_BIST_IRQ_BITMASK				0x00100000	// IRQ on BIST (build-in sel test)
#define TP6X_PCIC_AINT_RTC_IRQ_BITMASK				0x00080000	// IRQ on PCI-bus mastering read transfer complete
#define TP6X_PCIC_AINT_WTC_IRQ_BITMASK				0x00040000	// IRQ on PCI-bus mastering write transfer complete
#define TP6X_PCIC_AINT_OMBX_IRQ_BITMASK				0x00020000	// IRQ on outgoing MBX
#define TP6X_PCIC_AINT_IMBX_IRQ_BITMASK				0x00010000	// IRQ on incoming MBX
#define TP6X_PCIC_AINT_IRQ_ALL_BITMASK				0x003f0000	// all particular IRQ bitmask

		// particular IRQ codes
#define TP6X_PCIC_AINT_IRQ_ON						0x00800000	// summary output IRQ
#define TP6X_PCIC_AINT_AM_ERR_IRQ_ON				0x00200000	// IRQ on add-on PCI-bus mastering error
#define TP6X_PCIC_AINT_BIST_IRQ_ON					0x00100000	// IRQ on BIST
#define TP6X_PCIC_AINT_RTC_IRQ_ON					0x00080000	// IRQ on PCI-bus mastering read transfer complete
#define TP6X_PCIC_AINT_WTC_IRQ_ON					0x00040000	// IRQ on PCI-bus mastering write transfer complete
#define TP6X_PCIC_AINT_OMBX_IRQ_ON					0x00020000	// IRQ on outgoing MBX
#define TP6X_PCIC_AINT_IMBX_IRQ_ON					0x00010000	// IRQ on incoming MBX

		// IRQ clear codes
#define TP6X_PCIC_AINT_CLR_ALL_IRQ					0x003f0000	// clear all IRQ sources
#define TP6X_PCIC_AINT_CLR_AM_ERR_IRQ				0x00200000	// clear IRQ Add-on PCI-bus mastering error
#define TP6X_PCIC_AINT_CLR_BIST_ERR_IRQ				0x00100000	// clear IRQ BIST error
#define TP6X_PCIC_AINT_CLR_RTC_IRQ					0x00080000	// clear IRQ on PCI-bus mastering read transfer complete
#define TP6X_PCIC_AINT_CLR_WTC_IRQ					0x00040000	// clear IRQ on PCI-bus mastering write transfer complete
#define TP6X_PCIC_AINT_CLR_OMBX_IRQ					0x00020000	// clear IRQ on outgoing MBX
#define TP6X_PCIC_AINT_CLR_IMBX_IRQ					0x00010000	// clear IRQ on incoming MBX

		// particular interrupt enable masks/codes
#define TP6X_PCIC_AINT_IE_ALL_BITMASK				0x0000d010	// bitmask for all IE fields

#define TP6X_PCIC_AINT_RTC_IE_BITMASK				0x00008000	// IE on PCI-bus mastering read transfer complete
#define TP6X_PCIC_AINT_RTC_IE_ON					0x00008000
#define TP6X_PCIC_AINT_RTC_IE_OFF					0

#define TP6X_PCIC_AINT_WTC_IE_BITMASK				0x00004000	// IE on PCI-bus mastering write transfer complete
#define TP6X_PCIC_AINT_WTC_IE_ON					0x00004000	   
#define TP6X_PCIC_AINT_WTC_IE_OFF					0

#define TP6X_PCIC_AINT_OMBX_ALL_BITMASK				0x00001f00	// bitmask for all OMBX IE/sel data
#define TP6X_PCIC_AINT_OMBX_IE_BITMASK				0x00001000	// IE on outgoing MBX
#define TP6X_PCIC_AINT_OMBX_IE_ON					0x00001000	   
#define TP6X_PCIC_AINT_OMBX_IE_OFF					0
#define TP6X_PCIC_AINT_OMBX_SEL_BITMASK				0x00000c00	// OMBX number selector
#define TP6X_PCIC_AINT_OMBX_SEL_BITSHIFT			10			// number of shift bits for OMBX number selector for OMBX# =0..3
#define TP6X_PCIC_AINT_OMBX0_SEL					0x00000000	//		OMBX#0 selector
#define TP6X_PCIC_AINT_OMBX1_SEL					0x00000400	//		OMBX#1 selector
#define TP6X_PCIC_AINT_OMBX2_SEL					0x00000800	//		OMBX#2 selector
#define TP6X_PCIC_AINT_OMBX3_SEL					0x00000c00	//		OMBX#3 selector
#define TP6X_PCIC_AINT_OMBX_BYTE_BITMASK			0x00000300	// OMBX byte selector selector
#define TP6X_PCIC_AINT_OMBX_BYTE_BITSHIFT			8			// number of shift bits for OMBX byte selector for BYTE# =0..3
#define TP6X_PCIC_AINT_OMBX_BYTE0					0x00000000	//		BYTE#0 selector
#define TP6X_PCIC_AINT_OMBX_BYTE1					0x00000100	//		BYTE#1 selector
#define TP6X_PCIC_AINT_OMBX_BYTE2					0x00000200	//		BYTE#2 selector
#define TP6X_PCIC_AINT_OMBX_BYTE3					0x00000300	//		BYTE#3 selector

#define TP6X_PCIC_AINT_IMBX_ALL_BITMASK				0x0000001f	// bitmask for all IMBX IE/sel data
#define TP6X_PCIC_AINT_IMBX_IE_BITMASK				0x00000010	// IE on incoming MBX
#define TP6X_PCIC_AINT_IMBX_IE_ON					0x00000010
#define TP6X_PCIC_AINT_IMBX_IE_OFF					0
#define TP6X_PCIC_AINT_IMBX_SEL_BITMASK				0x0000000c	// IMBX number selector
#define TP6X_PCIC_AINT_IMBX_SEL_BITSHIFT			2			// number of shift bits for IMBX number selector for IMBX# =0..3
#define TP6X_PCIC_AINT_IMBX0_SEL					0x00000000	//		IMBX#0 selector
#define TP6X_PCIC_AINT_IMBX1_SEL					0x00000004	//		IMBX#1 selector
#define TP6X_PCIC_AINT_IMBX2_SEL					0x00000008	//		IMBX#2 selector
#define TP6X_PCIC_AINT_IMBX3_SEL					0x0000000c	//		IMBX#3 selector
#define TP6X_PCIC_AINT_IMBX_BYTE_BITMASK			0x00000003	// IMBX byte selector selector
#define TP6X_PCIC_AINT_IMBX_BYTE_BITSHIFT			0			// number of shift bits for IMBX byte selector for BYTE# =0..3
#define TP6X_PCIC_AINT_IMBX_BYTE0					0x00000000	//		BYTE#0 selector
#define TP6X_PCIC_AINT_IMBX_BYTE1					0x00000001	//		BYTE#1 selector
#define TP6X_PCIC_AINT_IMBX_BYTE2					0x00000002	//		BYTE#2 selector
#define TP6X_PCIC_AINT_IMBX_BYTE3					0x00000003	//		BYTE#3 selector


	// AGCSTS register bits/bitmasks defs
	// note: bits #D12..15 are not used for TORNADO-Pxxx boards
		// enable control for AOB initiated PCI-bus mastering transfers (transfer count enable)
#define TP6X_PCIC_AGCSTS_AMTEN_BITMASK				0x10000000	// Transfer Count enable bitmask
#define TP6X_PCIC_AGCSTS_AMTEN_ON					0x10000000	
#define TP6X_PCIC_AGCSTS_AMTEN_OFF					0

		// read/write transfer count equals to zero status (read-only)
#define TP6X_PCIC_AGCSTS_RTCZ_STAT_BITMASK			0x00000040	// status of read transfer count equals to zero
#define TP6X_PCIC_AGCSTS_RTCZ_ON					0x00000040	
#define TP6X_PCIC_AGCSTS_RTCZ_OFF					0

#define TP6X_PCIC_AGCSTS_WTCZ_STAT_BITMASK			0x00000080	// status of write transfer count equals to zero
#define TP6X_PCIC_AGCSTS_WTCZ_ON					0x00000080	
#define TP6X_PCIC_AGCSTS_WTCZ_OFF					0

		// MBX/FIFO status flags reset control (write-only)
#define TP6X_PCIC_AGCSTS_CLR_ALL_MBX_FIFO_STAT_FLAGS	0x0E000000	// clear IMBX/OMBX/RFIFO/WFIFO flags
#define TP6X_PCIC_AGCSTS_CLR_MBX_STAT_FLAGS			0x08000000	// clear IMBX/OMBX flags
#define TP6X_PCIC_AGCSTS_CLR_RFIFO_STAT_FLAGS		0x04000000	// clear read FIFO (PCI-to-AOB) flags
#define TP6X_PCIC_AGCSTS_CLR_WFIFO_STAT_FLAGS		0x02000000	// clear write FIFO (AOB-to-PCI) flags

		// W-FIFO status flags (read-only)
#define TP6X_PCIC_AGCSTS_WFIFO_ALL_STAT_BITMASK		0x00000007	// bitmask for all WFIFO status flags
#define TP6X_PCIC_AGCSTS_WFIFO_FULL_STAT_BITMASK	0x00000001	// bitmask for WFIFO FULL status flag
#define TP6X_PCIC_AGCSTS_WFIFO_HFULL_STAT_BITMASK	0x00000002	// bitmask for WFIFO half-FULL status flag (>=4words are FULL)
#define TP6X_PCIC_AGCSTS_WFIFO_EMPTY_STAT_BITMASK	0x00000004	// bitmask for WFIFO EMPTY status flag

#define TP6X_PCIC_AGCSTS_WFIFO_FULL_ON				0x00000001	
#define TP6X_PCIC_AGCSTS_WFIFO_HFULL_ON				0x00000002	
#define TP6X_PCIC_AGCSTS_WFIFO_EMPTY_ON				0x00000004	

#define TP6X_PCIC_AGCSTS_WFIFO_FULL_OFF				0	
#define TP6X_PCIC_AGCSTS_WFIFO_HFULL_OFF			0
#define TP6X_PCIC_AGCSTS_WFIFO_EMPTY_OFF			0

		// R-FIFO status flags (read-only)
#define TP6X_PCIC_AGCSTS_RFIFO_ALL_STAT_BITMASK		0x00000038	// bitmask for all RFIFO status flags
#define TP6X_PCIC_AGCSTS_RFIFO_FULL_STAT_BITMASK	0x00000008	// bitmask for RFIFO FULL status flag
#define TP6X_PCIC_AGCSTS_RFIFO_HEMPTY_STAT_BITMASK	0x00000010	// bitmask for RFIFO half-EMPTY status flag (>=4words are empty)
#define TP6X_PCIC_AGCSTS_RFIFO_EMPTY_STAT_BITMASK	0x00000020	// bitmask for RFIFO EMPTY status flag

#define TP6X_PCIC_AGCSTS_RFIFO_FULL_ON				0x00000008	
#define TP6X_PCIC_AGCSTS_RFIFO_HEMPTY_ON			0x00000010
#define TP6X_PCIC_AGCSTS_RFIFO_EMPTY_ON				0x00000020

#define TP6X_PCIC_AGCSTS_RFIFO_FULL_OFF				0	
#define TP6X_PCIC_AGCSTS_RFIFO_HEMPTY_OFF			0
#define TP6X_PCIC_AGCSTS_RFIFO_EMPTY_OFF			0

		// NvRAM control (not recommended to use; use host PC utilities instead)
#define TP6X_PCIC_AGCSTS_NV_AD_BITMASK				0x00ff0000	// address/data bitmask
#define TP6X_PCIC_AGCSTS_NV_AD_BITSHIFT				16			// address/data bitshift

			// NvRAM status (read-only)
#define TP6X_PCIC_AGCSTS_NV_STAT_BITMASK			0x80000000	// NvRAM status bitmask (read-only)
#define TP6X_PCIC_AGCSTS_NV_RDY						0			// READY status (read-only)
#define TP6X_PCIC_AGCSTS_NV_BUSY					0x80000000	// BUSY status (read-only)

			// NvRAM commands (write-only)
#define TP6X_PCIC_AGCSTS_NV_LOAD_LOW_ADDR_CMND		0x80000000	// load low address byte
#define TP6X_PCIC_AGCSTS_NV_LOAD_HIGH_ADDR_CMND		0xA0000000	// load high address byte
#define TP6X_PCIC_AGCSTS_NV_WRITE_CMND				0xC0000000	// begin write
#define TP6X_PCIC_AGCSTS_NV_READ_CMND				0xE0000000	// begin read



	// PCIC access macros
	// These macros meet TP6X_SET_/TP6X_GET_ syntax of TP6x DSP utilities and are recommended
	// for all designs. Aliases for compatibility with the previous API are provided below.
	// Note: for TP64XX boards, it is recommended to perform PCIC access using 32-bit
	//       word format with on-board enabled 32-bit data access mode.
#define TP6X_set_pcic_wfifo_rg(v)			(*(unsigned *) TP6X_PCIC_FIFO_ADDR)=v
#define TP6X_get_pcic_rfifo_rg()			(*(unsigned *) TP6X_PCIC_FIFO_ADDR)

#define TP6X_get_pcic_ambef_rg()			(*(unsigned *) TP6X_PCIC_AMBEF_ADDR)

#define TP6X_set_pcic_aint_rg(v)			(*(unsigned *) TP6X_PCIC_AINT_ADDR)=v
#define TP6X_get_pcic_aint_rg()				(*(unsigned *) TP6X_PCIC_AINT_ADDR)

#define TP6X_set_pcic_agcsts_rg(v)			(*(unsigned *) TP6X_PCIC_AGCSTS_ADDR)=v
#define TP6X_get_pcic_agcsts_rg()			(*(unsigned *) TP6X_PCIC_AGCSTS_ADDR)

#define TP6X_set_pcic_mwar_rg(v)			(*(unsigned *) TP6X_PCIC_MWAR_ADDR)=v
#define TP6X_get_pcic_mwar_rg()				(*(unsigned *) TP6X_PCIC_MWAR_ADDR)
#define TP6X_set_pcic_mwtc_rg(v)			(*(unsigned *) TP6X_PCIC_MWTC_ADDR)=v
#define TP6X_get_pcic_mwtc_rg()				(*(unsigned *) TP6X_PCIC_MWTC_ADDR)
#define TP6X_set_pcic_mrar_rg(v)			(*(unsigned *) TP6X_PCIC_MRAR_ADDR)=v
#define TP6X_get_pcic_mrar_rg()				(*(unsigned *) TP6X_PCIC_MRAR_ADDR)
#define TP6X_set_pcic_mrtc_rg(v)			(*(unsigned *) TP6X_PCIC_MRTC_ADDR)=v
#define TP6X_get_pcic_mrtc_rg()				(*(unsigned *) TP6X_PCIC_MRTC_ADDR)

#define TP6X_set_pcic_ombx_rg(n,v)			(*(unsigned *) (TP6X_PCIC_OMBX0_ADDR+(n%4)*4))=v
#define TP6X_get_pcic_ombx_rg(n)			(*(unsigned *) (TP6X_PCIC_OMBX0_ADDR+(n%4)*4))
#define TP6X_get_pcic_imbx_rg(n)			(*(unsigned *) (TP6X_PCIC_IMBX0_ADDR+(n%4)*4))

			// aliases for compatibility with the previous API
			// (not recommended for use in the new designs)
#define TP6X_set_pcic_wfifo(v)				TP6X_set_pcic_wfifo_rg(v)
#define TP6X_pcic_rfifo()					TP6X_get_pcic_rfifo_rg()

#define TP6X_pcic_ambef()					TP6X_get_pcic_ambef_rg()

#define TP6X_set_pcic_aint(v)				TP6X_set_pcic_aint_rg(v)
#define TP6X_pcic_aint()					TP6X_get_pcic_aint_rg()

#define TP6X_set_pcic_agcsts(v)				TP6X_set_pcic_mwar_rg(v)
#define TP6X_pcic_agcsts()					TP6X_get_pcic_agcsts_rg()

#define TP6X_set_pcic_mwar(v)				TP6X_set_pcic_mwar_rg(v)
#define TP6X_pcic_mwar()					TP6X_get_pcic_mwar_rg()
#define TP6X_set_pcic_mwtc(v)				TP6X_set_pcic_mwtc_rg(v)
#define TP6X_pcic_mwtc()					TP6X_get_pcic_mwtc_rg()
#define TP6X_set_pcic_mrar(v)				TP6X_set_pcic_mrar_rg(v)
#define TP6X_pcic_mrar()					TP6X_get_pcic_mrar_rg()
#define TP6X_set_pcic_mrtc(v)				TP6X_set_pcic_mrtc_rg(v)
#define TP6X_pcic_mrtc()					TP6X_get_pcic_mrtc_rg()

#define TP6X_set_pcic_ombx(n,v)				TP6X_set_pcic_ombx_rg(n,v)
#define TP6X_pcic_ombx(n)					TP6X_get_pcic_ombx_rg(n)
#define TP6X_pcic_imbx(n)					TP6X_get_pcic_imbx_rg(n)
//======================================================================



//********* extended TP6X API functions defs/declarations **************

	//--- DSP EMIF INIT
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
int TP6X_init_dsp_emif(unsigned xmem_type);
#endif
#ifdef	__TP6X_SELECT_TP64XX__
int TP6X_init_dsp_emif();
#endif

	//--- defs for the FLASH programming utilities
		// defs returned by the FLASH programming API functions
#define TP6X_FLASH_AMD_MANUF_ID				0x01	// AMD manufacturer ID
#define TP6X_FLASH_SECTOR_PROTECTED			1
#define TP6X_FLASH_SECTOR_UNPROTECTED		0
#define TP6X_FLASH_DATA_BITMASK				0xff

			// AMD FLASH chips ID returned by the TP6X_flash_get_info() API function
#define TP6X_FLASH_AMD_PLUGIN_128KX8_DEV_ID	0x20    // AMD plug-in PLCC-32 128Kx8 5v-only FLASH used in TP6x and TP64xx f/w rev.1A/1A-1/1A-2 boards
#define TP6X_FLASH_AMD_PLUGIN_512KX8_DEV_ID	0xa4	// AMD plug-in PLCC-32 512Kx8 5v-only FLASH used in TP6x and TP64xx f/w rev.1A/1A-1/1A-2 boards
#define TP6X_FLASH_AMD_EMBED_512KX8_DEV_ID	0x4f	// AMD embedded 512Kx8 FLASH used in TP64xx f/w rev.1B boards
#define TP6X_FLASH_AMD_EMBED_1MX8_DEV_ID	0x38	// AMD embedded 1Mx8 FLASH used in TP64xx f/w rev.1B boards
#define TP6X_FLASH_AMD_EMBED_2MX8_DEV_ID	0xc8	// AMD embedded 2Mx8 FLASH used in TP64xx f/w rev.1B boards
#define TP6X_FLASH_AMD_EMBED_4MX8_DEV_ID	0xa3	// AMD embedded 4Mx8 FLASH used in TP64xx f/w rev.1B boards
#define TP6X_FLASH_AMD_EMBED_8MX8_DEV_ID	0x93	// AMD embedded 8Mx8 FLASH used in TP64xx f/w rev.1B boards

				// aliases for compatibility with TP6x DSP s/w API prior rev.4A (not recommended for usage)
#define TP6X_FLASH_AM29F010_DEV_ID			TP6X_FLASH_AMD_PLUGIN_128KX8_DEV_ID
#define TP6X_FLASH_AM29F040_DEV_ID			TP6X_FLASH_AMD_PLUGIN_512KX8_DEV_ID


    	// FLASH function declarations
    		// new FLASH API function names introduced in API rev.4A (recommended for usage)
int 	TP6X_get_flash_info(unsigned *manuf_id, unsigned *dev_id,unsigned *len,unsigned *sectors, unsigned *sector_len);
int 	TP6X_erase_flash_sector(unsigned a,unsigned vf);
int 	TP6X_erase_flash_chip(unsigned vf);
int 	TP6X_write_flash_data(unsigned a, unsigned d);
int 	TP6X_read_flash_data(unsigned a);
int 	TP6X_get_flash_sector_protection(unsigned a);
			// function name aliases for compatibility with TP6x API prior rev.4A (not recommended for usage)
#define	TP6X_flash_info(v1,v2,v3,v4,v5)			TP6X_get_flash_info(v1,v2,v3,v4,v5)
#define	TP6X_flash_erase_sector(v1,v2)			TP6X_erase_flash_sector(v1,v2)
#define	TP6X_flash_erase_chip(v)				TP6X_erase_flash_chip(v)
#define	TP6X_flash_write(v1,v2)					TP6X_write_flash_data(v1,v2)
#define	TP6X_flash_read(v)						TP6X_read_flash_data(v)
#define	TP6X_flash_sector_wrprot(v)				TP6X_get_flash_sector_protection(v)


	//--- general API functions
		// new recommended API functions for getting the error flags
int 	TP6X_get_last_err();
int 	TP6X_get_err_flag();
int 	TP6X_clr_err_flag();
		// aliases for compatibility with the API prior rev.4A (not recommended for usage)
#define	TP6X_last_err()		TP6X_get_last_err()
#define	TP6X_err_flag()		TP6X_get_err_flag()



	//--- returned error codes
#define	TP6X_OK								0		// no errors
#define	TP6X_PARAM_ERR						-1		// erroneous function parameter
#define	TP6X_INVALID_HW_ERR					-2		// invalid/incomaptible H/W error
#define	TP6X_HW_ERR							-3		// H/W error (error writing to the IOX regs, etc)
#define	TP6X_FLASH_INV_DEV_ERR				-100	// unsupported FLASH device
#define	TP6X_FLASH_ERASE_ERR				-101	// FLASH erase error
#define	TP6X_FLASH_WR_ERR					-102	// FLASH write error
#define	TP6X_FLASH_CANNOT_WR_ERR			-103	// FLASH contents does not allow to write this data
#define	TP6X_FLASH_INV_RD_ERR				-104	// invalid FLASH read data during AUTO-SELECT

//*************************************************************************


/***************** Extended TP6x DSP API functions ************************
 Extended TP6x DSP API functions include functions for DSP EMIF INIT and
 FLASH programming control. However, in order to reduce the size of DSP code
 and in order to exclude mutiple function body appearance, any of the Extended API
 functions can be included/excluded during the compilation process. Inclusion/exclusion
 of particular Extended API function is controled via global vars, which can be defined
 in user code to perferm a particular action:

	__TP6X_EXCLUDE_INIT_DSP_EMIF__
		- if defined in user code, then the DSP EMIF INIT function are excluded
		  from the user source code. Default is INCLUDED.

	__TP6X_INCLUDE_AMD_FLASH_CONTROL__
		- if defined in user code, then the AMD FLASH PROGRAMMING CONTROL function
		  are included into the user source code. Default is EXCLUDED.
*/


#ifndef __TP6X_EXCLUDE_INIT_DSP_EMIF__
//=============== DSP EMIF init function ===============================
// This function performs initialization of TP6x DSP EMIF configiration registers
// in accordance with the TP6x board type.
// Input parameter XMEM must be defined for TP6202/TP6203 boards as the following:
//     == TP6X_XMEM_SBSRAM (0) for configuring CE2 as SBSRAM and SDRAM disabled
//     == TP6X_XMEM_SDRAM  (4) for configuring CE2/CE3 for SDRAM and SBSRAM disabled		4
// TP64xx boards do not require input parameter.

#include "c6x_def.h"	// include C6x DSP defs

#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
int TP6X_init_dsp_emif(unsigned xmem_type)
{
	//~~~ TP62/TP620x/TP67 boards initialization.
	//    	For TP6202/TP6203 boards, process input parameter, which defines external
	//		DSP memory type.

    if(TP62_BOARD)
	{
	  	*C6X_EMIF_GCR=TP62_EMIF_GCR_DATA;
	  	*C6X_EMIF_CE0=TP62_EMIF_CE0_DATA;
	  	*C6X_EMIF_CE1=TP62_EMIF_CE1_DATA;
	  	*C6X_EMIF_CE2=TP62_EMIF_CE2_DATA;
	  	*C6X_EMIF_CE3=TP62_EMIF_CE3_DATA;
	  	*C6X_EMIF_SDRAM=TP62_EMIF_SDRAM_DATA;
	  	*C6X_EMIF_SDRAM_TIMER=TP62_EMIF_SDRAM_TIMER_DATA;
	  	return(TP6X_OK);
	}

    if(TP67_BOARD)
	{
	  	*C6X_EMIF_GCR=TP67_EMIF_GCR_DATA;
	  	*C6X_EMIF_CE0=TP67_EMIF_CE0_DATA;
	  	*C6X_EMIF_CE1=TP67_EMIF_CE1_DATA;
	  	*C6X_EMIF_CE2=TP67_EMIF_CE2_DATA;
	  	*C6X_EMIF_CE3=TP67_EMIF_CE3_DATA;
	  	*C6X_EMIF_SDRAM=TP67_EMIF_SDRAM_DATA;
	  	*C6X_EMIF_SDRAM_TIMER=TP67_EMIF_SDRAM_TIMER_DATA;
	  	return(TP6X_OK);
	}

		// validate input parameter for TP6202/TP6203 board.
		// Return error in case input parameter is not valid.
    if(xmem_type!=TP6X_XMEM_SBSRAM && xmem_type!=TP6X_XMEM_SDRAM) return(TP6X_PARAM_ERR);

    if(TP6202_BOARD)
	{
	  	*C6X_EMIF_GCR=TP6202_EMIF_GCR_DATA;
	  	*C6X_EMIF_CE0=TP6202_EMIF_CE0_DATA;
	  	*C6X_EMIF_CE1=TP6202_EMIF_CE1_DATA;
	  	if(xmem_type==TP6X_XMEM_SBSRAM)	// check for external memory configuration
		{
		  TP6X_set_xmem_sbsram();
	  	  *C6X_EMIF_CE2=TP6202_EMIF_CE2_SBSRAM_DATA;
	  	  *C6X_EMIF_CE3=TP6202_EMIF_CE3_SBSRAM_DATA;
		}
		else
		{
		  	TP6X_set_xmem_sdram();
	  	  	*C6X_EMIF_CE2=TP6202_EMIF_CE2_SDRAM_DATA;
		  	*C6X_EMIF_CE3=TP6202_EMIF_CE3_SDRAM_DATA;
		}

		*C6X_EMIF_SDRAM=TP6202_EMIF_SDRAM_DATA;
		*C6X_EMIF_SDRAM_TIMER=TP6202_EMIF_SDRAM_TIMER_DATA;
   	   	return(TP6X_OK);
	}


    if(TP6203_BOARD)
	{
	  	*C6X_EMIF_GCR=TP6203_EMIF_GCR_DATA;
	  	*C6X_EMIF_CE0=TP6203_EMIF_CE0_DATA;
	  	*C6X_EMIF_CE1=TP6203_EMIF_CE1_DATA;
	  	if(xmem_type==TP6X_XMEM_SBSRAM)	// check for external memory configuration
		{
		  TP6X_set_xmem_sbsram();
	  	  *C6X_EMIF_CE2=TP6203_EMIF_CE2_SBSRAM_DATA;
	  	  *C6X_EMIF_CE3=TP6203_EMIF_CE3_SBSRAM_DATA;
		}
		else
		{
		  TP6X_set_xmem_sdram();
	  	  *C6X_EMIF_CE2=TP6203_EMIF_CE2_SDRAM_DATA;
		  *C6X_EMIF_CE3=TP6203_EMIF_CE3_SDRAM_DATA;
		}

	  	*C6X_EMIF_SDRAM=TP6203_EMIF_SDRAM_DATA;
	  	*C6X_EMIF_SDRAM_TIMER=TP6203_EMIF_SDRAM_TIMER_DATA;
		return(TP6X_OK);
	}

	// getting here denotes that the board ID is incorrect
	return (TP6X_INVALID_HW_ERR);
#endif

#ifdef	__TP6X_SELECT_TP64XX__
int TP6X_init_dsp_emif()
{
	//~~~ TP64xx boards initialization.

		// The TP64x boards may differ in SDRAM capacity installed, DSP clock (EMIF clock),
		// and board f/w revision, which result in different EMIF control register settings.

			// at first, set the EMIF control registers, which settings do not depend upon the
			// board f/w revision, DSP/EMIF clock and SDRAM capacity installed.

				// EMIF-A registers.
	*C64XX_EMIFA_GCR 		= TP64XX_EMIFA_GCR_DATA;
	*C64XX_EMIFA_CE0 		= TP64XX_EMIFA_CE0_DATA;
	*C64XX_EMIFA_CE0_SEC 	= TP64XX_EMIFA_CE0_SEC_DATA;
	*C64XX_EMIFA_CE1 		= TP64XX_EMIFA_CE1_DATA;
	*C64XX_EMIFA_CE1_SEC 	= TP64XX_EMIFA_CE1_SEC_DATA;
	*C64XX_EMIFA_CE2 		= TP64XX_EMIFA_CE2_DATA;
	*C64XX_EMIFA_CE2_SEC 	= TP64XX_EMIFA_CE2_SEC_DATA;
	*C64XX_EMIFA_CE3 		= TP64XX_EMIFA_CE3_DATA;

				// set EMIF-B registers
	*C64XX_EMIFB_GCR 		= TP64XX_EMIFB_GCR_DATA;
	*C64XX_EMIFB_CE2 		= TP64XX_EMIFB_CE2_DATA;
				// set EMIF-B@CE0 area to the extended preset setting initially in order to
				// configure it as 16-bit ASYNC area (dafaults to 8-bit ASYNC on DSP reset) and
				// in order to be able to process the revision ID and SBSRAM/SDRAM capacity ID.
	*C64XX_EMIFB_CE0 		= TP64XX_EMIFB_CE0_DATA_PRESET;



	// SDRAM control must be initialized upon the SDRAM capacity
	// 	1. For f/w rev.1A, the SDRAM control is set upon the setting of the __TP6X_SDRAM128M__
	//	   compiler key def.
	// 	2. For f/w rev.1A-1 and rev.1A-2, the __TP6X_SDRAM128M__ compiler key is ignored and the
	//	   SDRAM control is set upon the SDRAM length ID and board f/w revision.

	if(TP6X_REV_ID_IS_REV1A)
	{
		if(TP6X_SDRAM_LEN_IS_16M)
		{
			// configure for 600MHz DSP
			*C64XX_EMIFA_SDRAM 		= TP64XX_EMIFA_SDRAM16M_DATA_REV1A_600MHZ;
			*C64XX_EMIFA_SDRAM_TIMER= TP64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A_600MHZ;
		}

		if(TP6X_SDRAM_LEN_IS_4M)
		{
			// configure for TMS silicon with 600MHz DSP
			*C64XX_EMIFA_SDRAM 		= TP64XX_EMIFA_SDRAM4M_DATA_REV1A_600MHZ;
			*C64XX_EMIFA_SDRAM_TIMER= TP64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A_600MHZ;
		}

		// EMIF-B initialization
		*C64XX_EMIFA_SDRAM_EXT 	= TP64XX_EMIFA_SDRAM_EXT_DATA_REV1A_600MHZ;
		*C64XX_EMIFB_CE0 		= TP64XX_EMIFB_CE0_DATA_REV1A_600MHZ;
		*C64XX_EMIFB_CE1 		= TP64XX_EMIFB_CE1_DATA_REV1A_600MHZ;
		*C64XX_EMIFB_CE3 		= TP64XX_EMIFB_CE3_DATA_REV1A_600MHZ;

	    return(TP6X_OK);
	}


	if(TP6X_REV_ID_IS_REV1A1)
	{
		if(TP6X_SDRAM_LEN_IS_16M)
		{
			// configure for 16Mx64 SDRAM
			*C64XX_EMIFA_SDRAM 		= TP64XX_EMIFA_SDRAM16M_DATA_REV1A1;
			*C64XX_EMIFA_SDRAM_TIMER= TP64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A1;
		}

		if(TP6X_SDRAM_LEN_IS_4M)
		{
			// configure for 4Mx64 SDRAM
			*C64XX_EMIFA_SDRAM 		= TP64XX_EMIFA_SDRAM4M_DATA_REV1A1;
			*C64XX_EMIFA_SDRAM_TIMER= TP64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A1;
		}

		if(TP6X_SDRAM_LEN_IS_0M)
		{
			// if SDRAM is not installed, then re-configure EMIF-A@CE2 area, which is reserved for SDRAM, as
			// the SBSRAM area (EMIF-A@CE0) in order to exclude SDRAM refresh cycles and increase the
			// system performance. The SDRAM control register and SDRAM timer register are configured as
			// that for 4Mx64 SDRAM in order to provide correct operation.
			*C64XX_EMIFA_CE2 		= TP64XX_EMIFA_CE0_DATA;
			*C64XX_EMIFA_SDRAM 		= TP64XX_EMIFA_SDRAM4M_DATA_REV1A1;
			*C64XX_EMIFA_SDRAM_TIMER= TP64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A1;
		}

		// EMIF-B initialization
		*C64XX_EMIFA_SDRAM_EXT 	= TP64XX_EMIFA_SDRAM_EXT_DATA_REV1A1;
		*C64XX_EMIFB_CE0 		= TP64XX_EMIFB_CE0_DATA_REV1A1;
		*C64XX_EMIFB_CE1 		= TP64XX_EMIFB_CE1_DATA_REV1A1;
		*C64XX_EMIFB_CE3 		= TP64XX_EMIFB_CE3_DATA_REV1A1;

	    return(TP6X_OK);
	}


	if(TP6X_REV_ID_IS_REV1A2)
	{
		if(TP6X_SDRAM_LEN_IS_16M)
		{
			// configure for 16Mx64 SDRAM
			*C64XX_EMIFA_SDRAM 		= TP64XX_EMIFA_SDRAM16M_DATA_REV1A2;
			*C64XX_EMIFA_SDRAM_TIMER= TP64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A2;
		}

		if(TP6X_SDRAM_LEN_IS_4M)
		{
			// configure for 4Mx64 SDRAM
			*C64XX_EMIFA_SDRAM 		= TP64XX_EMIFA_SDRAM4M_DATA_REV1A2;
			*C64XX_EMIFA_SDRAM_TIMER= TP64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A2;
		}

		if(TP6X_SDRAM_LEN_IS_0M)
		{
			// if SDRAM is not installed, then re-configure EMIF-A@CE2 area, which is reserved for SDRAM, as
			// the SBSRAM area (EMIF-A@CE0) in order to exclude SDRAM refresh cycles and increase the
			// system performance. The SDRAM control register and SDRAM timer register are configured as
			// that for 4Mx64 SDRAM in order to provide correct operation.
			*C64XX_EMIFA_CE2 		= TP64XX_EMIFA_CE0_DATA;
			*C64XX_EMIFA_SDRAM 		= TP64XX_EMIFA_SDRAM4M_DATA_REV1A2;
			*C64XX_EMIFA_SDRAM_TIMER= TP64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A2;
		}

		// EMIF-B initialization
		*C64XX_EMIFA_SDRAM_EXT 	= TP64XX_EMIFA_SDRAM_EXT_DATA_REV1A2;
		*C64XX_EMIFB_CE0 		= TP64XX_EMIFB_CE0_DATA_REV1A2;
		*C64XX_EMIFB_CE1 		= TP64XX_EMIFB_CE1_DATA_REV1A2;
		*C64XX_EMIFB_CE3 		= TP64XX_EMIFB_CE3_DATA_REV1A2;

	    return(TP6X_OK);
	}


	if(TP6X_REV_ID_IS_REV1B)
	{
		if(TP6X_SDRAM_LEN_IS_16M)
		{
			// configure for 16Mx64 SDRAM
			*C64XX_EMIFA_SDRAM 		= TP64XX_EMIFA_SDRAM16M_DATA_REV1B;
			*C64XX_EMIFA_SDRAM_TIMER= TP64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1B;
		}

		if(TP6X_SDRAM_LEN_IS_4M)
		{
			// configure for 4Mx64 SDRAM
			*C64XX_EMIFA_SDRAM 		= TP64XX_EMIFA_SDRAM4M_DATA_REV1B;
			*C64XX_EMIFA_SDRAM_TIMER= TP64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1B;
		}

		if(TP6X_SDRAM_LEN_IS_0M)
		{
			// if SDRAM is not installed, then re-configure EMIF-A@CE2 area, which is reserved for SDRAM, as
			// the SBSRAM area (EMIF-A@CE0) in order to exclude SDRAM refresh cycles and increase the
			// system performance. The SDRAM control register and SDRAM timer register are configured as
			// that for 4Mx64 SDRAM in order to provide correct operation.
			*C64XX_EMIFA_CE2 		= TP64XX_EMIFA_CE0_DATA;
			*C64XX_EMIFA_SDRAM 		= TP64XX_EMIFA_SDRAM4M_DATA_REV1B;
			*C64XX_EMIFA_SDRAM_TIMER= TP64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1B;
		}

		// EMIF-B initialization
		*C64XX_EMIFA_SDRAM_EXT 	= TP64XX_EMIFA_SDRAM_EXT_DATA_REV1B;
		*C64XX_EMIFB_CE0 		= TP64XX_EMIFB_CE0_DATA_REV1B;
		*C64XX_EMIFB_CE1 		= TP64XX_EMIFB_CE1_DATA_REV1B;
		*C64XX_EMIFB_CE3 		= TP64XX_EMIFB_CE3_DATA_REV1B;

	    return(TP6X_OK);
	}


	// getting here denotes that the f/w revision ID is incorrect
	return (TP6X_INVALID_HW_ERR);

#endif
}
//======================================================================
#endif


#ifdef __TP6X_INCLUDE_AMD_FLASH_CONTROL__
//=============== general extended API functions ===============================
/*
    GENERAL EXTENDED API FUNCTIONS shall be included in case of any extended
    API functions have been included.

    General extended TP6X API functions are used to handle the run-time
    errors for FLASH programming control and other extended API functions:
	TP6X_last_err
	TP6X_err_flag
	TP6X_clr_err_flag

    Each function returns error code (<0). In case the function must also
    return data (as TP6X_flash_read()), then data can be recognized in case
    returned code is >=0, and error is recognized in case returned code is
    <0.
*/

static	int _tp6x_last_err =0,
	    _tp6x_err_flag =0;


int TP6X_get_last_err()
	// TP6X_LAST_ERR function returns errpr code of last executed
	// API function.
{
	return(_tp6x_last_err);
}


int TP6X_get_err_flag()
	// TP6X_ERR_FLAG function returns status of ERR_FLAG for API
	// functions. ERR_FLAG is set by each API function in case
	// error has been detected. ERR_FLAG can be reset by
	// TP6X_CLR_ERR_FLAG API function
{
	return(_tp6x_err_flag);
}

int TP6X_clr_err_flag()
{
   _tp6x_err_flag=TP6X_OFF;
   return(TP6X_OK);
}

	// local/non-API function,
	// used on exit fm API functions in order to set error code
static int _tp6x_set_err(int e)
{
	if(e<0)
    {
		_tp6x_err_flag=1;
       	_tp6x_last_err=e;
    }
    else
       _tp6x_last_err=TP6X_OK;

   	return(e);
}

//======================================================================
#endif


#ifdef __TP6X_INCLUDE_AMD_FLASH_CONTROL__
//=============== AMD FLASH PROGRAMMING CONTROL function ===============================
/*
	FLASH Programming Utilites shall be used to 'burn' the TP6x/TP64xx on-board FLASH memory.

    The following is a list of user FLASH Programming API functions:
		TP6X_get_flash_info(unsigned *manuf_id, unsigned *dev_id,unsigned *len,unsigned *sectors, unsigned *sector_len);
		TP6X_erase_flash_sector(unsigned a,unsigned vf);
		TP6X_erase_flash_chip(unsigned vf);
		TP6X_write_flash_data(unsigned a, unsigned d);
		TP6X_read_flash_data(unsigned a);
		TP6X_get_flash_sector_protection(unsigned a);

    At the runtime, FLASH programming utilities automatically detect presence of supported FLASH memory
	chip, and adjust internal environment to match the FLASH chip capacity. FLASH programming utilities
	automatically detect the board type and TP64xx f/w revision info in order to support different
	FLASH memory chips upon the board type.


	IMPORTANT NOTICES !!!
   	----------------------
   	   	1. FLASH programming utilities cannot be used to program the plugged-in EPROM memory chips
   	   	   in PLCC-32 IC package for TP6x and TP64xx f/w rev.1A/1A-1/1A-2 boards.
   	   	2. FLASH programming utilities for TP6x and TP64xx f/w rev.1A/1A-1/1A-2 boards have been
		   designed to support AMD Am29F010B/Am29F040B 5v-only FLASH memory chips only (PLCC-32 IC package)
           and will not run perfectly with FLASH memory chips from other manufacturers.
		3. For TP6x boards and TP64xx f/w rev.1A/1A-1/1A-2 boards, the FLASH programming
		   utilities (except for the TP6X_FLASH_READ function) require that the FLASH WRITE
		   feature is enabled via on-board J2 jumper set, otherwise the TP6X_FLASH_INVALID_ERR error
		   will be returned and no FLASH access will be done. This is due the FLASH programming utilies
    	   are auto-configured via the FLASH MANUF/DEVICE info, which is read from the FLASH chip
		   using the AUTO-SELECT command, which requires that the FLASH memory write is enabled.

*/

// TI C Compiler standard header
#include "stddef.h"

// local defs and static vars
#define	__TP6X_FLASH_LEN_128KX8						0x00020000
#define	__TP6X_FLASH_LEN_512KX8						0x00080000
#define	__TP6X_FLASH_LEN_1MX8						0x00100000
#define	__TP6X_FLASH_LEN_2MX8						0x00200000
#define	__TP6X_FLASH_LEN_4MX8						0x00400000
#define	__TP6X_FLASH_LEN_8MX8						0x00800000


#define __TP6X_FLASH_AMD_PLUGIN_128KX8_SECTORS		8
#define __TP6X_FLASH_AMD_PLUGIN_512KX8_SECTORS		8
#define __TP6X_FLASH_AMD_EMBED_512KX8_SECTORS		8
#define __TP6X_FLASH_AMD_EMBED_1MX8_SECTORS			16
#define __TP6X_FLASH_AMD_EMBED_2MX8_SECTORS			32
#define __TP6X_FLASH_AMD_EMBED_4MX8_SECTORS			64
#define __TP6X_FLASH_AMD_EMBED_8MX8_SECTORS			128



static int	_tp6x_flash_open_flag	= 0,
			_tp6x_flash_manuf_id	= 0,
			_tp6x_flash_dev_id	= 0,
			_tp6x_flash_len		= 0,
			_tp6x_flash_sectors	= 0,
			_tp6x_flash_sector_len  = 0;



static int _tp6x_flash_open()
	// local/non-API function, which is used to check whether the FLASH API has been open
	// in order to identify FLASH chip info and configure FLASH API environment
	// returns:
	//		OK: if opened successfully
	//		<error>: if any error has occurred
{
 	__TP6X_FLASH_DATA_TYPE__ *fptr,rd;
 	int r;

  	// Check for FLASH API has been open already.
	// If not open, then open it.
	if (_tp6x_flash_open_flag==TP6X_ON) return(TP6X_OK);	// exit if already opened

	// not open yet, now try to open FLASH utilities
	_tp6x_flash_open_flag=TP6X_OFF;
    fptr = (__TP6X_FLASH_DATA_TYPE__ *) TP6X_FLASH_SADDR;

	// read MANUFACTURER ID and DEVICE ID
    // assume error initially
	r=TP6X_FLASH_INV_DEV_ERR;
	fptr[0x555] = 0xaa;
    fptr[0x2aa] = 0x55;
    fptr[0x555] = 0x90;
    	// read nad verify MANUF ID
    _tp6x_flash_manuf_id = fptr[0] & TP6X_FLASH_DATA_BITMASK;
    if(_tp6x_flash_manuf_id != TP6X_FLASH_AMD_MANUF_ID) goto exit;

		// Read and verify the device ID.
		// Check the FLASH ID against the board type and check the FLASH CAPACITY ID (from the XMEM2_LEN_ID register)
		// matches the recognized length of the FLASH chip for TP64xx rev.1B boards with embedded FLASH memory.
		// Fill-in the FLASH length , number of sectors, & sector len local vars.
    _tp6x_flash_dev_id = fptr[1] & TP6X_FLASH_DATA_BITMASK;
    switch(_tp6x_flash_dev_id)
    {
			// plug-in 128K/512Kx8 FLASH memories for TP6x boards and TP64xx rev.1A/1A-1/1A-2 boards
       	case TP6X_FLASH_AMD_PLUGIN_128KX8_DEV_ID:
			// check whether this is either TP6x board or TP64xx rev.1A/1A1/1A2 board, exit with error otherwise
			if(
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
				!TP62_TP620X_TP67_BOARD
#endif
#ifdef	__TP6X_SELECT_TP64XX__
				!TP64XX_REV1A_REV1A1_REV1A2_BOARD
#endif
			  )
			{
				r= TP6X_INVALID_HW_ERR;
				goto exit;
			}
			_tp6x_flash_len		= __TP6X_FLASH_LEN_128KX8;
			_tp6x_flash_sectors	= __TP6X_FLASH_AMD_PLUGIN_128KX8_SECTORS;
			break;

		case TP6X_FLASH_AMD_PLUGIN_512KX8_DEV_ID:
			// check whether this is either TP6x board or TP64xx rev.1A/1A1/1A2 board, exit with error otherwise
			if(
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
				!TP62_TP620X_TP67_BOARD
#endif
#ifdef	__TP6X_SELECT_TP64XX__
				!TP64XX_REV1A_REV1A1_REV1A2_BOARD
#endif
			  )
			{
				r= TP6X_INVALID_HW_ERR;
				goto exit;
			}
			_tp6x_flash_len		= __TP6X_FLASH_LEN_512KX8;
			_tp6x_flash_sectors	= __TP6X_FLASH_AMD_PLUGIN_512KX8_SECTORS;
			break;


			// embedded FLASH memories for TP64xx rev.1B boards
		case TP6X_FLASH_AMD_EMBED_512KX8_DEV_ID:
			// check whether this is the TP64xx rev.1B board and whether the FLASH CAPACITY ID matches, exit with error otherwise
			if(
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
				TP62_TP620X_TP67_BOARD
#endif
#ifdef	__TP6X_SELECT_TP64XX__
					!TP64XX_REV1B_BOARD
				||	!TP6X_FLASH_LEN_IS_512K
#endif
			  )
			{
				r= TP6X_INVALID_HW_ERR;
				goto exit;
			}
			_tp6x_flash_len		= __TP6X_FLASH_LEN_512KX8;
			_tp6x_flash_sectors	= __TP6X_FLASH_AMD_EMBED_512KX8_SECTORS;
			break;

		case TP6X_FLASH_AMD_EMBED_1MX8_DEV_ID:
			// check whether this is the TP64xx rev.1B board and whether the FLASH CAPACITY ID matches, exit with error otherwise
			if(
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
				TP62_TP620X_TP67_BOARD
#endif
#ifdef	__TP6X_SELECT_TP64XX__
					!TP64XX_REV1B_BOARD
				||	!TP6X_FLASH_LEN_IS_1M
#endif
			  )
			{
				r= TP6X_INVALID_HW_ERR;
				goto exit;
			}
			_tp6x_flash_len		= __TP6X_FLASH_LEN_1MX8;
			_tp6x_flash_sectors	= __TP6X_FLASH_AMD_EMBED_1MX8_SECTORS;
			break;

		case TP6X_FLASH_AMD_EMBED_2MX8_DEV_ID:
			// check whether this is the TP64xx rev.1B board and whether the FLASH CAPACITY ID matches, exit with error otherwise
			if(
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
				TP62_TP620X_TP67_BOARD
#endif
#ifdef	__TP6X_SELECT_TP64XX__
					!TP64XX_REV1B_BOARD
				||	!TP6X_FLASH_LEN_IS_2M
#endif
			  )
			{
				r= TP6X_INVALID_HW_ERR;
				goto exit;
			}
			_tp6x_flash_len		= __TP6X_FLASH_LEN_2MX8;
			_tp6x_flash_sectors	= __TP6X_FLASH_AMD_EMBED_2MX8_SECTORS;
			break;

		case TP6X_FLASH_AMD_EMBED_4MX8_DEV_ID:
			// check whether this is the TP64xx rev.1B board and whether the FLASH CAPACITY ID matches, exit with error otherwise
			if(
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
				TP62_TP620X_TP67_BOARD
#endif
#ifdef	__TP6X_SELECT_TP64XX__
					!TP64XX_REV1B_BOARD
				||	!TP6X_FLASH_LEN_IS_4M
#endif
			  )
			{
				r= TP6X_INVALID_HW_ERR;
				goto exit;
			}
			_tp6x_flash_len		= __TP6X_FLASH_LEN_4MX8;
			_tp6x_flash_sectors	= __TP6X_FLASH_AMD_EMBED_4MX8_SECTORS;
			break;

		case TP6X_FLASH_AMD_EMBED_8MX8_DEV_ID:
			// check whether this is the TP64xx rev.1B board and whether the FLASH CAPACITY ID matches, exit with error otherwise
			if(
#ifdef	__TP6X_SELECT_TP62_TP67_TP620X__
				TP62_TP620X_TP67_BOARD
#endif
#ifdef	__TP6X_SELECT_TP64XX__
					!TP64XX_REV1B_BOARD
				||	!TP6X_FLASH_LEN_IS_8M
#endif
			  )
			{
				r= TP6X_INVALID_HW_ERR;
				goto exit;
			}
			_tp6x_flash_len		= __TP6X_FLASH_LEN_8MX8;
			_tp6x_flash_sectors	= __TP6X_FLASH_AMD_EMBED_8MX8_SECTORS;
			break;

		default:
			// unsupported FLASH device, exit with error
			goto exit;
    }

	// FLASH device identification has passed OK. Set FLASH sector length.
	// Set FLASH utilities OPEN flag in order to exclude this procedure for all further API function calls.
	_tp6x_flash_sector_len  = _tp6x_flash_len/_tp6x_flash_sectors;


	// finally, set the FLASH API OPEN FLAG so all further called FLASH API proceed with the known FLASH info
	_tp6x_flash_open_flag=TP6X_ON;
	r=TP6X_OK;

exit:
	// execute RESET command in order to release device from the FLASH command state and enable further reads
   	fptr[0] = 0xf0;
   	rd = *fptr;

   	// execute false operator in order to exclude warning message, which appears since RD var is never used.
	r = r | (rd&0);

    // return error code
    return(r);
}



int 	TP6X_get_flash_info(unsigned *manuf_id, unsigned *dev_id,
		    				unsigned *len,unsigned *sectors, unsigned *sector_len)
	// Returns FLASH chip info (manuf code, device code, etc.) via suplied params pointers.
	// Returns:
	//		OK: if flash info is available
	//		<error>: if any error has occurred.
{
	int r;

	// Check for the FLASH API has been open already.
	if ((r=_tp6x_flash_open())!=TP6X_OK) return(_tp6x_set_err(r));

	if(manuf_id!=NULL) 		*manuf_id   =_tp6x_flash_manuf_id;
	if(dev_id!=NULL) 		*dev_id     =_tp6x_flash_dev_id;
	if(len!=NULL) 			*len	    =_tp6x_flash_len;
	if(sectors!=NULL)		*sectors    =_tp6x_flash_sectors;
	if(sector_len!=NULL)	*sector_len =_tp6x_flash_sector_len;

	return(_tp6x_set_err(TP6X_OK));
}



int 	TP6X_get_flash_sector_protection(unsigned a)
	// Returns FLASH sector write-protection status for the sector, which corresponds to the address A.
	// Returns:
	//			<error>: if any error has occurred.
	//          <0>  - sector is not protected
	//          <1>  - sector is protected
{
	int r;
  	__TP6X_FLASH_DATA_TYPE__ *fptr,rd;

	// Check for FLASH API has been open already.
	if ((r=_tp6x_flash_open())!=TP6X_OK) return(_tp6x_set_err(r));

	// validate params
	if (a>=_tp6x_flash_len) return(_tp6x_set_err(TP6X_PARAM_ERR));

	// set FLASH pointer and sector#
	fptr = (__TP6X_FLASH_DATA_TYPE__ *)TP6X_FLASH_SADDR;


	// Set FLASH_PAGE[0..2] bits for TP64xx rev.1B board.
	// Modify the <A> input flash address so it becomes aligned within the current FLASH PAGE in order to keep the code clear and simple.
#ifdef	__TP6X_SELECT_TP64XX__
		// set and verify the FLASH_PAGE[0..2] bits for TP64xx rev1B board
	if(TP64XX_REV1B_BOARD)
	{
			// set FPASH PAGE bits
		TP6X_set_flash_page(a/TP64XX_FLASH_PAGE_LEN);
			// verify FLASH PAGE bits
		if(TP6X_get_flash_page() != (a/TP64XX_FLASH_PAGE_LEN))
			return(_tp6x_set_err(TP6X_HW_ERR));

			// modify the FLASH address parameter to align it within the FLASH page boundary
		a %= TP64XX_FLASH_PAGE_LEN;
	}
#endif


	// execute GET SECTOR PROTECTION FLAG COMMAND for sector protection status
    fptr[0x555]=0xaa;
    fptr[0x2aa]=0x55;
    fptr[0x555]=0x90;

	// read & verify sector wr-protection status (==0 or ==1)
	// Note, that we must read data at the <sector_start_address +2> in order to get the sector write protection feature.
		// assume error
	r=TP6X_FLASH_INV_RD_ERR;

	// Read sector protection flag from the <sector_start_addr+2>.
	rd= fptr[((a/_tp6x_flash_sector_len)*_tp6x_flash_sector_len) +2] & TP6X_FLASH_DATA_BITMASK;

    // verify read data (must be either '0' or '1')
	if(rd!=0 && rd!=1)
		goto exit;

    // exit OK
	r=TP6X_OK;

exit:
	// execute RESET command in order to release device from the FLASH command state and enable further reads
   	fptr[0] = 0xf0;
   	rd = *fptr;

   	// execute false operator in order to exclude warning message, which appears since RD var is never used.
	r = r | (rd&0);

    // return either error code or the sector protection status
    return(_tp6x_set_err(r<0? r:rd));
}





int 	TP6X_erase_flash_sector(unsigned a,unsigned vf)
	// Erase sector, which corresponds to the FLASH address A. Verify erased sector data if VF==1.
	// returns:
	//		OK: no errors
	//		<error>: if any error occurrs
{
	__TP6X_FLASH_DATA_TYPE__ *fptr, rd;
  	unsigned sa, i;
  	int r;

	// Check for FLASH API has been open already.
	if ((r=_tp6x_flash_open())!=TP6X_OK) return(_tp6x_set_err(r));

	// validate params
	if (   a>=_tp6x_flash_len
	    || vf>1
	   )
		return(_tp6x_set_err(TP6X_PARAM_ERR));


	// set FLASH array pointer
	fptr = (__TP6X_FLASH_DATA_TYPE__ *)TP6X_FLASH_SADDR;


	// Set FLASH_PAGE[0..2] bits for TP64xx rev.1B board.
	// Modify the <A> input flash address so it becomes aligned within the current FLASH PAGE in order to keep the code clear and simple.
#ifdef	__TP6X_SELECT_TP64XX__
		// set and verify the FLASH_PAGE[0..2] bits for TP64xx rev1B board
	if(TP64XX_REV1B_BOARD)
	{
			// set FPASH PAGE bits
		TP6X_set_flash_page(a/TP64XX_FLASH_PAGE_LEN);
			// verify FLASH PAGE bits
		if(TP6X_get_flash_page() != (a/TP64XX_FLASH_PAGE_LEN))
			return(_tp6x_set_err(TP6X_HW_ERR));

			// modify the FLASH address parameter to align it within the FLASH page boundary
		a %= TP64XX_FLASH_PAGE_LEN;
	}
#endif

	// set FLASH sector# (aligned to the FLASH page)
	sa = (a/_tp6x_flash_sector_len);

	// execute SECTOR ERASE command
    fptr[0x555]=0xaa;
    fptr[0x2aa]=0x55;
    fptr[0x555]=0x80;
    fptr[0x555]=0xaa;
    fptr[0x2aa]=0x55;
	fptr[a]    =0x30;

	while(1)
	{
		rd = fptr[a] & TP6X_FLASH_DATA_BITMASK;

		// verify read data
		if(rd==0xff) break;		// exit OK

	  	// if not OK, then check state of bit D5 as a timeout/failure condition
		if((rd&0x20)!=0)
	    {
			// if timeout, then read again
	    	rd = fptr[a] & TP6X_FLASH_DATA_BITMASK;

			// verify read data
            if(rd==0xff)
            	// completed successfully, exit
				break;

            // error and exit
			// execute RESET command in order to release device from the ERASE state
   			fptr[0] = 0xf0;
			return(_tp6x_set_err(TP6X_FLASH_ERASE_ERR));
	    }
	}

	// perform erased sector data verification if requested (must read as the 'FF' for correctly erased FLASH memory)
    if(vf!=0)
	{
		for(i=0, fptr = (__TP6X_FLASH_DATA_TYPE__ *)(TP6X_FLASH_SADDR + (sa*_tp6x_flash_sector_len));
            i<_tp6x_flash_sector_len;
            i++,fptr++)
		{
			if((*fptr &TP6X_FLASH_DATA_BITMASK)!=0xff)
		      	return(_tp6x_set_err(TP6X_FLASH_ERASE_ERR));
		}
	}

	// exit successfully
	return(_tp6x_set_err(TP6X_OK));
}




int 	TP6X_erase_flash_chip(unsigned vf)
	// Erase all sectors of FLASH. Verify erased data if VF==1.
	// returns:  error code
{
	__TP6X_FLASH_DATA_TYPE__ *fptr,rd;
  	unsigned i;
  	int r;


	// Check for FLASH API has been open already.
	if ((r=_tp6x_flash_open())!=TP6X_OK) return(_tp6x_set_err(r));

	// validate params
	if (vf>1) return(_tp6x_set_err(TP6X_PARAM_ERR));

	// set FLASH pointer
	fptr = (__TP6X_FLASH_DATA_TYPE__ *)TP6X_FLASH_SADDR;

	// Reset the FLASH_PAGE[0..2] bits for TP64xx rev.1B board.
#ifdef	__TP6X_SELECT_TP64XX__
		// set and verify the FLASH_PAGE[0..2] bits for TP64xx rev1B board
	if(TP64XX_REV1B_BOARD)
	{
			// set FPASH PAGE bits
		TP6X_set_flash_page(0);
			// verify FLASH PAGE bits
		if(TP6X_get_flash_page() != 0)
			return(_tp6x_set_err(TP6X_HW_ERR));
	}
#endif


	// execute ERASE CHIP command
    fptr[0x555]=0xaa;
    fptr[0x2aa]=0x55;
    fptr[0x555]=0x80;
    fptr[0x555]=0xaa;
    fptr[0x2aa]=0x55;
	fptr[0x555]=0x10;

	while(1)
	{
		rd = *fptr & TP6X_FLASH_DATA_BITMASK;

		// verify read data
	  	if(rd==0xff) break;

	  	// if not OK, then check state of bit D5 as a timeout/failure condition
	  	if((rd&0x20)!=0)
	    {
			// if timeout, then read again
	    	rd = *fptr & TP6X_FLASH_DATA_BITMASK;

			// verify read data
            if(rd==0xff)
            	// completed successfully, exit
				break;

            // error and exit
			// execute RESET command in order to release device from the ERASE state
   			fptr[0] = 0xf0;
			return(_tp6x_set_err(TP6X_FLASH_ERASE_ERR));
	    }
	}

	// perform erased chip data verification if requested
    if(vf!=0)
	{
		for(i=0L;i<_tp6x_flash_len;i++,fptr++)
		{
			// Set the FLASH_PAGE[0..2] bits for TP64xx rev.1B board.
			// Reset the <FPTR> pointer to the FLASH PAGE start in case FLASH PAGE change is detected via the <I> parameter.
#ifdef	__TP6X_SELECT_TP64XX__
			// set and verify the FLASH_PAGE[0..2] bits for TP64xx rev1B board
			if(		TP64XX_REV1B_BOARD
				&& 	i!=0
				&& 	(i%TP64XX_FLASH_PAGE_LEN)==0
			  )
			{
				// set FPASH PAGE bits
				TP6X_set_flash_page(i/TP64XX_FLASH_PAGE_LEN);
				// verify FLASH PAGE bits
				if(TP6X_get_flash_page() != (i/TP64XX_FLASH_PAGE_LEN))
					return(_tp6x_set_err(TP6X_HW_ERR));

				// reset the FPTR parameter to the FLASH PAGE start address.
				fptr = (__TP6X_FLASH_DATA_TYPE__ *)TP6X_FLASH_SADDR;
			}
#endif

			// read and verify the FLASH data (must read as the 'FF' for correctly erased FLASH memory)
			if((*fptr &TP6X_FLASH_DATA_BITMASK)!=0xff)
		      	return(_tp6x_set_err(TP6X_FLASH_ERASE_ERR));
		}
	}

	// Reset the FLASH_PAGE[0..2] bits for TP64xx rev.1B board.
#ifdef	__TP6X_SELECT_TP64XX__
		// set and verify the FLASH_PAGE[0..2] bits for TP64xx rev1B board
	if(TP64XX_REV1B_BOARD)
	{
			// set FPASH PAGE bits
		TP6X_set_flash_page(0);
			// verify FLASH PAGE bits
		if(TP6X_get_flash_page() != 0)
			return(_tp6x_set_err(TP6X_HW_ERR));
	}
#endif

	// exit successfully
	return(_tp6x_set_err(TP6X_OK));
}



int 	TP6X_write_flash_data(unsigned a, unsigned d)
	// writes D (bits 0..7 are valid) to FLASH addr #A
	// returns:  error code
{
	__TP6X_FLASH_DATA_TYPE__ *fptr,rd;
  	int r;


	// Check for FLASH API has been open already.
	if ((r=_tp6x_flash_open())!=TP6X_OK) return(_tp6x_set_err(r));

	// validate params
	if (   a>=_tp6x_flash_len
	    || d>TP6X_FLASH_DATA_BITMASK
	   )
		return(_tp6x_set_err(TP6X_PARAM_ERR));

	// set FLASH pointer
	fptr = (__TP6X_FLASH_DATA_TYPE__ *)TP6X_FLASH_SADDR;


	// Set the FLASH_PAGE[0..2] bits for TP64xx rev.1B board.
	// Modify the <A> input flash address so it becomes aligned within the current FLASH PAGE in order to keep the code clear and simple.
#ifdef	__TP6X_SELECT_TP64XX__
		// set and verify the FLASH_PAGE[0..2] bits for TP64xx rev1B board
	if(TP64XX_REV1B_BOARD)
	{
			// set FPASH PAGE bits
		TP6X_set_flash_page(a/TP64XX_FLASH_PAGE_LEN);
			// verify FLASH PAGE bits
		if(TP6X_get_flash_page() != (a/TP64XX_FLASH_PAGE_LEN))
			return(_tp6x_set_err(TP6X_HW_ERR));

			// modify the FLASH address parameter to align it within the FLASH page boundary
		a %= TP64XX_FLASH_PAGE_LEN;
	}
#endif


	// Verify that FLASH contents at address A is OK to program the <D> data value, i.e. the '1' bits in the <D> word
	// match the '1' bits in the current FLASH contents at this address. Otherwise, FLASH cannot be programmed without
	// being erased.
	if ((fptr[a] & d)!=d)
		return(_tp6x_set_err(TP6X_FLASH_CANNOT_WR_ERR));

	// execute PROGRAM BYTE command
    fptr[0x555]=0xaa;
    fptr[0x2aa]=0x55;
    fptr[0x555]=0xa0;
    fptr[a]=d ;

	// verify for program ends and errors
    while(1)
	{
		rd = (fptr[a]) & TP6X_FLASH_DATA_BITMASK;

		// verify read data against the programmed data
		if(rd==d)
			// exit successfully
			return(_tp6x_set_err(TP6X_OK));

	  	// if not OK, then check state of bit D5 as a timeout/failure condition
		if((rd&0x20)!=0)
	    {
			// if timeout, then read again
	    	rd = fptr[a] & TP6X_FLASH_DATA_BITMASK;

			//verify read data against the programmed data
            if(rd==d)
				// completed successfully
				return(_tp6x_set_err(TP6X_OK));

            // set error and exit
			// execute RESET command in order to release device from the PROGRAM state
   			fptr[0] = 0xf0;
			return(_tp6x_set_err(TP6X_FLASH_WR_ERR));
	    }
	}
}




int 	TP6X_read_flash_data(unsigned a)
	// reads FLASH data at address #A
	// returns:
	//		<0 - error code detected
	//	    >=0 - FLASH read data
	//
	// Implementation Notes:
	//		TP6X_read_flash_data() function does not generally require FLASH WRITE feature enabled via the on-board
	//		jumper/switch in order to have FLASH utilities running perfectly and to get the FLASH LENGTH parameter in
	//		to verify it against input FLASH address parameter.
	//
	//		For TP64xx rev.1B boards, the FLASH LENGTH is obtained from the read-back XMEM2_LEN_ID register via
	//		FLASH CAPACITY ID field.
	//
	//		For TP62/TP67/TP620x boards and TP64xx rev.1A/1A-1/1A-2 boards without the read-back XMEM2_LEN_ID register and
	//		FLASH CAPACITY ID field, this function tries to run the _tp6x_flash_open() local function in order to have it
	//		to read the FLASH CHIP ID and setup the FLASH length local var. If it completes successfully, then input FLASH
	//		ADDRESS parameter is verified against the obtained FLASH LENGTH var, otherwise input FLASH ADDRESS parameter is
	//		verified against maximum allowed FLASH length.
{
	__TP6X_FLASH_DATA_TYPE__ *fptr;


	// set FLASH pointer and read data
	fptr = (__TP6X_FLASH_DATA_TYPE__ *)TP6X_FLASH_SADDR;

	// Verify FLASH ADDRESS value using FLASH CAPACITY ID and set the FLASH_PAGE[0..2] bits for TP64xx rev.1B board with
	// embedded FLASH memory and FLASH CAPACITY ID available.
	// Modify the <A> input flash address so it becomes aligned within the current FLASH PAGE in order to keep the code clear and simple.
#ifdef	__TP6X_SELECT_TP64XX__
	if(TP64XX_REV1B_BOARD)
	{
		// verify the FLASH ADDRESS value using FLASH CAPACITY ID
		if(			TP6X_FLASH_IS_NOT_INSTALLED
				||	a>=TP6X_get_flash_len()
		  )
			return(_tp6x_set_err(TP6X_PARAM_ERR));

			// set FPASH PAGE bits
		TP6X_set_flash_page(a/TP64XX_FLASH_PAGE_LEN);
			// verify FLASH PAGE bits
		if(TP6X_get_flash_page() != (a/TP64XX_FLASH_PAGE_LEN))
			return(_tp6x_set_err(TP6X_HW_ERR));

			// modify the FLASH address parameter to align it within the FLASH page boundary
		a %= TP64XX_FLASH_PAGE_LEN;
	}
	else
#endif
	// this code executes for both TP6x boards and TP64xx boards rev.1A/1A-1/1A-2 with plug-in FLASH memory
	{
		// try to execute FLASH OPEN procedure in order to activate FLASH utilities and read the FLASH length
		// using the FLASH chip ID, however ignore returned error code
		_tp6x_flash_open();

		// validate FLASH address upon the FLASH length
		// Use actual FLASH chip length if FLASH OPEN completed successfully and use maximum allowed FLASH length
		if(    	(_tp6x_flash_open_flag==TP6X_ON  && a>=_tp6x_flash_len)
	    	||	(_tp6x_flash_open_flag==TP6X_OFF && a>=(TP6X_FLASH_LEN/TP6X_FLASH_ADDR_STEP))
	   	  )
			return(_tp6x_set_err(TP6X_PARAM_ERR));
	}

	// FLASH address verification completed successfully
	// set last_error code to OK and read/return the FLASH data
	_tp6x_last_err=TP6X_OK;

	// read and return the FLASH data
	return(fptr[a] & TP6X_FLASH_DATA_BITMASK);
}

//======================================================================
#endif
//*************************************************************************

#endif
